
/*
 * LibSylph -- E-Mail client library
 * Copyright (C) 1999-2006 Hiroyuki Yamamoto
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <glib.h>
#include <stdio.h>
#include <string.h>
#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "defs.h"

#include <glib.h>
#include <glib/gprintf.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <dirent.h>
#include <unistd.h>
#include <ctype.h>
#include <time.h>
#if HAVE_ICONV
#  include <iconv.h>
#endif


#include "utils.h"
#include "imap.h"
#include "socket.h"
#include "base64.h"
#include "md5_hmac.h"
#include "common.h"
#include "procmsg.h"
#include "procheader.h"
#include "folder.h"

#include "prefs_account.h"
#include "account.h"
#include "procmime.h"

#include "callbacks.h"
#include "recv.h"
#include "session.h"
#include "engine_errno.h"
#include "smime_security.h"
#include "email_gconf.h"
#include "notify.h"

#ifdef CL_USE
#include "cl.h"
extern EmailMgrCallback *email_mgr_cb;	/* Defined in manager.c file */
#endif

#define QUOTE_IF_REQUIRED(out, str)				\
{								\
	if (*str != '"' && strpbrk(str, " \t(){}%*") != NULL) {	\
		gchar *__tmp;					\
		gint len;					\
								\
		len = strlen(str) + 3;				\
		Xalloca(__tmp, len, return IMAP_ERROR);		\
		g_snprintf(__tmp, len, "\"%s\"", str);		\
		out = __tmp;					\
	} else {						\
		Xstrdup_a(out, str, return IMAP_ERROR);		\
	}							\
}

#define PARSE_ONE_ELEMENT(ch)					\
{								\
	cur_pos = strchr_cpy(cur_pos, ch, buf, sizeof(buf));	\
	if (cur_pos == NULL) {					\
		log_message("cur_pos == NULL\n");			\
		procmsg_msginfo_free(msginfo);			\
		return NULL;					\
	}							\
}
#define FREE_BUF_AND_RETURN_ERROR_IF_FAIL(cond)	                \
if (!(cond)) {			                                \
    g_free(buf);		                                \
    return IMAP_ERROR;	                                        \
}
/* all static globals for this file */
Imap4Callback *gl_imap4_callback_ptr;
//static GSList *gl_mimeinfo_list = NULL;
static guint32 gl_latest_uid = 0;
static guint32 gl_msgs_exists_server = 0;
static gboolean gl_imap_recv_cancel = FALSE;
static gboolean gl_cancel_flag = FALSE;
static gboolean gl_stop_timer = FALSE;
static PrefsAccount *gl_current_account = NULL;
static GSList *gl_imap_delete_list = NULL;
static gboolean not_mandatory_to_conn_serv = FALSE;

/* externals */
extern gboolean gl_callback_enable_flag;
/* defined in utils.c */
extern gint engine_errno;
/* defined in utils.c */

#define IMAP_CB(ret)     \
 if(gl_imap4_callback_ptr && \
               gl_imap4_callback_ptr->imap4_param_value_missing_cb \
               && CHECK_CB_FLAG  ){ \
         gl_imap4_callback_ptr-> imap4_param_value_missing_cb(ret); \
      }

#define IMAP_DISCONNECT(session,error) \
    if(gl_imap4_callback_ptr && \
               gl_imap4_callback_ptr->imap4_param_value_missing_cb \
               && CHECK_CB_FLAG  ){ \
         gl_imap4_callback_ptr-> imap4_param_value_missing_cb(error); \
    }\
    imap_session_disconnect (session, FALSE);

#define IMAP_PROGRESS_CB(cur,total)     \
 if(gl_imap4_callback_ptr && \
               gl_imap4_callback_ptr->imap4_set_msg_cb \
               && CHECK_CB_FLAG  ){ \
	osso_email_gconf_set_int(EMAIL_RECV_MAILS_PROGRESS_TOTAL, total); \
	osso_email_gconf_set_int(EMAIL_RECV_MAILS_PROGRESS_CUR, cur);\
      }
#define IMAP_SET_MSG_CB(error,param)                                \
 if(IMAP_HAS_CB){                                                   \
      if(gl_imap4_callback_ptr->imap4_set_msg_cb ){                 \
         gl_imap4_callback_ptr->imap4_set_msg_cb(error,"%s",param); \
      }                                                             \
 }

#define IMAP_HAS_CB \
 ( (gl_imap4_callback_ptr != NULL ) && (CHECK_CB_FLAG ==TRUE) )

#define IMAP_SET_ERROR_CB(error)     \
 if(IMAP_HAS_CB){                                                   \
      if(gl_imap4_callback_ptr->imap4_set_msg_cb ){                 \
         gl_imap4_callback_ptr->imap4_set_msg_cb(error,NULL);       \
      }                                                             \
 }                                                                  \

#define IMAP_SET_MSG_STR_CB(error,param1,param2)                              \
 if(IMAP_HAS_CB){                                                             \
      if(gl_imap4_callback_ptr->imap4_set_msg_cb ){                           \
         gl_imap4_callback_ptr->imap4_set_msg_cb(error,"%s%s",param1,param2); \
      }                                                                       \
 }
#define IMAP_SEND_DEL_LIST_CB(list)                                           \
if(IMAP_HAS_CB){                                                              \
      if(gl_imap4_callback_ptr->imap4_del_list_cb ){                          \
         gl_imap4_callback_ptr->imap4_del_list_cb(list);                      \
      }                                                                       \
}                                                                             \

static void msg_headers_send(GSList * new_list, Session * session,
			     PrefsAccount * prefs_account,
			     gint msg_count, gint * last_uid, gboolean * status);

static gint return_size(gchar * str);

static Session *imap_session_new(PrefsAccount * account, const gchar * msgid);

static void imap_session_destroy(Session * session);

static gint imap_greeting(IMAPSession * session);

static gint imap_cmd_namespace(IMAPSession * session, gchar ** ns_str);

static gchar *imap_cmd_body_structure(IMAPSession * session, const gchar * msgid);

static void imap_set_msginfo_fields(PrefsAccount * prefs_account, MsgInfo * msg_info);

/* changed IMAPAuthType to PasswdAuthType */
static gint imap_auth(IMAPSession * session, const gchar * user,
		      const gchar * pass, PasswdAuthType type, gboolean tmp_pass);

static gint imap_cmd_starttls(IMAPSession * session);

static GSList *imap_get_uncached_msgs(IMAPSession * session,
				      PrefsAccount * prefs_account,
				      const gchar * folder,
				      guint32 first_uid, guint32 * last_uid, gint * count);

#if USE_SSL
static SockInfo *imap_open(const gchar * server,
			   gushort port, SSLType ssl_type, const gchar * account);
#else
static SockInfo *imap_open(const gchar * server, gushort port, const gchar * account);
#endif


static gint imap_set_message_flags(IMAPSession * session,
				   const gchar * seq_set, IMAPFlags flags, gboolean is_set);

static GSList *mimeinfo_to_attachlist(MimeInfo * mimeinfo, const gchar * msgid);

static gchar *imap_get_flag_str(IMAPFlags flags);

static MsgFlags imap_parse_flags(const gchar * flag_str);

static gboolean imap_has_capability(IMAPSession * session, const gchar * capability);
static void imap_capability_free(IMAPSession * session);

/* low-level IMAP4rev1 commands */
static gint imap_cmd_capability(IMAPSession * session);

/* changed IMAPAuthType to PasswdAuthType */
static gint imap_cmd_authenticate(IMAPSession * session,
				  const gchar * user, const gchar * pass, PasswdAuthType type);

static gint imap_cmd_login(IMAPSession * session, const gchar * user, const gchar * pass);

static gint imap_cmd_logout(IMAPSession * session);

static gint imap_cmd_noop(IMAPSession * session);

static gint imap_cmd_do_select(IMAPSession * session,
			       const gchar * folder, gboolean examine,
			       gint * exists, gint * recent, gint * unseen, gint * uid_validity);

static gint imap_cmd_do_uid_select(IMAPSession * session,
				   const gchar * folder, gboolean examine,
				   gint * exists, gint * recent,
				   gint * unseen, gint * uid_validity,
				   PrefsAccount * prefs_account, gboolean send_error_cb);

static gint imap_cmd_envelope(IMAPSession * session, const gchar * seq_set);

static MsgInfo *imap_parse_env(IMAPSession * session, GString * line_str, gchar * header_path);

static gint imap_cmd_fetch(IMAPSession * session, guint32 uid, const gchar * filename);

static gint imap_cmd_fetch_attach(IMAPSession * session, guint32 uid,
				  const gchar * filename, const gchar * section_number);

static gint imap_cmd_fetch_header(IMAPSession * session, guint32 uid, const gchar * filename);

static gint imap_cmd_store(IMAPSession * session, const gchar * seq_set, const gchar * sub_cmd);

static gint imap_cmd_close(IMAPSession * session);

static gint imap_cmd_ok(IMAPSession * session, GPtrArray * argbuf);


static void imap_cmd_gen_send(IMAPSession * session, const gchar * format, ...);

static gint imap_cmd_gen_recv(IMAPSession * session, gchar ** ret);

/* misc utility functions */
static gchar *strchr_cpy(const gchar * src, gchar ch, gchar * dest, gint len);

static gchar *search_array_contain_str(GPtrArray * array, gchar * str);

static gchar *search_array_str(GPtrArray * array, gchar * str);

static gint imap_cmd_mail_size(Session * session, gint msgnum);

static gint imap_cmd_get_body_size(Session * session, gchar * msgid, gchar * sec_nu);

static gint imap_handle_success(MsgInfo * msg_info);

static gint imap_get_body(PrefsAccount * prefs_account, Session * session,
			  gint uid_no, const gchar * header_file_name,
			  const gchar * body_file_name, MsgHeader * header,
			  const gchar * file_name, MimeInfo * mimeinfo);
static gint
imap_msg_size_limit(PrefsAccount * prefs_account, Session * session,
		    gint uid_no, const gchar * header_file_name,
		    const gchar * body_file_name, MsgHeader * header,
		    const gchar * file_name, MimeInfo * mimeinfo, gboolean smime_check);

static int get_num_len(const guint32 num);

static gint download_imap_body(Session * session,
			       PrefsAccount * prefs_account,
			       const gchar * header_file_name,
			       const gchar * body_file_name,
			       MsgHeader * header, const gchar * sec_nu,
			       gint uid_no, const gchar * file_name, MimeInfo * mimeinfo);

static gint imap_file_append_attach_content(FILE * fp,
					    const gchar * sub_file_name, gboolean first_flag);

static gint
imap_write_mime_version(IMAPSession * imap_session, FILE * outfp,
			MimeInfo * mimeinfo, const gchar * msgid, PrefsAccount * prefs_account);
static gint imap_get_multipart_body(IMAPSession * imap_session,
				    MimeInfo * mimeinfo,
				    const gchar * msgid, PrefsAccount * account);

static gint imap_get_msg_for_hdr(MsgInfo * msg_info, Session * session,
				 PrefsAccount * prefs_account,
				 const gchar * header_file_name,
				 const gchar * body_file_name,
				 const gchar * file_name, MimeInfo * mimeinfo);

static gint imap_download_msg(MsgInfo * msg_info, Session * session,
			      PrefsAccount * prefs_account, MimeInfo * mimeinfo);
static gboolean imap_is_msg_exist(const gchar * msgid);

static gint imap_get_inline_part(IMAPSession * imap_session,
				 MimeInfo * mimeinfo, FILE * fp,
				 const gchar * msgid, PrefsAccount * prefs_account);

static gint imap_get_smime_type(MsgInfo * msg_info);

/**
  This function creates the session and returns.If the account 
  has retained session,that will be returned. 
  @param prefs_account   pointer to the account information. 
  @return pointer to the session created 
*/
static Session *imap_get_op_session(PrefsAccount * prefs_account);

/**
  This function sends the callback to UI.This is called on timeout 
  @param data   pointer to the progress data. 
  @return FALSE if the timer has to be stopped,else TRUE 
*/
static gboolean progress_func(gpointer data);

/**
  This function deletes messages from IMAP server once they are moved from
  Inbox folder to some other folder

  @param msgid - ID of message to be removed from server
  @param session - Session pointer
  @return success(IMAP_SUCCESS) or failure(IMAP_FAIL) value
*/
static gint imap_delete_moved_msg(const gchar * msgid, Session * session);

/**
 This function the part of message.
 This checks for the size of part before downloading
 @param imap_session    pointer to the session
 @param prefs_account    pointer to the account information
 @param msgid   message id of the message
 @param mimeinfo  mimeinformation of the part 
 @return IMAP_FAIL if the contents could not be downloaded.
 IMAP_SUCCESS on success 
*/
static gint imap_get_part(IMAPSession * imap_session,
			  PrefsAccount * prefs_account, const gchar * msgid, MimeInfo * mimeinfo);

/**
 This function downloads the multipart body of a complete message,
 which can be displayed in a browser
 @param prefs_account    pointer to the account information
 @param imap_session    pointer to the session
 @param msgid   message id of the message
 @return IMAP_FAIL if the contents could not be downloaded.
 IMAP_SUCCESS on success 
*/
static gint imap_get_full_body(PrefsAccount * prefs_account,
			       IMAPSession * imap_session, const gchar * msgid);
/**
  This function fetches imap message for folder move/copy operation 
  @param session imap session  
  @param msgid    message id of the message  
  @param prefs_account    account information of the messages  
  @param move_flag    indicates whether the operation is move 
  @param srcfolder    source folder for the operation  
  @param filename    name of the file  
  @param local_copy    indicates whether the file has local copy  
  @param total    total number of messages to be fetched  
  @return IMAP_OK or imap error code 
*/
static gint
imap_cmd_fetch_foldermsg(Session * session, const gchar * msgid,
			 PrefsAccount * prefs_account,
			 gboolean move_flag, const gchar * srcfolder,
			 const gchar * filename, gboolean local_copy, gint total);
/**
  This function fetches imap flags for a message 
  @param session imap session  
  @param uid    uid of the message  
  @param imap_flag_ptr    pointer to the imap flag  
  @return IMAP_OK or imap error code 
*/
static gint imap_cmd_flags(IMAPSession * session, gint uid, MsgFlags * imap_flag_ptr);

/**
  This function checks the error received by folder operation and 
  imap operation and returns the effective error. 
  @param tmpret_val   error occurred during IMAP operation  
  @param ret_val    error occurred during folder operation   
  @param imap_ret    (OUT) this has the error to take into account   
  @param update_list_ptr    (OUT) this is pointer to the list messages 
  that have to be updated on successful folder operation.   
  @param folder_delete_list    (OUT) this is pointer to the list messages 
  that have to be deleted on folder operation.   
  @param prefs_account    pointer to account information  
  @param msgid    message id of the file  
  @return none 
*/
static void imap_get_op_error(gint tmpret_val, gint ret_val,
			      gint * imap_ret, GSList ** update_list_ptr,
			      GSList ** folder_delete_list,
			      PrefsAccount * prefs_account, const gchar * msgid);
/**
  This function searches for the messages that can be downloaded 
  from server
  @param session imap session  
  @param first_uid_ptr (OUT)pointer to uid of the first message 
  in the message sequence  
  @param last_uid_ptr (OUT)pointer to uid of the last message 
  in the message sequence  
  @param prefs_account    pointer to account information 
  @param update_first_ptr (OUT)ndicates whther the first uid index 
  in account has information has to be updated 
  @param msg_last_uid    the last uid available in server 
  @param new_msg_exist    indicates that there are new messages 
  to be fetched  
  @return IMAP_SUCCESS or IMAP_FAIL(-1) 
*/
static gint
imap_cmd_search(IMAPSession * session, gint * first_uid_ptr,
		gint * last_uid_ptr,
		PrefsAccount * prefs_account,
		gboolean * update_first_ptr, gint msg_last_uid, gboolean new_msg_exist);

/**
  This function checks if body is downloaded 
  from server
  @param session    imap session  
  @param  mimeinfo    mimeinformation of the message  
  @param  msgid    message id of the message  
  @param  prefs_account    account information of the message  
  @param  uid_no    uid number of the message  
  @param  section_number    section number of part  
  @param  body_file_name    name of the file to save the part  
  @return IMAP_SUCCESS or IMAP_FAIL 
*/
static gint imap_download_body_check(Session * session,
				     MimeInfo * mimeinfo,
				     const gchar * msgid,
				     PrefsAccount * prefs_account,
				     gint uid_no,
				     const gchar * section_number, const gchar * body_filename);

/**
  This function reads mimeinformation from the file 
  @param  mimeinfo    mimeinformation of the message  
  @param  msgid    message id of the message  
  @param  file_path    name of the file to read  
  @param  session    imap session used  
  @return  mimeinfo    mimeinformation read from file,NULL on error  
*/
static MimeInfo *imap_scan_mimeinfo(const gchar * msgid, const gchar * filepath, Session * session);

/**
  This function checks the last uid and first uid that can be 
  actually downloaded.Checks the actual count taking into account 
  deleted messages  
  @param  uid_array    array of list of undeleted uid numbers on server  
  @param  first_uid_ptr    pointer to the first uid to use (OUT param)  
  @param  last_uid_ptr    pointer to the last uid to use(OUT param)  
  @param  prefs_account    pointer to the account  
  @return  total number of messages to receive,0 on error    
*/
static gint imap_get_msg_count(GArray * uid_array,
			       gint * first_uid_ptr,
			       gint * last_uid_ptr, PrefsAccount * prefs_account);

/**
  This function gets the last uid and first uid that can be 
  actually downloaded.
  @param  tmp_first_ptr    pointer to the first uid to use (OUT param)  
  @param  tmp_last_ptr    pointer to the last uid to use(OUT param)  
  @param  update_last_msgid    pointer to the flag which indicates 
                         whether to update the uid in account(OUT param)  
  @param  prefs_account    pointer to the account information 
  @param  session    pointer to the session information 
  @param  msg_last_uid    (OUT) pointer to the last uid on the server 
  @return  IMAP_SUCCESS on succes or IMAP_FAIL on error.    
*/
static gint imap_get_range(PrefsAccount * prefs_account,
			   gint * tmp_last_ptr, gint * tmp_first_ptr,
			   gboolean * update_last_msgid, Session * session, gint * msg_last_uid);

/**
  This function checks if the header saved in local folder is 
  deleted on server.If the message is deleted,deletes the local header. 
  @param  uid_array    pointer to the array of undeleted messages  
  @param  msgid    pointer to the message id  
  @param  remove_msg_list    pointer to the list of 
          messages removed(OUT parameter) 
  @param  last_srch_num    (OUT) pointer to the last numbers searched 
  @return  TRUE if the message is deleted.FALSE otherwise   
*/
static gboolean imap_is_header_deleted(GArray * uid_array,
				       const gchar * msgid,
				       GSList ** remove_msg_list, gint * last_srch_num);

/**
  This function checks whether any of the headers saved in local folder,  
  are deleted on server.If any message is deleted,deletes the local 
  header for that message. 
  @param  uid_array    pointer to the array of undeleted messages  
  @param  account_name    account name of the account  
  @return  none   
*/
static void imap_check_deleted_msgs(GArray * uid_array, const gchar * account_name);

/**
  This function checks deletes the headers files saved in local folder,  
  @param  c_list    list of message headers to be deleted 
  @return  IMAP_SUCCESS if successfully deleted IMAP_FAIL otherwise   
*/
static gint imap_remove_list_msgs(GSList * c_list);

/**
  This function fetches the mimeinformation for the message   
  @param  msg_info    pointer to the message infortmation 
  @param  session    pointer to the session infortmation 
  @param  prefs_account    pointer to the account infortmation 
  @return  mimeinformation if successful,NULL otherwise   
*/
static MimeInfo *imap_get_msg_mimeinfo(MsgInfo * msg_info,
				       Session * session, PrefsAccount * prefs_account);

/**
  This function will be called to register the callbacks.
  @param cb_ptr    pointer to the callback
  @return TRUE if successfully registered.FALSE otherwise
 */
gboolean imap4_init(Imap4Callback * cb_ptr)
{
	if (cb_ptr != NULL) {
		gl_imap4_callback_ptr = cb_ptr;
		return TRUE;
	} else {
		gl_imap4_callback_ptr = NULL;
		return FALSE;
	}
}

/**
  This function handles the greeting response from the server
  @param session    pointer to the session
  @return success(IMAP_SUCCESS)/error(IMAP_ERROR) code
*/
static gint imap_greeting(IMAPSession * session)
{
	gchar *greeting = NULL;
	gint ok = IMAP_SUCCESS;

	if ((ok = imap_cmd_gen_recv(session, &greeting)) != IMAP_SUCCESS) {
		return ok;
	}

	if (greeting[0] != '*' || greeting[1] != ' ') {
		ok = IMAP_ERROR;
	} else if (!strncmp(greeting + 2, "OK", 2)) {
		ok = IMAP_SUCCESS;
	} else if (!strncmp(greeting + 2, "PREAUTH", 7)) {
		session->authenticated = TRUE;
		ok = IMAP_SUCCESS;
	} else {
		ok = IMAP_ERROR;
	}
	g_free(greeting);
	return ok;
}

/**
  This function authenticates the user 
  @param session    pointer to the session
  @param user    userid of the user account
  @param pass    password of the user account
  @param type   authentication type(login or CRAM-MD5)
  @param tmp_pass   temporary passwd to be used for authentication 
  @return success(IMAP_SUCCESS)/IMAP error code
*/
static gint
imap_auth(IMAPSession * session, const gchar * user,
	  const gchar * pass, PasswdAuthType type, gboolean tmp_pass)
{
	gint ok = 0;
	gint ui_closed = FALSE;

	CHECK_RETURN_VAL_IF_FAIL(session != NULL, IMAP_PARAMETER_MISSING);

	if (user == NULL) {
		IMAP_CB(USERID_MISSING);
		return IMAP_PARAMETER_MISSING;
	}
	if (pass == NULL) {
		ok = IMAP_PASSWD_FAIL;
	}
	if ((type != AUTH_LOGIN) && (type != AUTH_CRAM_MD5)) {
		IMAP_CB(PASSWD_AUTH_TYPE_MISSING);
		return IMAP_PARAMETER_MISSING;
	}
	if (pass != NULL) {
		if (type == AUTH_LOGIN) {
			ok = imap_cmd_login(session, user, pass);
		} else {
			ok = imap_cmd_authenticate(session, user, pass, type);
		}
		if( get_engine_errno() == ENGINE_SOCKET_TIME_OUT) {
			return IMAP_ERROR;
		}
		/* Informing upper layers that connection error */
		if(ok == IMAP_SOCKET) {
			return ok;
		}
		if ((ok != IMAP_SUCCESS) && (tmp_pass == FALSE)) {
			return IMAP_AUTHFAIL;
		}
	}
	/* check for temporary password */
	if (ok == IMAP_PASSWD_FAIL) {

		if (session->ac_prefs->user_settings.scheduled_op == TRUE) {
			return ok;
		}
		g_free(session->ac_prefs->user_settings.tmp_passwd);
		session->ac_prefs->user_settings.tmp_passwd = NULL;
		session->ac_prefs->user_settings.cancel = FALSE;
		if (tmp_pass == TRUE) {
			/* Always send CB */
			if (gl_imap4_callback_ptr->imap4_set_msg_cb) {
				gl_imap4_callback_ptr->
				    imap4_set_msg_cb(IMAP_PASSWD_INCORRECT, "%s",
						     session->ac_prefs->user_settings.account_name);
			}

		} else {

			/* Always send CB */
			if (gl_imap4_callback_ptr->imap4_set_msg_cb) {
				gl_imap4_callback_ptr->imap4_set_msg_cb(IMAP_PASSWD_FAIL,
									"%s",
									session->ac_prefs->
									user_settings.account_name);
			}
		}

		while (1) {
			(void) g_main_context_iteration(NULL, FALSE);

			if (session->ac_prefs->user_settings.tmp_passwd != NULL) {
				break;
			}
			if (session->ac_prefs->user_settings.cancel == TRUE) {
				break;
			}
			if(osso_email_gconf_get_int(EMAIL_GCONF_UI_CLOSED,&ui_closed)) {
				if(ui_closed) {
					break;
				}
			} else if (imap_is_session_cancelled()) {
				break;
			}
		}
		if (session->ac_prefs->user_settings.tmp_passwd != NULL) {

			ok = imap_auth(session, user,
				       session->ac_prefs->user_settings.tmp_passwd, type, TRUE);
			g_free(session->ac_prefs->user_settings.tmp_passwd);
			session->ac_prefs->user_settings.tmp_passwd = NULL;
		}

	}
	if (ok == IMAP_SUCCESS) {
		session->authenticated = TRUE;
	}
	return ok;
}

#define THROW(err) { ok = err; goto catch; }

/**
  This function fetches the name space response from the server 
  @param session    pointer to the session
  @param ns_str    OUT parameter,pointer to response to 
  imap name space command
  @return success(IMAP_SUCCESS)/IMAP error code
*/
static gint imap_cmd_namespace(IMAPSession * session, gchar ** ns_str)
{
	gint ok = 0;
	GPtrArray *argbuf = NULL;
	gchar *str = NULL;

	argbuf = g_ptr_array_new();

	imap_cmd_gen_send(session, "NAMESPACE");
	if ((ok = imap_cmd_ok(session, argbuf)) != IMAP_SUCCESS) {
		THROW(ok);
	}

	str = search_array_str(argbuf, "NAMESPACE");
	if (!str) {
		THROW(IMAP_ERROR);
	}

	*ns_str = g_strdup(str);

      catch:
	ptr_array_free_strings(argbuf);
	g_ptr_array_free(argbuf, TRUE);

	return ok;
}

#undef THROW

/**
  This function destroys & frees the user session 
  @param session    pointer to the session
  @return none
*/
static void imap_session_destroy(Session * session)
{
	imap_capability_free(IMAP_SESSION(session));
	g_free(IMAP_SESSION(session)->mbox);
}

/**
  This function opens socket with the server 
  @param server    IP address or name of server,to open the socket 
  @param port    port number
  @param ssl_type    type of ssl connection to be used,if SSL is selected
  @param account    account to be used 
  @return pointer to opened socket 
*/

#if USE_SSL
static SockInfo *imap_open(const gchar * server, gushort port, SSLType ssl_type,
			   const gchar * account)
#else
static SockInfo *imap_open(const gchar * server, gushort port, const gchar * account)
#endif
{
	SockInfo *sock = NULL;
	sock_imap_cancel(TRUE, 0);	/*  for cancel support */
	if ((sock = sock_connect(server, port)) == NULL) {
		log_message("Can't connect to IMAP4 server: %s:%d\n", server, port);
		IMAP_SET_MSG_STR_CB(IMAP_CONNECT_SERVER_ERROR, server, account);
		IMAP_SET_MSG_CB(IMAP_COMPLETE, account);
		if (gl_cancel_flag == TRUE) {
			gl_cancel_flag = FALSE;
			SET_CB_FLAG;
		}
		return NULL;
	}
#if USE_SSL
	if (ssl_type == SSL_TUNNEL) {
		gint ret = 0;
		ret = ssl_init_socket(sock);
		if (ret != SSL_SUCCESS) {
			log_message("Can't establish IMAP4 session with: %s:%u\n", server, port);
			sock_close(sock);
			IMAP_CB(ret);
			IMAP_SET_MSG_CB(IMAP_COMPLETE, account);
			return NULL;
		}
	}
#endif

	return sock;
}

/**
  This function fetches the headers information  
  @param session    pointer to the session
  @param cur_pos    curser position
  @param headers    OUT parameter,header information.
  This has to be freed by the calling function
  @param str    line string
  @return string
*/
static gchar *imap_get_header(IMAPSession * session, gchar * cur_pos,
			      gchar ** headers, GString * str)
{
	gchar *nextline = NULL;
	gchar buf[IMAP_HDRCMD_SIZE];
	gint len = 0;
	gint block_len = 0;

	memset(buf, '\0', IMAP_HDRCMD_SIZE);

	*headers = NULL;

	CHECK_RETURN_VAL_IF_FAIL(str != NULL, cur_pos);

	while (isspace(*cur_pos)) {
		cur_pos++;
	}
	CHECK_RETURN_VAL_IF_FAIL(*cur_pos == '{', cur_pos);

	cur_pos = strchr_cpy(cur_pos + 1, '}', buf, sizeof(buf));
	len = atoi(buf);
	CHECK_RETURN_VAL_IF_FAIL(len >= 0, cur_pos);

	g_string_truncate(str, 0);
	cur_pos = str->str;

	do {
		if ((nextline = sock_getline(SESSION(session)->sock)) == NULL) {
			return cur_pos;
		}
		block_len += strlen(nextline);
		g_string_append(str, nextline);
		cur_pos = str->str;
		strretchomp(nextline);
		g_free(nextline);
	} while (block_len < len);

	debug_print("IMAP4< [contents of RFC822.HEADER]\n");

	*headers = g_strndup(cur_pos, len);
	cur_pos += len;

	while (isspace(*cur_pos)) {
		cur_pos++;
	}
	while (*cur_pos == '\0') {
		if ((nextline = sock_getline(SESSION(session)->sock)) == NULL) {
			return cur_pos;
		}
		g_string_assign(str, nextline);
		cur_pos = str->str;
		strretchomp(nextline);
		g_free(nextline);

		while (isspace(*cur_pos)) {
			cur_pos++;
		}
	}

	return cur_pos;
}

/**
  This function parses flag string 
  @param flag_str    flag string
  @return message flags,with the flags set.
  This should not be freed by the calling function.
*/
static MsgFlags imap_parse_flags(const gchar * flag_str)
{
	const gchar *p = flag_str;
	MsgFlags flags = { 0, 0 };

	flags.perm_flags = MSG_UNREAD;

	while ((p = strchr(p, '\\')) != NULL) {
		p++;

		if (g_strncasecmp(p, "Recent", 6) == 0 && MSG_IS_UNREAD(flags)) {
			MSG_SET_PERM_FLAGS(flags, MSG_NEW);
		} else if (g_strncasecmp(p, "Seen", 4) == 0) {
			MSG_UNSET_PERM_FLAGS(flags, MSG_NEW | MSG_UNREAD);
		} else if (g_strncasecmp(p, "Deleted", 7) == 0) {
			MSG_SET_PERM_FLAGS(flags, MSG_DELETED);
		} else if (g_strncasecmp(p, "Flagged", 7) == 0) {
			MSG_SET_PERM_FLAGS(flags, MSG_MARKED);
		} else if (g_strncasecmp(p, "Answered", 8) == 0) {
			MSG_SET_PERM_FLAGS(flags, MSG_REPLIED);
		}
	}

	return flags;
}

/**
  This function prepares flag string depending on the flags
  @param flags    IMAPFlags structure
  @return flag string.This has to be freed by the calling function
*/

static gchar *imap_get_flag_str(IMAPFlags flags)
{
	GString *str = NULL;
	gchar *ret = NULL;

	str = g_string_new(NULL);

	if (IMAP_IS_SEEN(flags)) {
		g_string_append(str, "\\Seen ");
	}
	if (IMAP_IS_ANSWERED(flags)) {
		g_string_append(str, "\\Answered ");
	}
	if (IMAP_IS_FLAGGED(flags)) {
		g_string_append(str, "\\Flagged ");
	}
	if (IMAP_IS_DELETED(flags)) {
		g_string_append(str, "\\Deleted ");
	}
	if (IMAP_IS_DRAFT(flags)) {
		g_string_append(str, "\\Draft");
	}
	if (str->len > 0 && str->str[str->len - 1] == ' ') {
		g_string_truncate(str, str->len - 1);
	}

	ret = str->str;
	g_string_free(str, FALSE);

	return ret;
}

/**
  This function sets flags for the set of messages in the seq_set 
  @param session    session pointer
  @param seq_set    set of message uids for executing the command
  @param flags    IMAPFlags 
  @param is_set indicates whether flags have to be made silent 
  @return IMAP_SUCCESS,on successfully sending the command 
*/
static gint
imap_set_message_flags(IMAPSession * session,
		       const gchar * seq_set, IMAPFlags flags, gboolean is_set)
{
	gchar *cmd = NULL;
	gchar *flag_str = NULL;
	gint ok = 0;

	flag_str = imap_get_flag_str(flags);
	cmd = g_strconcat(is_set ? "+FLAGS.SILENT (" : "-FLAGS.SILENT (", flag_str, ")", NULL);
	g_free(flag_str);

	ok = imap_cmd_store(session, seq_set, cmd);
	g_free(cmd);

	return ok;
}

/** 
  This function checks whether imap server has cretain capability
  @param session    pointer to the IMAPSession object created 
  @param capability    pointer to the capability string to be checked
  @return	true on having / false on not having
*/
static gboolean imap_has_capability(IMAPSession * session, const gchar * capability)
{
	gchar **p = NULL;

	for (p = session->capability; *p != NULL; ++p) {
		if (!g_strcasecmp(*p, capability)) {
			return TRUE;
		}
	}

	return FALSE;
}

/** 
  This function frees the capability member pointer in the IMAPsession object
  @param pointer to the IMAPSession object created 
  @return none
*/


static void imap_capability_free(IMAPSession * session)
{
	g_strfreev(session->capability);
	session->capability = NULL;
}


/* low-level IMAP4rev1 commands */

#define THROW(err) { ok = err; goto catch; }
/**
  This function fetches servers capability
  @param session    pointer to the session
  @return success(IMAP_SUCCESS)/error code
*/
static gint imap_cmd_capability(IMAPSession * session)
{
	gint ok = 0;
	GPtrArray *argbuf = NULL;
	gchar *capability = NULL;

	argbuf = g_ptr_array_new();

	/* send the command capability through the imap_cmd_gen_send */
	imap_cmd_gen_send(session, "CAPABILITY");

	/* collect the response for the capability in the argbuf through 
	   imap_cmd_ok */
	if ((ok = imap_cmd_ok(session, argbuf)) != IMAP_SUCCESS) {
		THROW(ok);
	}

	/* search for the string CAPBILITY  in the reponse */
	capability = search_array_str(argbuf, "CAPABILITY ");
	if (!capability) {
		THROW(IMAP_ERROR);
	}

	capability += strlen("CAPABILITY ");
	IMAP_SESSION(session)->capability = g_strsplit(capability, " ", IMAP_FAIL);

      catch:
	ptr_array_free_strings(argbuf);
	g_ptr_array_free(argbuf, TRUE);

	return ok;
}

#undef THROW
/** 
  This function sends the authenticate imap command  to the imap server 
  @param session    pointer to the IMAPSession object created 
  @param user    pointer to gchar userid of the mail account
  @param pass    pointer to gchar password of the mail account for the 
  corresponding the  userid
  @param type     PasswdAuthType indicates type of authentication use
  (LOGIN or CRAM_MD5) 
  @return  success(IMAP_SUCCESS)/ error code
*/
static gint
imap_cmd_authenticate(IMAPSession * session, const gchar * user,
		      const gchar * pass, PasswdAuthType type)
{
	gchar *auth_type = NULL;
	gint ok = 0;
	gchar *buf = NULL;
	gchar *challenge = NULL;
	gint challenge_len = 0;
	gchar hexdigest[HEXDIGEST_SIZE];
	gchar *response = NULL;
	gchar *response64 = NULL;

	memset(hexdigest, '\0', HEXDIGEST_SIZE);

	CHECK_RETURN_VAL_IF_FAIL(type == AUTH_CRAM_MD5, IMAP_ERROR);

	auth_type = "CRAM-MD5";

	imap_cmd_gen_send(session, "AUTHENTICATE %s", auth_type);
	ok = imap_cmd_gen_recv(session, &buf);
	if (ok != IMAP_SUCCESS || buf[0] != '+' || buf[1] != ' ') {
		g_free(buf);
		return IMAP_ERROR;
	}

	challenge = g_malloc(strlen(buf + 2) + 1);
	if (challenge == NULL) {
		return IMAP_FAIL;
	}
	challenge_len = base64_decode((guchar *) challenge, buf + 2, IMAP_FAIL);
	challenge[challenge_len] = '\0';
	g_free(buf);
	log_message("IMAP< [Decoded: %s]\n", challenge);

	md5_hex_hmac(hexdigest, (const unsigned char *) challenge,
		     challenge_len, (const unsigned char *) pass, strlen(pass));

	g_free(challenge);

	response = g_strdup_printf("%s %s", user, hexdigest);
	log_message("IMAP> [Encoded: %s]\n", response);
	response64 = g_malloc((strlen(response) + 3) * 2 + 1);
	if (response64 == NULL) {
		g_free(response);
		return IMAP_FAIL;
	}
	base64_encode(response64, (const guchar *) response, strlen(response));
	g_free(response);

	log_message("IMAP> %s\n", response64);
	sock_puts(SESSION(session)->sock, response64);
	ok = imap_cmd_ok(session, NULL);
	if (ok != IMAP_SUCCESS) {
		log_message("IMAP4 authentication failed.\n");
		return IMAP_PASSWD_FAIL;
	}
	return ok;
}

/** 
  This function sends the login imap command  to the imap server 
  @param session    pointer to the IMAPSession  
  @param user    pointer to gchar userid of the mail account
  @param pass    pointer to gchar password of the mail account for the 
  corresponding the  userid
  @return  (IMAP_SUCCESS)success/ error code
*/
static gint imap_cmd_login(IMAPSession * session, const gchar * user, const gchar * pass)
{
	gchar *user_ = NULL, *pass_ = NULL;
	gint ok = 0;
	CHECK_RETURN_VAL_IF_FAIL(session != NULL, IMAP_PARAMETER_MISSING);
	CHECK_RETURN_VAL_IF_FAIL(user != NULL, IMAP_PARAMETER_MISSING);
	CHECK_RETURN_VAL_IF_FAIL(pass != NULL, IMAP_PASSWD_FAIL);

	QUOTE_IF_REQUIRED(user_, user);
	QUOTE_IF_REQUIRED(pass_, pass);
	imap_cmd_gen_send(session, "LOGIN %s %s", user_, pass_);

	ok = imap_cmd_ok(session, NULL);
	if(ok == IMAP_SOCKET){
		log_message("IMAP4 socket error.\n");
		return ok;
	}
	if (ok != IMAP_SUCCESS ) {
		log_message("IMAP4 login failed(%d).\n", ok);
		return IMAP_PASSWD_FAIL;
	} 

	return ok;
}

/** 
  This function sends the logout imap command  to the imap server 
  @param session    pointer to the IMAPSession object created 
  @return  (IMAP_SUCCESS) success/ error code
*/

static gint imap_cmd_logout(IMAPSession * session)
{
	imap_cmd_gen_send(session, "LOGOUT");
	return imap_cmd_ok(session, NULL);
}

/** 
  This function sends the NOOP imap command  to the imap server 
  @param session    pointer to the IMAPSession object created 
  @return  (IMAP_SUCCESS) success/ error code
*/
static gint imap_cmd_noop(IMAPSession * session)
{
	imap_cmd_gen_send(session, "NOOP");
	return imap_cmd_ok(session, NULL);
}

/** 
  This function sends the tls imap command  to the imap server,
  to start TLS session 
  @param session    pointer to the IMAPSession object created 
  @return  (IMAP_SUCCESS)success/ error code
*/
static gint imap_cmd_starttls(IMAPSession * session)
{
	imap_cmd_gen_send(session, "STARTTLS");
	return imap_cmd_ok(session, NULL);
}

#define THROW goto catch

/** 
  This function sends the select imap command  to the imap server 
  @param session    IMAP session information 
  @param folder    name of the folder
  @param examine    flag to select the specified folder in read-write 
  or read only mode
  @param exists    pointer to the integer get existing messages in the folder
  @param recent    pointer to the integer get recent messages in the folder
  @param unseen    pointer to the integer get unseen messages in the folder
  @param uid_validity    pointer to the integer get uid_validity in the folder
  @return (IMAP_SUCCESS) success or error code
*/
static gint
imap_cmd_do_select(IMAPSession * session, const gchar * folder,
		   gboolean examine,
		   gint * exists, gint * recent, gint * unseen, gint * uid_validity)
{
	gint ok = 0;
	gchar *resp_str = NULL;
	GPtrArray *argbuf = NULL;
	gchar *select_cmd = NULL;
	gchar *folder_ = NULL;

	*exists = *recent = *unseen = *uid_validity = 0;
	argbuf = g_ptr_array_new();

	if (examine) {
		select_cmd = "EXAMINE";
	} else {
		select_cmd = "SELECT";
	}
	folder_ = g_strdup(INBOX_DIR);
	imap_cmd_gen_send(session, "%s %s", select_cmd, folder_);
	g_free(folder_);

	if ((ok = imap_cmd_ok(session, argbuf)) != IMAP_SUCCESS) {
		THROW;
	}

	resp_str = search_array_contain_str(argbuf, "EXISTS");

	if (resp_str && !*(exists)) {
		if (sscanf(resp_str, "%d EXISTS", exists) != 1) {
			THROW;
		}
	}

	resp_str = search_array_contain_str(argbuf, "RECENT");
	if (resp_str && !*(recent)) {
		if (sscanf(resp_str, "%d RECENT", recent) != 1) {
			THROW;
		}
	}

	resp_str = search_array_contain_str(argbuf, "UIDVALIDITY");
	if (resp_str && !*(uid_validity)) {
		if (sscanf(resp_str, "OK [UIDVALIDITY %u] ", (unsigned int *) uid_validity) != 1) {
			THROW;
		}
	}

	resp_str = search_array_contain_str(argbuf, "UNSEEN");
	if (resp_str && !*(unseen)) {
		if (sscanf(resp_str, "OK [UNSEEN %d] ", unseen) != 1) {
			THROW;
		}
	}

      catch:
	ptr_array_free_strings(argbuf);
	g_ptr_array_free(argbuf, TRUE);

	return ok;
}

#undef THROW

/** 
  This function sends the fetch imap command  to the imap server 
  for the part or header of the message
  @param session    pointer to the IMAPSession object created
  @param uid     message number
  @param filename    file name where the message is to be stored
  @param section_number    section number of the part/ to get the info 
  @return  (IMAP_SUCCESS)success or IMAP error code
*/
static gint
imap_cmd_mime_header(IMAPSession * session, guint32 uid,
		     const gchar * filename, gchar * section_number)
{
	gint ok = 0;
	gchar *buf = NULL;
	gchar *cur_pos = NULL;
	gchar size_str[IMAP_CMD_SIZE];
	glong size_num = 0;

	CHECK_RETURN_VAL_IF_FAIL(filename != NULL, IMAP_ERROR);

	memset(size_str, '\0', IMAP_CMD_SIZE);

	if (strcmp(section_number, HEADER_SECTION) == 0) {

		imap_cmd_gen_send(session, IMAP_BODY_PEEKMIME_CMD, uid, "HEADER");

	} else {
		imap_cmd_gen_send(session, IMAP_BODY_PEEKMIME_CMD, uid, section_number);
	}

	while ((ok = imap_cmd_gen_recv(session, &buf)) == IMAP_SUCCESS) {

		if (buf[0] != ASTERISK || buf[1] != SPACE_CHAR) {
			g_free(buf);
			return IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER;
		}
		if (strstr(buf, IMAP_FETCH_CMD) != NULL) {
			break;
		}
		g_free(buf);
	}
	if (ok != IMAP_SUCCESS) {
		return ok;
	}
	cur_pos = strchr(buf, IMAP_CURLY_START);
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(cur_pos != NULL);
	cur_pos = strchr_cpy(cur_pos + 1, IMAP_CURLY_END, size_str, sizeof(size_str));
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(cur_pos != NULL);
	size_num = atol(size_str);
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(size_num >= 0);

	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(*cur_pos == '\0');

	g_free(buf);
	if (recv_bytes_write_to_file(SESSION(session)->sock, size_num, filename) != 0) {
		return IMAP_ERROR;
	}

	if (imap_cmd_gen_recv(session, &buf) != IMAP_SUCCESS) {
		return IMAP_ERROR;
	}

	if (buf[0] == '\0' || buf[strlen(buf) - 1] != IMAP_END_BR) {
		g_free(buf);
		return IMAP_ERROR;
	}
	g_free(buf);
	ok = imap_cmd_ok(session, NULL);
	return ok;
}

#define THROW(err) { ok = err; goto catch; }

/** 
  This function sends the fetch imap command  to the imap server for 
  the body of the message
  @param session   pointer to the IMAPSession object created
  @param uid    UID message number
  @param filename    file name where the message is to be stored
  @return (IMAP_SUCCESS)success or error code
*/
static gint imap_cmd_fetch(IMAPSession * session, guint32 uid, const gchar * filename)
{
	gint ok = 0;
	gchar *buf = NULL;
	gchar *cur_pos = NULL;
	gchar size_str[IMAP_CMD_SIZE];
	glong size_num = 0;

	CHECK_RETURN_VAL_IF_FAIL(filename != NULL, IMAP_ERROR);
	memset(size_str, '\0', IMAP_CMD_SIZE);

	imap_cmd_gen_send(session, IMAP_BODY_PEEK_CMD, uid);

	while ((ok = imap_cmd_gen_recv(session, &buf)) == IMAP_SUCCESS) {

		if (buf[0] != ASTERISK || buf[1] != SPACE_CHAR) {
			g_free(buf);
			return IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER;
		}
		if (strstr(buf, IMAP_FETCH_CMD) != NULL) {
			break;
		}
		g_free(buf);
	}
	if (ok != IMAP_SUCCESS) {
		return ok;
	}
	cur_pos = strchr(buf, IMAP_CURLY_START);
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(cur_pos != NULL);
	cur_pos = strchr_cpy(cur_pos + 1, IMAP_CURLY_END, size_str, sizeof(size_str));
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(cur_pos != NULL);
	size_num = atol(size_str);
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(size_num >= 0);

	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(*cur_pos == '\0');

	g_free(buf);
	if (recv_bytes_write_to_file(SESSION(session)->sock, size_num, filename) != 0) {
		return IMAP_ERROR;
	}

	if (imap_cmd_gen_recv(session, &buf) != IMAP_SUCCESS) {
		return IMAP_ERROR;
	}

	if (buf[0] == '\0' || buf[strlen(buf) - 1] != IMAP_END_BR) {
		g_free(buf);
		return IMAP_ERROR;
	}
	g_free(buf);
	ok = imap_cmd_ok(session, NULL);
	return ok;
}

#define THROW(err) { ok = err; goto catch; }

/** 
  This function sends the fetch imap command  to the imap server for 
  the body of the message
  @param session    pointer to the IMAPSession 
  @param uid    UID message number
  @param filename    file name where the message is to be stored
  @param section_number    section  number of the attachment 
  in the multipart message 	
  @return  (IMAP_SUCCESS)success/error code
*/

static gint
imap_cmd_fetch_attach(IMAPSession * session, guint32 uid,
		      const gchar * filename, const gchar * section_number)
{
	gint ok = IMAP_FAIL;
	gchar *buf = NULL;
	gchar *cur_pos = NULL;
	gchar size_str[IMAP_CMD_SIZE];
	glong size_num = 0;
	gchar *tmp_pos = NULL;
	gchar *body = NULL;
	gboolean left_flag = FALSE;
	gboolean right_flag = FALSE;
	FILE *fp = NULL;

	CHECK_RETURN_VAL_IF_FAIL(filename != NULL, IMAP_ERROR);

	memset(size_str, '\0', IMAP_CMD_SIZE);

	if (strcmp(section_number, HEADER_SECTION) == 0) {
		imap_cmd_gen_send(session, IMAP_PEEK_HEADER_CMD, uid);
	} else {
		imap_cmd_gen_send(session, "UID FETCH %d BODY[%s]", uid, section_number);
	}

	while ((ok = imap_cmd_gen_recv(session, &buf)) == IMAP_SUCCESS) {
		if (buf[0] != '*' || buf[1] != ' ') {
			g_free(buf);
			return IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER;
		}
		if (strstr(buf, "FETCH") != NULL) {
			break;
		}
		g_free(buf);
	}
	if (ok != IMAP_SUCCESS) {
		return ok;
	}
	left_flag = FALSE;
	right_flag = FALSE;
	cur_pos = strchr(buf, '{');
	if (cur_pos == NULL) {
		left_flag = TRUE;
	}
	tmp_pos = strchr(buf, '}');
	if (tmp_pos == NULL) {
		right_flag = TRUE;
	}
	if (left_flag == TRUE && right_flag == TRUE) {
		log_message("{ } not found \n");
		tmp_pos = strchr(buf, '"');
		FREE_BUF_AND_RETURN_ERROR_IF_FAIL(tmp_pos != NULL);
		tmp_pos++;
		cur_pos = strchr(tmp_pos, '"');
		FREE_BUF_AND_RETURN_ERROR_IF_FAIL(cur_pos != NULL);
		body = g_strndup(tmp_pos, (cur_pos - tmp_pos));
		fp = fopen(filename, "w+");
		if (fp == NULL) {
			send_engine_error_to_ui(errno);
			imap_recv_cancel();
			g_free(body);
			g_free(buf);
			return IMAP_ERROR;
		}
		if (fwrite(body, strlen(body), 1, fp) < 0) {
			send_engine_error_to_ui(errno);
			imap_recv_cancel();
			g_free(body);
			g_free(buf);
			fclose(fp);
			return IMAP_ERROR;
		}
		fclose(fp);
		g_free(buf);
		if(strlen(body) != 0) {
			if (imap_cmd_gen_recv(session, &buf) != IMAP_SUCCESS) {
				g_free(body);
				return IMAP_ERROR;
			}
			if (buf[0] == '\0' || buf[strlen(buf) - 1] != ')') {
				g_free(body);
				g_free(buf);
				return IMAP_ERROR;
			}
			g_free(buf);
		}	
		g_free(body);
		ok = imap_cmd_ok(session, NULL);
		return ok;

	} else if (left_flag == TRUE || right_flag == TRUE) {
		g_free(buf);
		return IMAP_ERROR;
	}

	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(cur_pos != NULL);
	cur_pos = strchr_cpy(cur_pos + 1, '}', size_str, sizeof(size_str));
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(cur_pos != NULL);
	size_num = atol(size_str);
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(size_num >= 0);

	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(*cur_pos == '\0');

	g_free(buf);
	if (recv_bytes_write_to_file(SESSION(session)->sock, size_num, filename) != 0) {
		return IMAP_ERROR;
	}
	if (imap_cmd_gen_recv(session, &buf) != IMAP_SUCCESS) {
		return IMAP_ERROR;
	}
	if (buf[0] == '\0' || buf[strlen(buf) - 1] != ')') {
		g_free(buf);
		return IMAP_ERROR;
	}
	g_free(buf);
	ok = imap_cmd_ok(session, NULL);
	return ok;
}

/** 
  This function send the fetch imap command  to the imap server
  for the complete envelope of the message
  @param session    pointer to the IMAPSession object created
  @param seq_set   pointer to the set of message numbers
  @return  success(IMAP_SUCCESS) or error code
*/

gint imap_cmd_envelope(IMAPSession * session, const gchar * seq_set)
{
	imap_cmd_gen_send(session, "UID FETCH %s (UID FLAGS RFC822.SIZE RFC822.HEADER)", seq_set);

	return IMAP_SUCCESS;
}

/** 
  This function sends the store imap command  to the imap server
  @param session    pointer to the IMAPSession object created
  @param seq_set   pointer to the set of message numbers
  @param sub_cmd   options for storing 
  @return  success(IMAP_SUCCESS) or error code
*/

static gint imap_cmd_store(IMAPSession * session, const gchar * seq_set, const gchar * sub_cmd)
{
	gint ok = 0;

	imap_cmd_gen_send(session, "UID STORE %s %s", seq_set, sub_cmd);

	if ((ok = imap_cmd_ok(session, NULL)) != IMAP_SUCCESS) {
		log_message("error while imap command: STORE %s %s\n", seq_set, sub_cmd);
		return ok;
	}

	return IMAP_SUCCESS;
}

/** 
  This function sends the close imap command  to the imap server
  @param session    pointer to the IMAPSession object created
  @return  success(IMAP_SUCCESS) or error code
*/

static gint imap_cmd_close(IMAPSession * session)
{
	gint ok = 0;

	imap_cmd_gen_send(session, "CLOSE");
	if ((ok = imap_cmd_ok(session, NULL)) != IMAP_SUCCESS)
		log_message("error while imap command: CLOSE\n");

	return ok;
}

/** 
  This function precesses the complete response for the imap command  
  @param session    pointer to the IMAPSession object created
  @param arg_buf    pointer the string with the imap response 
  @return  success(IMAP_SUCCESS) or error code
*/
static gint imap_cmd_ok(IMAPSession * session, GPtrArray * argbuf)
{
	gint ok = 0;
	gchar *buf = NULL;
	gchar cmd_status[IMAPBUFSIZE + 1];
	gint cmd_num = 0;

	memset(cmd_status, '\0', sizeof(cmd_status));
	/*
	   receive the continues reponses form the server
	   and put the reponses in the variable "argbuf"
	 */
	while ((ok = imap_cmd_gen_recv(session, &buf)) == IMAP_SUCCESS) {
		/*
		   check for the untagged responses which will started with the 
		   charater "* "
		 */
		if (buf[0] == '*' && buf[1] == ' ') {
			if (g_strrstr(buf, "BYE") != NULL) {
				g_free(buf);
				return IMAP_MAIL_BOX_LOCK_LOST;
			}
			if (argbuf) {
				g_memmove(buf, buf + 2, strlen(buf + 2) + 1);
				g_ptr_array_add(argbuf, buf);
			} else {
				g_free(buf);
			}
			continue;
		}
		/*
		   check for the command completion reponses which 
		   will be in the form of tag OK/BAD/NO 
		 */
		if (sscanf(buf, "%d %" Xstr(IMAPBUFSIZE) "s", &cmd_num, cmd_status) < 2) {
			/*
			   split the response in terms of comand number(tag) 
			   and the response string OK/BAD
			 */
			g_free(buf);
			return IMAP_ERROR;
		} else if (cmd_num == session->cmd_count && !strcmp(cmd_status, "OK")) {
			/*
			   if the command tag number in the response and the command count 
			   is equal copy the buf in to the argbuf return success
			 */
			if (argbuf) {
				g_ptr_array_add(argbuf, buf);
			} else {
				g_free(buf);
			}
			return IMAP_SUCCESS;
		} else {
			g_free(buf);
			return IMAP_ERROR;
		}
	}

	return ok;
}

/** 
  This function sends the command to the imap server  
  @param session    pointer to the IMAPSession object created
  @param format    pointer the format string 
  @return  none 
*/
static void imap_cmd_gen_send(IMAPSession * session, const gchar * format, ...)
{
	gchar buf[IMAPBUFSIZE];
	gchar tmp[IMAPBUFSIZE];
	gchar *p = NULL;
	va_list args;

	if (gl_cancel_flag == TRUE) {
		return;
	}

	memset(buf, '\0', IMAPBUFSIZE);
	memset(tmp, '\0', IMAPBUFSIZE);
	va_start(args, format);
	g_vsnprintf(tmp, sizeof(tmp), format, args);
	va_end(args);

	session->cmd_count++;

	g_snprintf(buf, sizeof(buf), "%d %s\r\n", session->cmd_count, tmp);
	if (!strncasecmp(tmp, "LOGIN ", 6)
	    && (p = strchr(tmp + 6, ' '))) {
		*p = '\0';
		log_message("IMAP4> %d %s ********\n", session->cmd_count, tmp);
	} else {
		log_message("IMAP4> %d %s\n", session->cmd_count, tmp);
	}
	sock_write_all(SESSION(session)->sock, buf, strlen(buf));
}

/** 
  This function response received from the imap server
  @param sesssion    pointer to the IMAPSession object created
  @param OUT parameter,pointer to response received
  @return (IMAP SUCCESS or error)specifies status of the response captured 
*/

static gint imap_cmd_gen_recv(IMAPSession * session, gchar ** ret)
{
	if ((gl_cancel_flag == TRUE) || (session == NULL)) {
		return IMAP_SOCKET;
	}
	if ((*ret = sock_getline(SESSION(session)->sock)) == NULL) {
		return IMAP_SOCKET;
	}

	strretchomp(*ret);
	log_message("IMAP4< %s\n", *ret);

	return IMAP_SUCCESS;
}


/* misc utility functions */
/** 
  This function returns the part of the string,till the 
  character is found.Memory should be allocated for destination 
  string by the calling function.
  @param src    pointer to the string to copy 
  @param ch    character to find 
  @param dest    pointer to the destination string 
  @param len    length of the string
  @return  pointer to the string 
*/

static gchar *strchr_cpy(const gchar * src, gchar ch, gchar * dest, gint len)
{
	gchar *tmp = NULL;

	dest[0] = '\0';
	tmp = strchr(src, ch);
	if (!tmp) {
		return NULL;
	}

	memcpy(dest, src, MIN(tmp - src, len - 1));
	dest[MIN(tmp - src, len - 1)] = '\0';

	return tmp + 1;
}

/** 
  This function searches the specified string in the array .
  @param array    pointer to the GPtrArray object created
  @param str    pointer to the gchar created
  @return pointer to the string found
*/

static gchar *search_array_contain_str(GPtrArray * array, gchar * str)
{
	gint i = 0;

	for (i = 0; i < array->len; i++) {
		gchar *tmp = NULL;

		tmp = g_ptr_array_index(array, i);
		if (strstr(tmp, str) != NULL) {
			return tmp;
		}
	}

	return NULL;
}

/** 
  This function searches the specified string in the array .
  @param array   pointer to the GPtrArray object created
  @param str    pointer to the string to search 
  @retur string found in the array.Return value should not be freed
*/

static gchar *search_array_str(GPtrArray * array, gchar * str)
{
	gint i = 0;
	gint len = 0;

	len = strlen(str);

	for (i = 0; i < array->len; i++) {
		gchar *tmp = NULL;

		tmp = g_ptr_array_index(array, i);
		if (!strncmp(tmp, str, len)) {
			return tmp;
		}
	}

	return NULL;
}

/** 
  This funtion returns the length of the num   
  @param number  number get the length
  @return length of the number
*/
static int get_num_len(const guint32 num)
{
	gint count = 0, result = 1;
	guint32 test = 1;

	for (count = 1; result > 0; count++) {
		test = test * 10;
		result = num / test;
	}
	return count - 1;
}

/** 
  This function scans inbox present in the IMAP server.
  @param prefs_account    pointer to the PrefsAccount object created
  @param inbox_status    pointer to the InboxStatus object created
  @return  none 
*/
void imap_scan_inbox(PrefsAccount * prefs_account, InboxStatus * inbox_status)
{
	Session *session = NULL;
	gint exists = 0, unseen = 0, recent = 0, uid_validity = 0;

	if ((prefs_account == NULL) || (inbox_status == NULL)) {
		engine_errno = IMAP_PARAMETER_MISSING;
		IMAP_CB(IMAP_PARAMETER_MISSING);
		return;
	}
	/* Creates a Session */
	session = imap_session_new(prefs_account, NULL);
	if (session == NULL) {
		engine_errno = ENGINE_ERROR_CONNECT_SERVER;
		return;
	}
	/* selects the inbox in read only mode & fetches the inbox status. */
	if ((imap_cmd_do_uid_select(IMAP_SESSION(session), INBOX_DIR,
				    FALSE, &exists, &unseen, &recent,
				    &uid_validity, prefs_account, TRUE))
	    == IMAP_SUCCESS) {
		inbox_status->exists = exists;
		inbox_status->unseen = unseen;
		inbox_status->recent = recent;
		inbox_status->uid_validity = uid_validity;

	} else {
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		return;
	}
	IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
	imap_session_disconnect(session, FALSE);
	IMAP_CB(IMAP_SUCCESS);
	engine_errno = IMAP_SUCCESS;
}				//end of imap_scan_inbox function

/** 
  This function sends the select imap command through imap_cmd_do_select  
  the imap server also gets the latest uid value in the server 
  @param session   IMAP session imformation 
  @param folder     name of the folder
  @param examine    flag to select he specified folder in read-write or 
  read only mode
  @param exists    pointer to the integer get existing messages in the folder
  @param recent    pointer to the integer get recent messages in the folder
  @param unseen    pointer to the integer get unseen messages in the folder
  @param uid_validity    pointer to the integer get uid_validity in the folder
  @param prefs_account    pointer to the account 
  @param send_error_cb    indicates whether error has to be sent 
  @return    (IMAP_SUCCESS)success or error code
*/
static gint
imap_cmd_do_uid_select(IMAPSession * session,
		       const gchar * folder,
		       gboolean examine,
		       gint * exists, gint * recent,
		       gint * unseen, gint * uid_validity,
		       PrefsAccount * prefs_account, gboolean send_error_cb)
{
	gchar *tmp = NULL, *temp_last = NULL, *temp_uid = NULL;
	gint exts = 0, unsn = 0, rcnt = 0, uid_vldty = 0, length = 0;
	gint ret_val = -1;
	Session *session_new = NULL;

	ret_val = imap_cmd_do_select(session, folder, examine, &exts, &unsn, &rcnt, &uid_vldty);
	if (ret_val == IMAP_MAIL_BOX_LOCK_LOST) {
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(SESSION(session), TRUE);
		prefs_account->user_settings.session = NULL;
		session_new = imap_session_new(prefs_account, NULL);
		if (session_new == NULL) {
			log_message("not able to connect %s  server at %d port",
				    prefs_account->receive_settings.recv_server,
				    prefs_account->receive_settings.port);
			engine_errno = ENGINE_ERROR_CONNECT_SERVER;
			return ENGINE_ERROR_CONNECT_SERVER;
		}
		session = IMAP_SESSION(session_new);
		ret_val = imap_cmd_do_select(session, folder,
					     examine, &exts, &unsn, &rcnt, &uid_vldty);
	}
	if (ret_val == IMAP_SUCCESS) {
		gl_msgs_exists_server = exts;
		*exists = exts;
		*recent = rcnt;
		*unseen = unsn;
		if (exts == 0) {
			return IMAP_SUCCESS;
		}

		tmp = g_strdup_printf("FETCH %d (UID)", exts);

		imap_cmd_gen_send(IMAP_SESSION(session), tmp);
		g_free(tmp);

		while (imap_cmd_gen_recv(IMAP_SESSION(session), &tmp) == IMAP_SUCCESS) {
			if (tmp[0] != '*' || tmp[1] != ' ') {
				g_free(tmp);
				break;
			}
			temp_last = strchr(tmp, 'D');
			length = 0;

			if (temp_last == NULL) {
				g_free(tmp);
				break;

			}
			length = (strlen(tmp) - (temp_last - tmp) - 3);
			temp_uid = g_strndup(temp_last + 2, length);

			*uid_validity = uid_vldty;
			gl_latest_uid = atoi(temp_uid);

			g_free(temp_uid);
			g_free(tmp);
		}

	} else {
		engine_errno = ENGINE_ERROR_OPERATION;
		if(!imap_is_session_cancelled())
			IMAP_CB(IMAP_INBOX_SELECT_ERROR);
		return IMAP_INBOX_SELECT_ERROR;
	}
	if ((prefs_account->receive_settings.last_uid_validity != 0) &&
	    (prefs_account->receive_settings.last_uid_validity != *uid_validity)) {

		engine_errno = ENGINE_ERROR_IMAP_SYNC;

		if (send_error_cb == TRUE) {
			IMAP_SET_MSG_CB(IMAP_SERVER_SYNC_ERROR,
					prefs_account->user_settings.account_name);
		} else {
			IMAP_SET_ERROR_CB(IMAP_SERVER_SYNC_ERROR);
			prefs_account->receive_settings.last_uid_validity = *uid_validity;
			prefs_account->receive_settings.last_msgid = 0;
			prefs_account->receive_settings.first_msgid = 0;
			prefs_account->receive_settings.num_of_msgs = 0;
			account_update_list();
		}

		return IMAP_SERVER_SYNC_ERROR;
	} else if (prefs_account->receive_settings.last_uid_validity == 0) {
		prefs_account->receive_settings.last_uid_validity = *uid_validity;
		prefs_account->receive_settings.last_msgid = 0;
		prefs_account->receive_settings.first_msgid = 0;
		account_update_list();
	}
	return IMAP_SUCCESS;

}

/** 
  This function deletes the messages from the IMAP server.This function 
  also used to set the seen status on the server for a 
  list of read messages.
  @param prefs_account    pointer to the prefs_account object created
  @param msglist    pointer to the GSList of messages to be deleted  	 
  @param folder_succ_list    (OUT)pointer to the GSList of messages 
  to deleted successfully  	 
  @param type    indicates whether seen operation or delete  	 
  @param curptr    (OUT)pointer to the current message number  	 
  @param total    number of total messages  	 
  @param progress_cb    pointer to the callback function,to be called 
  for progress indication to UI.  	 
  @return none
*/

void
imap_expunge_msg(PrefsAccount * prefs_account, GSList * msglist,
		 GSList ** folder_succ_list, ListType type,
		 gint * cur_ptr, gint total, FolderProgressCb progress_cb)
{
	Session *session = NULL;
	gint exists = 0, unseen = 0, recent = 0, uid_validity = 0, ok = IMAP_FAIL;
	gchar *uid_no = NULL;
	GSList *cur = NULL;
	gint msgnum = 0;

	engine_errno = ENGINE_ERROR_DEFAULT;
	if ((prefs_account == NULL) || (msglist == NULL)) {
		IMAP_CB(IMAP_PARAMETER_MISSING);
		engine_errno = IMAP_PARAMETER_MISSING;
		return;
	}
	/* Creates a Session */
	/* flag to check if server connection is mandatory or not
        This is done to Fix bug 45338*/   
        not_mandatory_to_conn_serv = TRUE;
	session = imap_session_new(prefs_account, NULL);
	not_mandatory_to_conn_serv = FALSE;

	if (session == NULL) {
		for (cur = msglist; cur != NULL; cur = cur->next) {
			if (type == DELETE) {
				folder_add_list(cur->data, FALSE);
			} else if (type == SEEN) {
				folder_message_add_list(cur->data, SEEN);
			}
		}
		engine_errno = ENGINE_ERROR_CONNECT_SERVER;
		return;
	}
	ACCNT_SET_FLAGS(prefs_account, ACCNT_RX);
	if ((imap_cmd_do_uid_select(IMAP_SESSION(session), INBOX_DIR,
				    FALSE, &exists, &unseen, &recent,
				    &uid_validity, prefs_account, TRUE)) != IMAP_SUCCESS) {
		for (cur = msglist; cur != NULL; cur = cur->next) {
			if (type == DELETE) {
				folder_add_list(cur->data, FALSE);
			} else if (type == SEEN) {
				folder_message_add_list(cur->data, SEEN);
			}
		}
		engine_errno = ENGINE_ERROR_CONNECT_SERVER;
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		return;
	}

	for (cur = msglist; cur != NULL; cur = cur->next) {

		if ((progress_cb != NULL) && (cur_ptr != NULL)) {
			(*cur_ptr)++;
			progress_cb(*cur_ptr, total);
		}
		msgnum = get_msgid_num(cur->data);
		if ((msgnum) > gl_latest_uid) {
			IMAP_DISCONNECT(session, IMAP_INVALID_UID);
			IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
			engine_errno = ENGINE_ERROR_IMAP_MSG_DELETE;
			if (folder_succ_list != NULL) {

				*folder_succ_list =
				    folder_append_header_list(*folder_succ_list, cur->data,
							      NULL, prefs_account, TRUE,
							      NULL, NULL);
			}
			return;
		}
		uid_no = g_strdup_printf("%d", msgnum);
		if (type == DELETE) {
			/* sets the message flag deleted */
			ok = imap_set_message_flags(IMAP_SESSION(session),
						    uid_no, IMAP_FLAG_DELETED, TRUE);
			if (ok != IMAP_SUCCESS) {
				engine_errno = ENGINE_ERROR_CONNECT_SERVER;
				folder_add_list(cur->data, FALSE);
			}
		} else if (type == SEEN) {
			/* sets the message flag Seen */
			ok = imap_set_message_flags(IMAP_SESSION(session),
						    uid_no, IMAP_FLAG_SEEN, TRUE);
			if (ok == IMAP_SUCCESS) {
				folder_message_del_list(cur->data, SEEN);
			} else {
				folder_message_add_list(cur->data, SEEN);
			}
		}

		if (ok != IMAP_SUCCESS) {
			engine_errno = ENGINE_ERROR_IMAP_MSG_DELETE;
		}
		g_free(uid_no);
		if (folder_succ_list != NULL) {

			*folder_succ_list =
			    folder_append_header_list(*folder_succ_list, cur->data,
						      NULL, prefs_account, TRUE, NULL, NULL);
		}

		(void) g_main_context_iteration(NULL, FALSE);

		if (gl_imap_recv_cancel == TRUE) {

			gl_imap_recv_cancel = FALSE;
			break;
		}
	}
	if ((type == DELETE) && (engine_errno == ENGINE_ERROR_DEFAULT)) {
		ok = imap_cmd_close(IMAP_SESSION(session));
	}
	/* closes the session */
	IMAP_DISCONNECT(session, IMAP_SUCCESS);

	IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
	return;

}

/** 
  This function fetches the UID of a message(sequence number) from server.
  @param session    pointer to the IMAP session
  @param seq_no    message sequence number  	 
  @return UID returned by server,else zero 
*/

static gint imap_cmd_get_uid(IMAPSession * session, gint seq_no)
{
	gchar *tmp = NULL, *temp_last = NULL, *temp_uid = NULL;
	gint len = 0, uid_no = 0;


	tmp = g_strdup_printf("FETCH %d (UID)", seq_no);

	imap_cmd_gen_send(IMAP_SESSION(session), tmp);
	g_free(tmp);

	while (imap_cmd_gen_recv(IMAP_SESSION(session), &tmp) == IMAP_SUCCESS) {
		if (tmp[0] != '*' || tmp[1] != ' ') {
			g_free(tmp);
			break;
		}
		temp_last = strchr(tmp, 'D');
		if(temp_last == NULL) {
			g_free(tmp);
			break;
		}
		len = (strlen(tmp) - (temp_last - tmp) - 3);
		if((len >0) && (len < strlen(tmp))) {
			temp_uid = g_strndup(temp_last + 2, len);
			uid_no = atoi(temp_uid);
			g_free(temp_uid);
		}
		g_free(tmp);
	}
	return uid_no;
}

/** 
  This function fetches the message headers from the IMAP server.
  @param prefs_account    pointer to the prefs_account object created
  @param folder    folder name 	 
  @param first_msgid     first uid to be fetched
  @param last_msgid     last uid to be fetched
  @return none
*/
void
imap_get_headers(PrefsAccount * prefs_account,
		 const gchar * folder, gint first_msgid, gint last_msgid)
{
	Session *session = NULL;
	gint exists = 0;
	gint unseen = 0;
	gint recent = 0;
	gint uid_validity = 0;
	GSList *new_list = NULL;
	gint msg_count = 0;
	gint first_uid = 0;
	gint last_uid = 0;
	gint msg_last_uid = 0;
	gboolean update_first = TRUE;
	gboolean update_last = TRUE;
	gboolean status = FALSE;

	if ((prefs_account == NULL) || (folder == NULL)) {
		engine_errno = IMAP_PARAMETER_MISSING;
		IMAP_CB(IMAP_PARAMETER_MISSING);
		return;
	}
	if ((first_msgid < 0) || (last_msgid < 0)) {
		engine_errno = IMAP_INVALID_RECENT_MAIL_SETTING;
		IMAP_CB(IMAP_INVALID_RECENT_MAIL_SETTING);
		return;
	}
	IMAP_SET_MSG_CB(IMAP_START, prefs_account->user_settings.account_name);

	/* Creates a Session */
	session = imap_session_new(prefs_account, NULL);

	if (session == NULL) {
		engine_errno = ENGINE_ERROR_CONNECT_SERVER;
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		return;
	} else {
		IMAP_SET_MSG_CB(IMAP_CONNECT_SERVER_SUCCESS,
				prefs_account->user_settings.account_name);
	}

	/* selects the inbox in read only mode & fetches the inbox status. */

	if ((imap_cmd_do_uid_select(IMAP_SESSION(session), INBOX_DIR,
				    FALSE, &exists, &unseen, &recent,
				    &uid_validity, prefs_account, FALSE)) != IMAP_SUCCESS) {
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		return;
	}
	if (exists == 0) {
		imap_check_deleted_msgs(NULL, prefs_account->user_settings.account_name);
		if (gl_imap_delete_list != NULL) {
			IMAP_SEND_DEL_LIST_CB(gl_imap_delete_list);
			slist_free_strings(gl_imap_delete_list);
			g_slist_free(gl_imap_delete_list);
			gl_imap_delete_list = NULL;
		}
	}
	if (exists <= 0) {

		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		return;
	}
	if ((last_msgid > gl_latest_uid) || (first_msgid > gl_latest_uid) ||
	    (first_msgid > last_msgid)) {
		IMAP_DISCONNECT(session, IMAP_INVALID_UID);
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		return;
	}
	if (prefs_account->receive_settings.recent_mails < 0) {
		IMAP_DISCONNECT(session, IMAP_NO_HEADERS_DOWNLOADED);
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		return;
	}
	if (imap_get_range(prefs_account, &last_uid, &first_uid,
			   &update_last, session, &msg_last_uid) == IMAP_FAIL) {

		/* call search to sync with server */
		if ((first_uid != 0) && (last_uid != 0)) {

			msg_count = imap_cmd_search(IMAP_SESSION(session),
						    &first_uid, &last_uid,
						    prefs_account, &update_first,
						    msg_last_uid, FALSE);

		}
		if (gl_imap_delete_list != NULL) {
			IMAP_SEND_DEL_LIST_CB(gl_imap_delete_list);
			slist_free_strings(gl_imap_delete_list);
			g_slist_free(gl_imap_delete_list);
			gl_imap_delete_list = NULL;
		}
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		return;
	}

	gl_imap_recv_cancel = FALSE;

	msg_count = imap_cmd_search(IMAP_SESSION(session), &first_uid,
				    &last_uid, prefs_account, &update_first, msg_last_uid, TRUE);
	if (gl_imap_delete_list != NULL) {
		IMAP_SEND_DEL_LIST_CB(gl_imap_delete_list);
		slist_free_strings(gl_imap_delete_list);
		g_slist_free(gl_imap_delete_list);
		gl_imap_delete_list = NULL;
	}

	if (prefs_account->receive_settings.recv_type == HEADERS) {
		if (msg_count <= 0) {
			IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
			imap_session_disconnect(session, FALSE);
			return;

		}

 		if(gl_imap4_callback_ptr && gl_imap4_callback_ptr->imap4_set_msg_cb ){
			gl_imap4_callback_ptr->imap4_set_msg_cb(IMAP_PROGRESS,"%d%d",1,msg_count);
		}
		IMAP_PROGRESS_CB(1, msg_count);
	}
	new_list = imap_get_uncached_msgs(IMAP_SESSION(session),
					  prefs_account, folder, first_uid, &last_uid, &msg_count);

        // Check if someone closed the connection while retrieving messages
        if (prefs_account->user_settings.session == NULL) {
                while (new_list) {
                        procmsg_msginfo_free((MsgInfo *) new_list->data);
                        new_list = g_slist_remove(new_list, new_list->data);
                }
                IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
                return;
        }

	if (new_list == NULL) {
		log_message("not able to fetch new messages\n");
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		return;
	}
	if (prefs_account->receive_settings.recv_type != HEADERS) {
		IMAP_PROGRESS_CB(1, msg_count);
	}
	if ((gl_imap_recv_cancel == TRUE)
	    && (prefs_account->receive_settings.recv_type != HEADERS)) {

		while (new_list) {
			procmsg_msginfo_free((MsgInfo *) new_list->data);
			new_list = g_slist_remove(new_list, new_list->data);
		}
		new_list = NULL;
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		return;
	} else {
		/* Reset cancel flag if the recv type is headers */
		gl_imap_recv_cancel = FALSE;
	}
	status = TRUE;
	msg_headers_send(new_list, session, prefs_account, msg_count, &last_uid, &status);

	if (status == TRUE) {
		if (update_first == TRUE) {
			prefs_account->receive_settings.first_msgid = first_uid;
		}
		if (update_last == TRUE) {
			prefs_account->receive_settings.last_msgid = last_uid;
		}
		prefs_account->receive_settings.num_of_msgs = gl_msgs_exists_server;
		account_update_list();

	}
	IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
	imap_session_disconnect(session, FALSE);

	return;
}

/** 
  This function fetches the message body for header from the IMAP server.
  @param session    pointer to the session used 
  @param prefs_account    pointer to the account object created
  @param header_file_name    header file name 	 
  @param body_file_name    file name in which body has to be saved	 
  @param header    pointer to the message header information	 
  @param sec_nu    pointer to the section number of body 	 
  @param uid_no    UID number of the message 	 
  @param file_name    file name in which entire message will be saved 	 
  @param mimeinfo     pointer to the mime information of the message 	 
  @return IMAP_SUCCESS on success or IMAP error 
*/
static gint
download_imap_body(Session * session, PrefsAccount * prefs_account,
		   const gchar * header_file_name,
		   const gchar * body_file_name,
		   MsgHeader * header, const gchar * sec_nu,
		   gint uid_no, const gchar * file_name, MimeInfo * mimeinfo)
{
	gint body_size = 0, ret = IMAP_FAIL;

	body_size = imap_cmd_get_body_size((Session *) session, header->msgid, MSG_BODY_SEC);
	if (body_size == IMAP_FAIL) {
		return ret;
	}
	if (((float) body_size) / ONE_KB > (float) prefs_account->receive_settings.size_limit) {
		debug_print("body_size : %d \n", body_size);
		engine_errno = ENGINE_SIZE_ERROR;
		return ret;
	} else {

		ret = imap_get_body(prefs_account, session, uid_no,
				    header_file_name, body_file_name, header, file_name, mimeinfo);
	}
	return ret;
}

/**
  This function returns the complete file path 
  @param msgid    message id of the message
  @param sec_nu    section number of the part,for which to get filename 
  @return file path,in which the part of the message has to be saved.
  This string returned should be freed by calling function.
 */
static
gchar *imap_get_file_name(const gchar * msgid, const gchar * sec_nu)
{
	gchar *file_name = NULL;
	file_name = g_strdup_printf("%s%c%s%c%s%c%s",
				    get_mail_dir(), G_DIR_SEPARATOR,
				    INBOX_DIR, G_DIR_SEPARATOR, msgid, SECTION_SEPARATOR, sec_nu);
	return file_name;
}

/**
  This function fetches the body for imap mails as per the receive type
  and sends the msginfo list to the upper layer
  @param  new_list    list of message information
  @param  session    session information
  @param  prefs_account    account information
  @param  msg_count    count of messages to receive 
  @param  last_uid    (OUT)pointer to the last uid recevied. 
  @param  status    (OUT)indicates whether to save the last uid 
  and other indexes 
  @return none
*/
void
msg_headers_send(GSList * new_list, Session * session,
		 PrefsAccount * prefs_account, gint msg_count, gint * last_uid, gboolean * status)
{
	gint ret = IMAP_FAIL;
	GSList *c_list = NULL;
	GSList *prev = NULL;
	MsgInfo *msg_info = NULL;
	gint cur_msg = 1;
	gboolean size_error_occurred = FALSE;
	MimeInfo *tmp_mimeinfo = NULL;
	/*cache changes */
	gchar *file_path = NULL;
	gchar *header_path = NULL;
	gchar *msgid = NULL;
	GSList *list = NULL;
	PrefsAccount *new_account = NULL;
	gint len = 0;
	gint length = 0;
	gint counter = 0;
	gint count = 0;


	engine_errno = ENGINE_ERROR_DEFAULT;

	for (c_list = new_list; (c_list != NULL); prev = c_list, c_list = c_list->next) {

		if (prefs_account->receive_settings.recv_type == HEADERS) {
			break;
		}

		(void) g_main_context_iteration(NULL, FALSE);
		if (gl_imap_recv_cancel == TRUE) {
			gl_imap_recv_cancel = FALSE;
			if (prev != NULL) {
				prev->next = NULL;
				msg_info = prev->data;
				if (msg_info != NULL) {
					*last_uid = get_msgid_num(msg_info->header->msgid);
				}
			} else {
				/* not downloaded anything,do not save the indexes */
				*status = FALSE;
				new_list = NULL;
			}
			imap_remove_list_msgs(c_list);
			c_list = NULL;
			break;
		}
		if ((engine_errno != ENGINE_SIZE_ERROR) &&
		    (prefs_account->receive_settings.recv_type != HEADERS)) {
			IMAP_PROGRESS_CB(cur_msg, msg_count);
		}
		engine_errno = ENGINE_ERROR_DEFAULT;
		cur_msg++;
		ret = IMAP_FAIL;
		msg_info = (MsgInfo *) c_list->data;

		tmp_mimeinfo = imap_get_msg_mimeinfo(msg_info, session, prefs_account);

		if ((msg_info != NULL) && (tmp_mimeinfo != NULL)) {
			/* download the message,for which header is already 
			 * downloaded */
			ret = imap_download_msg(msg_info, session, prefs_account,
						(MimeInfo *) tmp_mimeinfo);
			msgid = g_strdup(msg_info->header->msgid);
			file_path = folder_get_message_file_path(msgid);
			if (is_file_entry_exist(file_path) == TRUE) {
				header_path = g_strdup_printf("%s%c%s", file_path, DOT, HEAD);
				unlink(header_path);
				g_free(header_path);
			}
			g_free(msgid);
			g_free(file_path);
			procmime_mimeinfo_free_all(tmp_mimeinfo);
			tmp_mimeinfo = NULL;
		}

		if (engine_errno == ENGINE_SIZE_ERROR) {
			size_error_occurred = TRUE;
		}

		msg_info->header->smime_type = imap_get_smime_type(msg_info);
		if ((msg_info->header->msgtag.fullydownloaded == TRUE) &&
		    MSG_IS_ATTACH(msg_info->header->flags)) {

			MSG_SET_TMP_FLAGS(msg_info->header->flags, MSG_PURGABLE);

		}
		if (gl_imap4_callback_ptr) {
			if(gl_imap4_callback_ptr->imap4_msg_headers_cb && CHECK_CB_FLAG) {
				list = g_slist_append(list, msg_info);
				gl_imap4_callback_ptr->imap4_msg_headers_cb(list);
                                notify_mail_list(list);
				g_slist_free(list);
				list = NULL;
			}
		}
	}
	if (size_error_occurred == TRUE) {
		IMAP_SET_ERROR_CB(IMAP_MSG_SIZE_EXCEED_ERROR);

	}
        
        if ( gl_imap4_callback_ptr && gl_imap4_callback_ptr->imap4_msg_headers_cb && CHECK_CB_FLAG) {
                /*send msginfo to UI in list of NUM_MAILS mailsin each callback */
                len = g_slist_length(new_list);
        	if (prefs_account->receive_settings.recv_type == HEADERS) { 
                	for (counter = 0; counter < len; counter++) {
                        	list = g_slist_append(list, g_slist_nth_data(new_list, counter));
                        	if ((g_slist_length(list) == NUM_MAILS) || (counter == len - 1)) {
                                        IMAP_PROGRESS_CB(counter+1, len);
                                	gl_imap4_callback_ptr->imap4_msg_headers_cb(list);
                                        notify_mail_list(list);
                                	/*after sending the list free its memory */
                                	length = g_slist_length(list);
                                	for (count = 0; count < length; count++) {
                                        	list = g_slist_remove(list, g_slist_nth_data(list, count));
                                	}
                                	g_slist_free(list);
                                	list = NULL;
                        	}
			}
			IMAP_PROGRESS_CB(len, len);
                }
        }

	if ((new_list != NULL) && (len > 0) &&
	    (prefs_account->receive_settings.is_not_editable == FALSE)) {
		prefs_account->receive_settings.is_not_editable = TRUE;
		new_account = account_copy_prefs(prefs_account);
		if (account_set_settings(new_account->user_settings.account_name,
					 RECEIVE_SETTING, new_account) == FALSE) {
			debug_print("Unable to set account settings for imap\n");
		}
		account_free_account(new_account);
	}

	/* free the list after sending */
	while (new_list) {
		procmsg_msginfo_free((MsgInfo *) new_list->data);
		new_list = g_slist_remove(new_list, new_list->data);
	}
	new_list = NULL;
}

/**
  This function returns the size of the mail
  @param session session information
  @param msgnum mail uid
  @return size of the mail 

*/
static gint imap_cmd_mail_size(Session * session, gint msgnum)
{
	gchar *tmp = NULL;
	gint size = 0;

	tmp = g_strdup_printf("UID FETCH %d RFC822.size", msgnum);
	imap_cmd_gen_send(IMAP_SESSION(session), tmp);
	g_free(tmp);
	while ((imap_cmd_gen_recv(IMAP_SESSION(session), &tmp) == IMAP_SUCCESS)) {
		if (tmp[0] != '*' || tmp[1] != ' ') {
			g_free(tmp);
			break;
		}
		size = return_size(tmp);
		g_free(tmp);
	}
	return size;
}

/**
  This function returns the part size of the mail
  @param session    pointer to the session information
  @param msgid    message id
  @param sec_nu    section number of the mail
  @return size of the part
*/
static gint imap_cmd_get_body_size(Session * session, gchar * msgid, gchar * sec_nu)
{
	gchar *body_str = NULL;
	MimeInfo *mime_info = NULL;
	gint return_size = 0;
	FILE *mime_fp;
	guchar buf[BUFSIZ];
	gchar *mime_file_name = NULL;
	gchar *tmp_str = NULL;
	size_t len;

	mime_file_name = g_strdup_printf("%s%c%s%c%s%c%s",
					 get_mail_dir(),
					 G_DIR_SEPARATOR, INBOX_DIR,
					 G_DIR_SEPARATOR, msgid, SECTION_SEPARATOR, MIME_SUFFIX);
	memset(buf, '\0', BUFSIZ);
	mime_fp = fopen(mime_file_name, "r");
	if (mime_fp == NULL) {
		return IMAP_FAIL;
	}
	while ((len = fread(buf, sizeof(gchar), sizeof(buf) - 1, mime_fp)) > 0) {
		if (tmp_str != NULL) {
			body_str = g_strconcat(tmp_str, buf, NULL);
			g_free(tmp_str);
		} else {
			body_str = g_strconcat(buf, NULL);
		}
		if (body_str == NULL) {
			fclose(mime_fp);
			return IMAP_FAIL;
		}
		tmp_str = body_str;
	}

	mime_info = procmime_scan_body(body_str, strlen(body_str));
	g_free(body_str);
	fclose(mime_fp);
	if (mime_info != NULL) {
		return_size = procmime_get_body_size(mime_info, sec_nu);
	} else {
		return IMAP_FAIL;
	}
	procmime_mimeinfo_free_all(mime_info);

	return return_size;
}

/** 
  This function fetches message body ftom the IMAP server.
  @param msgid    pointer to the msgid
  @param prefs_account    pointer to the account information 	 
  @param read_flag    indicates whether the message is already read 	 
  @param size_check    indicates whether the size check is applicable 	 
  @param smime_check    indicates whether the size check is applicable 	 
  @param in_session    pointer to the session used 	 
  @param close_session    indicates whether the session has to be 
  closed after use 	 
  @return  none
*/
void
imap_cmd_fetch_msg_body(const gchar * msgid, PrefsAccount * prefs_account,
			gboolean read_flag, gboolean size_check,
			gboolean smime_check, Session * in_session, gboolean close_session)
{
	Session *session = NULL;
	gint exists = 0, unseen = 0, recent = 0, uid_validity = 0;
	gint ok = IMAP_FAIL, msgnum = 0;
	gchar *filename = NULL;
	gchar *body_file_name = NULL;
	gchar *header_file_name = NULL;
	MsgHeader *header = NULL;
	gboolean fullydownloaded = FALSE;
	gboolean bodydownloaded = FALSE;
	MimeInfo *mime_info = NULL;
	MsgFlags imap_flags = { 0, 0 };

	if ((msgid == NULL) || (prefs_account == NULL)) {
		engine_errno = IMAP_PARAMETER_MISSING;
		IMAP_CB(IMAP_PARAMETER_MISSING);
		return;
	}
	if ((check_msgid(msgid) == FALSE)) {
		engine_errno = IMAP_INVALID_MSGID_ERROR;
		IMAP_CB(IMAP_INVALID_MSGID_ERROR);
		return;
	}
	/* Creates a Session */
	session = in_session;
	msgnum = get_msgid_num(msgid);

	if (session == NULL) {

		session = imap_session_new(prefs_account, msgid);

		if (session == NULL) {
			engine_errno = ENGINE_ERROR_CONNECT_SERVER;
			return;
		} else {
			IMAP_SET_MSG_STR_CB(IMAP_CONNECT_SERVER_SUCCESS,
					    prefs_account->user_settings.account_name, msgid);
		}

		/* selects the inbox in read only mode & fetches the inbox status. */

		if ((imap_cmd_do_uid_select(IMAP_SESSION(session), INBOX_DIR,
					    FALSE, &exists, &unseen, &recent,
					    &uid_validity, prefs_account, TRUE)) != IMAP_SUCCESS) {
			if (close_session == TRUE) {

				IMAP_SET_MSG_CB(IMAP_COMPLETE,
						prefs_account->user_settings.account_name);
				imap_session_disconnect(session, FALSE);
			}
			return;
		}

		ok = imap_cmd_flags(IMAP_SESSION(session), msgnum, &imap_flags);

		if ((ok != IMAP_SUCCESS)
		    || (imap_flags.perm_flags & MSG_DELETED)) {
			if(!imap_is_session_cancelled())
				IMAP_SET_MSG_CB(IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER, msgid);
			imap_delete_header_file(msgid);
			if (close_session == TRUE) {
				imap_session_disconnect(session, FALSE);
				IMAP_SET_MSG_CB(IMAP_COMPLETE,
						prefs_account->user_settings.account_name);
			}
			return;
		}
	}


	filename = folder_get_message_file_path(msgid);

	if (is_file_entry_exist(filename)) {
		g_free(filename);
		IMAP_DISCONNECT(session, IMAP_MAIL_ALREADY_DOWNLOADED);
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		return;
	}
	if (prefs_account->receive_settings.enable_size_limit &&
	    CHECK_CB_FLAG && (size_check == TRUE)) {

		mime_info = imap_scan_mimeinfo(msgid, filename, session);

		if (mime_info == NULL) {
			g_free(filename);

			if (close_session == TRUE) {
				IMAP_SET_MSG_CB(IMAP_COMPLETE,
						prefs_account->user_settings.account_name);
				imap_session_disconnect(session, FALSE);
			}
			return;

		}
		header = g_new0(MsgHeader, 1);

		if (header == NULL) {
			g_free(filename);
			IMAP_DISCONNECT(session, IMAP_ERROR);
			IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
			return;
		}
		header->msgid = g_strdup(msgid);
		body_file_name = imap_get_file_name(msgid, MSG_BODY_SEC);

		header_file_name = imap_get_file_name(msgid, HEAD);

		if (procmime_mime_attachment(mime_info) == TRUE) {
			header->flags.perm_flags |= MSG_ATTACH;
		}

		ok = imap_msg_size_limit(prefs_account, session,
					 msgnum, header_file_name,
					 body_file_name, header, filename, mime_info, smime_check);
		
		/*remove the related files if sessin is cancelled*/
		gchar *mime_file = NULL;
		if(imap_is_session_cancelled()){
                         mime_file = imap_get_file_name(header->msgid,MIME_SUFFIX);
                         if(is_file_entry_exist(mime_file))
                            unlink(mime_file);
                         if(is_file_entry_exist(body_file_name))
                             unlink(body_file_name);
                 }
		
		g_free(body_file_name);
		g_free(header_file_name);
		if (engine_errno == ENGINE_SIZE_ERROR) {
			IMAP_SET_ERROR_CB(IMAP_MSG_SIZE_EXCEED_ERROR);
		} else if (ok != IMAP_SUCCESS) {
			if(!imap_is_session_cancelled())
				IMAP_CB(IMAP_MSG_FETCH_ERROR);
		} else {
			if(!imap_is_session_cancelled()){
				bodydownloaded = TRUE;
				if ((header->flags.perm_flags & MSG_ATTACH) == 0) {
					fullydownloaded = TRUE;
				} else {
					fullydownloaded = header->msgtag.fullydownloaded;
				}
			}
		}
		procmsg_msgheader_free(header);
		procmime_mimeinfo_free_all(mime_info);

	} else {
		ok = imap_cmd_fetch(IMAP_SESSION(session), msgnum, filename);
		if (ok == IMAP_SUCCESS) {
			fullydownloaded = TRUE;
			bodydownloaded = TRUE;
		} else if (ok == IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER) {
			IMAP_SET_MSG_CB(IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER, msgid);
			imap_delete_header_file(msgid);
		}
	}
	g_free(filename);

	if (ok == IMAP_SUCCESS) {

		if (IMAP_HAS_CB) {
			if (gl_imap4_callback_ptr->imap4_set_msg_cb != NULL) {
				gl_imap4_callback_ptr->imap4_set_msg_cb(IMAP_MSG_DOWNLOAD,
									"%d%d%s",
									fullydownloaded,
									bodydownloaded, msgid);
			}
		}

	} else {
		if (close_session == TRUE) {
			IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
			imap_session_disconnect(session, FALSE);
		}
		return;

	}

	/* closes the session */
	if (close_session == TRUE) {
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
	}
	if (read_flag == FALSE) {
		if (folder_message_add_list(msgid, UNREAD) == FOLDER_SUCCESS) {
			ok = IMAP_SUCCESS;
		} else {
			ok = IMAP_ERROR;
		}
		/* add to the remote flag list if it is already not added */
		if (folder_is_msg_remote(msgid) == FALSE) {
			if (folder_add_list(msgid, TRUE) == FOLDER_SUCCESS) {
				ok = IMAP_SUCCESS;
			} else {
				ok = IMAP_ERROR;
			}
		}

	}
	if (ok == IMAP_SUCCESS) {
		IMAP_CB(IMAP_SUCCESS);
	}
	return;

}				//end of imap_cmd_fetch_msg_body function


/** 
  This function disconnects a session with the IMAP server.
  @param session   pointer to the Session object created
  @return none
*/

void imap_session_disconnect(Session * session, gboolean flag)
{
	IMAPSession *imap_session = IMAP_SESSION(session);
	g_return_if_fail((imap_session != NULL) && (imap_session->ac_prefs != NULL));
	ACCNT_RESET_FLAGS(imap_session->ac_prefs, ACCNT_RX);
	if (imap_session->state == TRUE) {
		imap_cmd_logout(imap_session);
	}
	imap_session->ac_prefs->user_settings.session = NULL;
	session_destroy(session);
	gl_imap_recv_cancel = FALSE;
	gl_cancel_flag = FALSE;
	gl_current_account = NULL;
}

/** 
  This function fetches the message headers  from the IMAP server 
  called by the imap_get_headers .
  @param session    pointer to the Session object created
  @param prefs_account    pointer to the account name 	 
  @param folder    name of the folder 
  @param first_uid    first uid to be fetched
  @param last_uid last    uid to be fetched
  @param count    count of uids to be fetched
  @return	pointer to the GSList object created with the MsgInfo structure
*/

static GSList *imap_get_uncached_msgs(IMAPSession * session,
				      PrefsAccount * prefs_account, const gchar * folder,
				      guint32 first_uid, guint32 * last_uid, gint * count)
{
	gchar *tmp = NULL;
	gchar seq_set[IMAP_SEQ_SIZE], *tmp_msgid = NULL;
	GSList *newlist = NULL;
	GString *str = NULL;
	MsgInfo *msginfo = NULL;
	gint len = 0;
	ProgData *data_ptr = NULL;
	gint msg_num = 0;

	/* cache changes */
	gchar *tmp_file = NULL;
	gchar *header_path = NULL;
	gchar *file_path = NULL;

	CHECK_RETURN_VAL_IF_FAIL(session != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(prefs_account != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(folder != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(first_uid <= *last_uid, NULL);
	CHECK_RETURN_VAL_IF_FAIL((first_uid >= 0)
				 && (*last_uid >= 0), NULL);

	/* if first uid and last uid not specified all the messages 
	   in the INBOX will be fetched else only specified messages fetched */
	if (first_uid == 0 && *last_uid == 0) {
		strcpy(seq_set, "1:*");
	} else if (*last_uid == first_uid) {
		len = get_num_len(*last_uid);
		g_snprintf(seq_set, len + 1, "%u", *last_uid);
	} else {
		g_snprintf(seq_set, sizeof(seq_set), "%u:%u", first_uid, *last_uid);
	}
	if (imap_cmd_envelope(session, seq_set) != IMAP_SUCCESS) {
		log_message("can't get envelope\n");
		return NULL;
	}
	str = g_string_new(NULL);

	if (prefs_account->receive_settings.recv_type == HEADERS) {
		data_ptr = g_new0(ProgData, 1);
		data_ptr->cur = 1;
		data_ptr->total = *count;
		g_timeout_add(200, progress_func, data_ptr);
		gl_stop_timer = FALSE;
	}

	for (;;) {
		if ((tmp = sock_getline(SESSION(session)->sock)) == NULL) {
			log_message("error occurred while getting envelope.\n");
			g_string_free(str, TRUE);
			return newlist;
		}
		strretchomp(tmp);
		if (tmp[0] != '*' || tmp[1] != ' ') {
			log_message("IMAP4< %s\n", tmp);
			g_free(tmp);
			break;
		}
		if (strstr(tmp, "FETCH") == NULL) {
			log_message("IMAP4< %s\n", tmp);
			g_free(tmp);
			continue;
		}
		log_message("IMAP4< %s\n", tmp);
		g_string_assign(str, tmp);
		g_free(tmp);

		/*cache changes */
		tmp_file = get_tmp_file();

		/* parses the cmd envelope response */
		msginfo = imap_parse_env(session, str, tmp_file);
		if (get_engine_errno() == ENGINE_ERROR_DEVICE_MEMORY_FULL) {
			procmsg_msginfo_free(msginfo);
			break;
		}
		if (!msginfo) {
			log_message("can't parse envelope: %s\n", str->str);
			continue;
		}
		if (msginfo->header->flags.perm_flags & MSG_DELETED) {
			procmsg_msginfo_free(msginfo);
			continue;
		}
		tmp_msgid = g_strdup_printf("%s%c%s%c%d",
					    prefs_account->user_settings.
					    account_name, MSGID_SEPARATOR, folder,
					    MSGID_SEPARATOR, msginfo->header->msgnum);


		if (imap_is_msg_exist(tmp_msgid) == TRUE) {
			procmsg_msginfo_free(msginfo);
			g_free(tmp_msgid);
			continue;
		}

		if (prefs_account->receive_settings.recv_type == HEADERS) {
			if (data_ptr->cur < *count) {
				data_ptr->cur++;
			}
		}
		g_free(msginfo->header->msgid);
		msginfo->header->msgid = g_strdup(tmp_msgid);
		msginfo->header->account_name = g_strdup(prefs_account->user_settings.account_name);

		file_path = folder_get_message_file_path(tmp_msgid);
		header_path = g_strdup_printf("%s%c%s", file_path, DOT, HEAD);
		move_file(tmp_file, header_path, TRUE);

		if ((msginfo->header->flags.perm_flags & MSG_UNREAD) != 0) {
			folder_message_add_list(msginfo->header->msgid, UNREAD);
		}
		if (msginfo->header->msgid != NULL) {
			imap_set_msginfo_fields(prefs_account, msginfo);
			msg_num = get_msgid_num(msginfo->header->msgid);
		}
		newlist = g_slist_append(newlist, msginfo);
		g_free(tmp_msgid);
		g_free(tmp_file);
		g_free(header_path);
		g_free(file_path);

		(void) g_main_context_iteration(NULL, FALSE);

                // Check if someone closed the connection while retrieving messages
                if (prefs_account->user_settings.session == NULL) {
                        break;
                }

		if (gl_imap_recv_cancel == TRUE) {
			gl_stop_timer = TRUE;
			*last_uid = msg_num;
			break;
		}

	}
	if (prefs_account->receive_settings.recv_type == HEADERS) {
		/* send last progress cb */
		IMAP_PROGRESS_CB(data_ptr->cur, data_ptr->total);
		gl_stop_timer = TRUE;
		g_string_free(str, TRUE);
		return newlist;
	}
	g_string_free(str, TRUE);

	return newlist;
}

/** 
  This function creates a session with the IMAP server.
  @param account    pointer to the account information 
  @param msgid 
  @return  piointer to the Session object created.
  This is to be detsroyed by the caller.
*/

static Session *imap_session_new(PrefsAccount * account, const gchar * msgid)
{
	IMAPSession *session = NULL;
	SockInfo *sock = NULL;
	gint port = 0;
	gchar *argbuf = NULL;
	gint ok = 0;

#if USE_SSL
	SSLType ssl_type = SSL_NONE;
#endif
	CHECK_RETURN_VAL_IF_FAIL(account != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(account->receive_settings.recv_server != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(account->user_settings.userid != NULL, NULL);
	gl_imap_recv_cancel = FALSE;
	gl_cancel_flag = FALSE;
	gl_current_account = account;

	IMAP_PROGRESS_CB(0, 0);
	if (msgid == NULL) {
		IMAP_SET_MSG_CB(IMAP_START, account->user_settings.account_name);
	} else {
		IMAP_SET_MSG_STR_CB(IMAP_START, account->user_settings.account_name, msgid);
	}
#if CL_USE
	debug_print(" Recv IMAP Mail through iap \n");
	if (cl_iap_connect(account->advanced_settings.iap_name, not_mandatory_to_conn_serv) != CL_OK) {
		debug_print("CL connection Error in imap \n");
		IMAP_SET_MSG_CB(IMAP_COMPLETE, account->user_settings.account_name);
		return NULL;
	}
	if (gl_cancel_flag == TRUE) {
		return NULL;
	}
#endif

	if (account->user_settings.session != NULL) {
		/* FIX Disconnection problem */
		if (imap_cmd_noop(IMAP_SESSION(account->user_settings.session))
		    == IMAP_SUCCESS) {

			if (msgid == NULL) {
				IMAP_SET_MSG_CB(IMAP_START, account->user_settings.account_name);
			} else {
				IMAP_SET_MSG_STR_CB(IMAP_START,
						    account->user_settings.account_name, msgid);

			}
			return account->user_settings.session;
		} else {
			imap_session_disconnect(account->user_settings.session, TRUE);
			account->user_settings.session = NULL;
		}
		/* FIX Disconnection problem */
	}
#if USE_SSL
	if (account->advanced_settings.use_ssl) {

		if (account->advanced_settings.use_tls == TRUE) {
			ssl_type = SSL_STARTTLS;	// STARTTLS

		} else {
			ssl_type = SSL_TUNNEL;	// TUNNEL

		}

	} else {
		if (account->receive_settings.port == 0) {
			account->receive_settings.port = IMAP4_PORT;
		}

	}
#else
	if (account->receive_settings.port == 0) {
		account->receive_settings.port = IMAP4_PORT;
	}
#endif

	port = account->receive_settings.port;
	log_message("creating IMAP4 connection to %s:%d ...\n",
		    account->receive_settings.recv_server, account->receive_settings.port);

/* opens the socket */
#if USE_SSL
	if ((sock = imap_open(account->receive_settings.recv_server, port,
			      ssl_type, account->user_settings.account_name)) == NULL) {
		return NULL;
	}
#else
	if ((sock = imap_open(account->receive_settings.recv_server,
			      port, account->user_settings.account_name)) == NULL) {
		return NULL;

	}
#endif

	session = g_new0(IMAPSession, 1);
	if (session == NULL) {
		return NULL;
	}
	/* initializes the session as a IMAP session */
	session_init(SESSION(session));

	SESSION(session)->type = SESSION_IMAP;
	SESSION(session)->server = g_strdup(account->receive_settings.recv_server);
	SESSION(session)->sock = sock;
	SESSION(session)->last_access_time = time(NULL);
	SESSION(session)->data = NULL;

	SESSION(session)->destroy = imap_session_destroy;

	session->authenticated = FALSE;
	session->mbox = NULL;
	session->cmd_count = 0;
	session->ac_prefs = account;
	session->state = FALSE;

	/* recieves the greeting sequence */
	if (imap_greeting(session) != IMAP_SUCCESS) {
		imap_session_disconnect(SESSION(session), FALSE);
		IMAP_SET_MSG_STR_CB(IMAP_CONNECT_SERVER_ERROR,
				    account->receive_settings.recv_server,
				    account->user_settings.account_name);
		IMAP_SET_MSG_CB(IMAP_COMPLETE, account->user_settings.account_name);
		return NULL;
	}
	/* session is now connected */
	session->state = TRUE;
	/* polls the capability of the server */
	if (imap_cmd_capability(session) != IMAP_SUCCESS) {
		IMAP_SET_MSG_CB(IMAP_COMPLETE, account->user_settings.account_name);
		imap_session_disconnect(SESSION(session), FALSE);
		IMAP_CB(IMAP_ERROR);
		return NULL;
	}
	/* checks the capability of the session is same or not? */
	if (imap_has_capability(session, "UIDPLUS"))
		session->uidplus = TRUE;

#if USE_SSL
	if (account->advanced_settings.use_ssl && (ssl_type == SSL_STARTTLS)) {
		if (imap_has_capability(session, "STARTTLS")) {

			ok = imap_cmd_starttls(session);
			if (ok != IMAP_SUCCESS) {
				debug_print("Can't start TLS session.\n");
				IMAP_SET_MSG_CB(IMAP_COMPLETE, account->user_settings.account_name);
				imap_session_disconnect(SESSION(session), FALSE);
				IMAP_CB(IMAP_STARTTLS_ERROR);
				return NULL;
			}
			gint ret = 0;
			ret = ssl_init_socket_with_method(sock, SSL_METHOD_TLSv1);
			if (ret != SSL_SUCCESS) {
				IMAP_SET_MSG_CB(IMAP_COMPLETE, account->user_settings.account_name);
				imap_session_disconnect(SESSION(session), FALSE);
				IMAP_CB(ret);
				return NULL;
			}
		} else {
			IMAP_CB(IMAP_STARTTLS_ERROR);
			return NULL;
		}
	}
#endif

	/*check for session authentication */
	if (!session->authenticated &&
	    (ok = imap_auth(session, account->user_settings.userid,
			    account->user_settings.passwd,
			    account->receive_settings.passwd_auth_type, FALSE)) != IMAP_SUCCESS) {
		if (ok != IMAP_PASSWD_FAIL) {
			IMAP_SET_MSG_STR_CB(((ok==IMAP_SOCKET)?ok:IMAP_AUTHFAIL),
					    account->receive_settings.recv_server,
					    account->user_settings.account_name);
		}
		IMAP_SET_MSG_CB(IMAP_COMPLETE, account->user_settings.account_name);
		imap_session_disconnect(SESSION(session), FALSE);
		return NULL;
	}
	if (imap_has_capability(session, "NAMESPACE")) {
		if (imap_cmd_namespace(session, &argbuf) != IMAP_SUCCESS) {
			g_free(argbuf);
			IMAP_CB(IMAP_ERROR);
			return NULL;
		}
		g_free(argbuf);
	}
	account->user_settings.session = SESSION(session);
	return SESSION(session);
}				// end of imap_session_new


/** 
  This function parses the string and populates the MsgInfo structure.
  @param line_str	imap command envelope response string 
  @param session	Session structure
  @param header_path    path of the header file
  @return	pointer to the MsgInfo object created
*/

static MsgInfo *imap_parse_env(IMAPSession * session, GString * line_str, gchar * header_path)
{
	gchar buf[IMAPBUFSIZE];
	MsgInfo *msginfo = NULL;
	gchar *cur_pos = NULL;
	gint msgnum = 0;
	guint32 uid = 0;
	size_t size = 0;
	MsgFlags flags = { 0, 0 }, imap_flags = {
	0, 0};
	gchar *headers = NULL;

	/*cache changes */
	FILE *fp = NULL;

	memset(buf, '\0', IMAPBUFSIZE);

	CHECK_RETURN_VAL_IF_FAIL(header_path != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(line_str != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(line_str->str[0] == '*' && line_str->str[1] == ' ', NULL);

	cur_pos = line_str->str + 2;


	PARSE_ONE_ELEMENT(' ');
	msgnum = atoi(buf);

	PARSE_ONE_ELEMENT(' ');
	CHECK_RETURN_VAL_IF_FAIL(!strcmp(buf, "FETCH"), NULL);

	CHECK_RETURN_VAL_IF_FAIL(*cur_pos == '(', NULL);
	cur_pos++;

	while (*cur_pos != '\0' && *cur_pos != ')') {
		while (*cur_pos == ' ') {
			cur_pos++;
		}

		if (!strncmp(cur_pos, "UID ", 4)) {
			cur_pos += 4;
			uid = strtoul(cur_pos, &cur_pos, 10);
		} else if (!strncmp(cur_pos, "FLAGS ", 6)) {
			cur_pos += 6;
			if (*cur_pos != '(') {
				log_message("*cur_pos != '('\n");
				procmsg_msginfo_free(msginfo);
				return NULL;
			}
			cur_pos++;
			PARSE_ONE_ELEMENT(')');
			imap_flags = imap_parse_flags(buf);
		} else if (!strncmp(cur_pos, "RFC822.SIZE ", 12)) {
			cur_pos += 12;
			size = strtol(cur_pos, &cur_pos, 10);
		} else if (!strncmp(cur_pos, "RFC822.HEADER ", 14)) {

			cur_pos += 14;
			/* move the pointer to the header info in the response */
			cur_pos = imap_get_header(session, cur_pos, &headers, line_str);

			msginfo = procheader_parse_str(headers, flags, TRUE);

			/*cache changes */
			fp = fopen(header_path, "w+");
			if (fp == NULL) {
				debug_print("Unable to open header file\n");
				send_engine_error_to_ui(errno);
				imap_recv_cancel();
				return NULL;
			}
			fputs(headers, fp);
			fclose(fp);
			g_free(headers);
		} else {
			g_warning("invalid FETCH response: %s\n", cur_pos);
			break;
		}
	}

	if (msginfo) {
		msginfo->header->msgnum = uid;
		msginfo->header->size = size;
		msginfo->header->flags.tmp_flags |= imap_flags.tmp_flags;
		msginfo->header->flags.perm_flags |= imap_flags.perm_flags;
	}

	return msginfo;
}

/**
  This function fetches the particular part of the mail
  @param prefs_account    pointer to the prefs_account
  @param msgid    message id
  @param section_number    section number of the message to fetch 
  @param size_limit_check    flag indicates whether size limit check 
  has to be done 
  @param session    pointer to the session if session is already open. 
  @param mimeinfo    pointer to the mimeinformation  
  @param close_session    indicates whther the session has to be 
  closed after fetching.  
  @return none
*/

gint
imap_fetch_msg_attachment(PrefsAccount * prefs_account, const gchar * msgid,
			  const gchar * section_number,
			  gboolean size_limit_check,
			  Session * session, MimeInfo * mimeinfo, gboolean close_session)
{
	gint exists = 0, unseen = 0, recent = 0, uid_validity = 0;
	gint ok = IMAP_SUCCESS;
	gint msgnum = 0;
	gchar *header_file_name = NULL, *attach_file_name = NULL;
	gchar *tmp_file_path = NULL;
	MsgFlags imap_flags = { 0, 0 };

	if ((msgid == NULL) || (prefs_account == NULL)
	    || (section_number == NULL)) {
		IMAP_CB(IMAP_PARAMETER_MISSING);
		return IMAP_FAIL;
	}
	if ((check_msgid(msgid) == FALSE)) {
		IMAP_CB(IMAP_INVALID_MSGID_ERROR);
		return IMAP_FAIL;
	}
	msgnum = get_msgid_num(msgid);

	if (session == NULL) {
		/* Creates a Session */
		session = imap_session_new(prefs_account, msgid);

		if (session == NULL) {
			engine_errno = ENGINE_ERROR_CONNECT_SERVER;
			return IMAP_FAIL;
		} else {
			IMAP_SET_MSG_STR_CB(IMAP_CONNECT_SERVER_SUCCESS,
					    prefs_account->user_settings.account_name, msgid);
		}
		/* selects the inbox in read only mode & fetches the 
		 * inbox status. */

		if ((imap_cmd_do_uid_select(IMAP_SESSION(session), INBOX_DIR,
					    FALSE, &exists, &unseen, &recent,
					    &uid_validity, prefs_account, TRUE)) != IMAP_SUCCESS) {
			if (close_session == TRUE) {
				IMAP_SET_MSG_CB(IMAP_COMPLETE,
						prefs_account->user_settings.account_name);
				imap_session_disconnect(session, FALSE);
			}
			return IMAP_FAIL;
		}
		ok = imap_cmd_flags(IMAP_SESSION(session), msgnum, &imap_flags);

		if ((ok != IMAP_SUCCESS)
		    || (imap_flags.perm_flags & MSG_DELETED)) {

			if (size_limit_check == TRUE) {
				if(!imap_is_session_cancelled())
					IMAP_SET_MSG_CB(IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER, msgid);
				imap_delete_header_file(msgid);
			} else {
				if(!imap_is_session_cancelled())
					IMAP_SET_MSG_CB(IMAP_EMAIL_ATTACH_NOT_AVAILABLE_ERROR, msgid);
			}
			if (close_session == TRUE) {
				imap_session_disconnect(session, FALSE);
				IMAP_SET_MSG_CB(IMAP_COMPLETE,
						prefs_account->user_settings.account_name);
			}
			return IMAP_FAIL;
		}

	}
	tmp_file_path = folder_get_message_file_path(msgid);

	header_file_name = g_strdup_printf("%s%c%s", tmp_file_path, SECTION_SEPARATOR, HEAD);
	g_free(tmp_file_path);
	if (!is_file_exist(header_file_name)) {
		ok = imap_cmd_fetch_header(IMAP_SESSION(session), msgnum, header_file_name);

		if (ok != IMAP_SUCCESS) {
			g_free(header_file_name);
			if (size_limit_check == TRUE) {
                     		if(!imap_is_session_cancelled())
					IMAP_SET_MSG_CB(IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER, msgid);
				imap_delete_header_file(msgid);
			} else {
                     		if(!imap_is_session_cancelled())
					IMAP_SET_MSG_CB(IMAP_EMAIL_ATTACH_NOT_AVAILABLE_ERROR, msgid);
			}

			if (close_session == TRUE) {
				IMAP_SET_MSG_CB(IMAP_COMPLETE,
						prefs_account->user_settings.account_name);
				imap_session_disconnect(session, FALSE);
			}
			return IMAP_FAIL;
		}
	}
	g_free(header_file_name);
	tmp_file_path = folder_get_message_file_path(msgid);

	attach_file_name = g_strdup_printf("%s%c%s",
					   tmp_file_path, SECTION_SEPARATOR, section_number);
	g_free(tmp_file_path);

	if (size_limit_check == TRUE) {

		ok = imap_download_body_check(session, mimeinfo, msgid,
					      prefs_account, msgnum, section_number,
					      attach_file_name);

	} else {

		ok = imap_cmd_fetch_attach(IMAP_SESSION(session), msgnum,
					   attach_file_name, (gchar *) section_number);
		if (ok == IMAP_ERROR && !imap_is_session_cancelled()) {
			IMAP_SET_MSG_CB(IMAP_MSG_FETCH_ERROR, msgid);
		}
	}
	g_free(attach_file_name);

	if (ok != IMAP_SUCCESS) {
		if (close_session == TRUE) {
			IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
			imap_session_disconnect(session, FALSE);
		}
		return IMAP_FAIL;
	}

	/* closes the session */
	if (close_session == TRUE) {
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
	}

	if (size_limit_check == TRUE) {
		/* Bug Fix For #42191 */
		
        	if(is_message_unread((gchar *)msgid)) {
	        	if (folder_message_add_list((gchar *) msgid, UNREAD) == FOLDER_SUCCESS) {
			        ok = IMAP_SUCCESS;
		        } else {
			        ok = IMAP_FAIL;
		        }
	       }

		if (folder_is_msg_remote((gchar *) msgid) == FALSE) {
			/* add into remote list if not added already */
			if (folder_add_list((gchar *) msgid, TRUE) == FOLDER_SUCCESS) {
				ok = IMAP_SUCCESS;
			} else {
				ok = IMAP_FAIL;
			}
		}

		if (IMAP_HAS_CB) {
			if (gl_imap4_callback_ptr->imap4_set_msg_cb != NULL) {
				gl_imap4_callback_ptr->imap4_set_msg_cb(IMAP_MSG_DOWNLOAD,
									"%d%d%s",
									FALSE, TRUE, msgid);
			}
		}
	}

	return ok;
}

/**
  This function fetches mimeinfo of  the mail
  @param prefs_account    pointer to the account information
  @param msgid    message id
  @param section_number   section number of the part   
  @param file_name  name of the file,to store the info
  @return IMAP_SUCCESS(success) or IMAP error code 
*/
gint
imap_fetch_mime_header(PrefsAccount * prefs_account,
		       const gchar * msgid, gchar * section_number, gchar * file_name)
{

	Session *session = NULL;
	gint exists = 0, unseen = 0, recent = 0, uid_validity = 0;
	gint ok = 0, len = 0;
	gint msgnum = 0;
	gchar *tmp = NULL, *temp_last = NULL, *temp_uid = NULL;
	gchar *temp = NULL;

	if ((msgid == NULL) || (prefs_account == NULL)
	    || (section_number == NULL)) {
		engine_errno = IMAP_PARAMETER_MISSING;
		return IMAP_ERROR;
	}
	if ((check_msgid(msgid) == FALSE)) {
		engine_errno = IMAP_INVALID_MSGID_ERROR;
		return IMAP_ERROR;
	}
/* Creates a Session */
	session = imap_session_new(prefs_account, msgid);

	if (session == NULL) {
		engine_errno = ENGINE_ERROR_CONNECT_SERVER;
		return IMAP_CONNECT_SERVER_ERROR;
	} else {

		IMAP_SET_MSG_STR_CB(IMAP_CONNECT_SERVER_SUCCESS,
				    prefs_account->user_settings.account_name, msgid);
	}

	/* selects the inbox in read only mode & fetches the inbox status. */
	if ((imap_cmd_do_select(IMAP_SESSION(session), INBOX_DIR,
				FALSE, &exists, &unseen, &recent, &uid_validity)) == IMAP_SUCCESS) {

		/* converts the latest seq_number in to the UID
		   by fetching the uids from the server */

		tmp = g_strdup_printf("FETCH %d (UID)", exists);

		gl_msgs_exists_server = exists;
		imap_cmd_gen_send(IMAP_SESSION(session), tmp);
		g_free(tmp);

		while ((imap_cmd_gen_recv(IMAP_SESSION(session), &tmp) == IMAP_SUCCESS)) {

			if (tmp[0] != '*' || tmp[1] != ' ') {
				g_free(tmp);
				break;
			}
			temp_last = strchr(tmp, 'D');
			if(temp_last == NULL) {
				g_free(tmp);
				break;
			}
			len = (strlen(tmp) - (temp_last - tmp) - 3);
			if((len >0) && (len < strlen(tmp))) {
				temp_uid = g_strndup(temp_last + 2, len);
				gl_latest_uid = atoi(temp_uid);
				g_free(temp_uid);
			}
			g_free(tmp);
		}
	} else {
		engine_errno = IMAP_INBOX_SELECT_ERROR;
		return IMAP_INBOX_SELECT_ERROR;
	}

	temp = strrchr(msgid, MSGID_SEPARATOR);
	msgnum = atoi(temp + 1);
	ok = imap_cmd_mime_header(IMAP_SESSION(session), msgnum, file_name, section_number);
	/* closes the session */
	IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
	imap_session_disconnect(session, FALSE);
	engine_errno = ok;
	return ok;

}


/** 
  This function sends the logout imap command  to the imap server 
  @param prefs_account    pointer to the account information 
  @param msgid    message id
  @param session_ptr   (OUT) pointer to the session 
  @return mime information of the message.This has to be freed by the 
  calling function. 
*/
MimeInfo *imap_fetch_msg_body_structure(PrefsAccount * prefs_account,
					const gchar * msgid, Session ** session_ptr)
{

	MimeInfo *mime_info = NULL;
	Session *session = NULL;
	gint exists = 0, unseen = 0, recent = 0, uid_validity = 0, ok = IMAP_FAIL;
	gchar *tmp_str = NULL;
	gchar *mime_file_name = NULL;
	FILE *mime_fp = NULL;
	gint msgnum = 0;
	MsgFlags imap_flags = { 0, 0 };

	CHECK_RETURN_VAL_IF_FAIL(msgid != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(prefs_account != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(check_msgid(msgid), NULL);

	if (session_ptr != NULL) {
		session = *session_ptr;
	}
	if (session == NULL) {
		/* Creates a Session */
		session = imap_session_new(prefs_account, msgid);

		if (session == NULL) {
			log_message("not able to connect %s  server at %d port",
				    prefs_account->receive_settings.recv_server,
				    prefs_account->receive_settings.port);
			engine_errno = ENGINE_ERROR_CONNECT_SERVER;

			return NULL;
		} else {

			IMAP_SET_MSG_STR_CB(IMAP_CONNECT_SERVER_SUCCESS,
					    prefs_account->user_settings.account_name, msgid);
		}
	}
	if (session_ptr != NULL) {
		*session_ptr = session;
	}

	/* selects the inbox in read only mode & fetches the inbox status. */

	if ((imap_cmd_do_uid_select(IMAP_SESSION(session), INBOX_DIR,
				    FALSE, &exists, &unseen, &recent,
				    &uid_validity, prefs_account, TRUE)) != IMAP_SUCCESS) {
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		*session_ptr = NULL;
		return NULL;
	}
	msgnum = get_msgid_num(msgid);
	ok = imap_cmd_flags(IMAP_SESSION(session), msgnum, &imap_flags);

	if ((ok == IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER)
	    || (imap_flags.perm_flags & MSG_DELETED)) {
		IMAP_SET_MSG_CB(IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER, msgid);
		imap_delete_header_file(msgid);
		imap_session_disconnect(session, FALSE);
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		*session_ptr = NULL;
		return NULL;
	}
	tmp_str = imap_cmd_body_structure(IMAP_SESSION(session), msgid);
	if (tmp_str == NULL) {
		IMAP_SET_MSG_CB(IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER, msgid);
		imap_delete_header_file(msgid);
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		*session_ptr = NULL;
		return NULL;
	}

	if ((strstr(tmp_str, "BAD")) != NULL) {
		g_free(tmp_str);
		IMAP_CB(IMAP_SERVER_NOT_SUPPORT_ERROR);
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		*session_ptr = NULL;
		return NULL;
	}
	mime_info = (MimeInfo *) procmime_scan_body(tmp_str, strlen(tmp_str));
	/* save it to file */
	mime_file_name = g_strdup_printf("%s%c%s%c%s%c%s",
					 get_mail_dir(),
					 G_DIR_SEPARATOR, INBOX_DIR,
					 G_DIR_SEPARATOR, msgid, SECTION_SEPARATOR, MIME_SUFFIX);

	mime_fp = fopen(mime_file_name, "w");
	if (mime_fp != NULL) {
		if (fwrite(tmp_str, strlen(tmp_str), 1, mime_fp) < 0) {
			send_engine_error_to_ui(errno);
			imap_recv_cancel();
			fclose(mime_fp);
			unlink(mime_file_name);
		} else {
			fclose(mime_fp);
		}
	}

	g_free(mime_file_name);
	g_free(tmp_str);
	return mime_info;
}

/**
  This function return the size of the string
  @param str    string to get size
  @return length of the string
*/
gint return_size(gchar * str)
{
	gchar *pch = NULL, *tmp = NULL;
	gint num = 0;
	pch = strtok(str, " ");
	while (pch != NULL) {
		pch = strtok(NULL, " ,),(,.");
		if ((tmp != NULL) && (!strcmp("SIZE", tmp))) {
			num = atoi(pch);
			g_free(tmp);
			break;
		} else {
			g_free(tmp);
			tmp = g_strdup(pch);
		}
	}
	debug_print("[%s : %d ] num :%d", __FILE__, __LINE__, num);
	return num;
}

/**
  This function fetches the body structure of the message
  @param session    pointer to the session
  @param msgid   message id
  @return body structure string
*/

static gchar *imap_cmd_body_structure(IMAPSession * session, const gchar * msgid)
{
	gchar *tmp = NULL;
	gchar *tmp1 = NULL;
	gchar *return_str = NULL;
	gint msgnum = 0;

	CHECK_RETURN_VAL_IF_FAIL(session != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(msgid != NULL, NULL);
	if (gl_cancel_flag == TRUE) {
		return NULL;
	}

	msgnum = get_msgid_num(msgid);

	tmp = g_strdup_printf(IMAP_BODYSTR_CMD, msgnum);

	imap_cmd_gen_send(session, tmp);

	g_free(tmp);

	while ((imap_cmd_gen_recv(session, &tmp) == IMAP_SUCCESS)) {
		if (g_strrstr(tmp, "completed") != NULL || g_strrstr(tmp, "Completed") != NULL) {
			g_free(tmp);
			break;
		}
		if (return_str == NULL) {
			return_str = g_strndup(tmp, strlen(tmp));
		} else {
			tmp1 = g_strdup_printf("%s%s", return_str, tmp);
			g_free(return_str);
			return_str = NULL;
			return_str = g_strndup(tmp1, strlen(tmp1));
			g_free(tmp1);
		}
		g_free(tmp);
	}
	return return_str;
}

/** 
  This function sends the fetch imap command  to the imap server 
  for the header of the message
  @param session    pointer to the IMAPSession object created
  @param uid     UID message number
  @param filename     file name where the message is to be stored
  @return  success(IMAP_SUCCESS) or error code
*/
static gint imap_cmd_fetch_header(IMAPSession * session, guint32 uid, const gchar * filename)
{
	gint ok = 0;
	gchar *buf = NULL;
	gchar *cur_pos = NULL;
	gchar size_str[IMAP_CMD_SIZE];
	glong size_num = 0;

	CHECK_RETURN_VAL_IF_FAIL(filename != NULL, IMAP_ERROR);
	memset(size_str, '\0', IMAP_CMD_SIZE);

	imap_cmd_gen_send(session, IMAP_PEEK_HEADER_CMD, uid);

	while ((ok = imap_cmd_gen_recv(session, &buf)) == IMAP_SUCCESS) {

		if (buf[0] != ASTERISK || buf[1] != SPACE_CHAR) {
			g_free(buf);
			return IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER;
		}
		if (strstr(buf, IMAP_FETCH_CMD) != NULL) {
			break;
		}
		g_free(buf);
	}
	if (ok != IMAP_SUCCESS) {
		return ok;
	}
	cur_pos = strchr(buf, IMAP_CURLY_START);
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(cur_pos != NULL);
	cur_pos = strchr_cpy(cur_pos + 1, IMAP_CURLY_END, size_str, sizeof(size_str));
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(cur_pos != NULL);
	size_num = atol(size_str);
	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(size_num >= 0);

	FREE_BUF_AND_RETURN_ERROR_IF_FAIL(*cur_pos == '\0');

	g_free(buf);
	if (recv_bytes_write_to_file(SESSION(session)->sock, size_num, filename) != 0) {
		return IMAP_ERROR;
	}
	if (imap_cmd_gen_recv(session, &buf) != IMAP_SUCCESS) {
		return IMAP_ERROR;
	}
	if (buf[0] == '\0' || buf[strlen(buf) - 1] != IMAP_END_BR) {
		g_free(buf);
		return IMAP_ERROR;
	}
	g_free(buf);

	ok = imap_cmd_ok(session, NULL);

	return ok;
}

/** 
  This function scans inbox present in the IMAP server about the message num.
  @param prefs_account  pointer to the account information
  @param status OUT parameter,pointer to the MsgSatus information returned 
  @return IMAP error code or success(IMAP_SUCCESS)
*/
gint imap_get_message_status(PrefsAccount * prefs_account, MsgStatus ** status)
{
	Session *session = NULL;
	gint exists = 0, unseen = 0, recent = 0, uid_validity = 0;
	gboolean msg_exists_flag = FALSE;
	gchar *tmp = NULL;

	CHECK_RETURN_VAL_IF_FAIL(status != NULL, IMAP_PARAMETER_MISSING);

	/* Creates a Session */
	session = imap_session_new(prefs_account, NULL);

	if (session == NULL) {
		log_message("not able to connect %s  server at %d port",
			    prefs_account->receive_settings.recv_server,
			    prefs_account->receive_settings.port);
		engine_errno = ENGINE_ERROR_CONNECT_SERVER;

		return IMAP_CONNECT_SERVER_ERROR;
	}
	/* selects the inbox in read only mode & fetches the inbox status. */
	if ((imap_cmd_do_select(IMAP_SESSION(session), INBOX_DIR,
				FALSE, &exists, &unseen, &recent, &uid_validity)) == IMAP_SUCCESS) {

		gl_msgs_exists_server = exists;

		tmp = g_strdup_printf("UID FETCH %d (flags)", (*status)->msgnum);

		imap_cmd_gen_send(IMAP_SESSION(session), tmp);
		g_free(tmp);

		msg_exists_flag = FALSE;
		while (imap_cmd_gen_recv(IMAP_SESSION(session), &tmp) == IMAP_SUCCESS) {
			debug_print("imap.c : tmp %s\n", tmp);
			if (tmp[0] != '*' || tmp[1] != ' ') {
				g_free(tmp);
				break;
			}
			msg_exists_flag = TRUE;
			if (strstr(tmp, "Seen") != NULL) {
				(*status)->msgflags.perm_flags &= FLAG_MSG_READ;
			}
			g_free(tmp);
		}

	} else {
		log_message("not able to select the inbox\n");
		return IMAP_INBOX_SELECT_ERROR;
	}
	/* closes the session */
	IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
	imap_session_disconnect(session, FALSE);
	if (msg_exists_flag == TRUE) {
		debug_print("msg_exists_flag : message exists in the server \n");
		return IMAP_SUCCESS;
	} else {
		return IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER;
	}
}

/** 
  This function prepares the attachment list from mime information
  @param mimeinfo    mime information to read.This is allocated and 
  freed by the calling function. 
  @param msgid    message id of file
  @return list of attachment information.
  This memory has to be freed by the calling function.
*/
static GSList *mimeinfo_to_attachlist(MimeInfo * mimeinfo, const gchar * msgid)
{
	GSList *attach_list = NULL;
	MimeInfo *cur = NULL;
	AttachInfo *attach_info = NULL;

	CHECK_RETURN_VAL_IF_FAIL(mimeinfo != NULL, NULL);
	CHECK_RETURN_VAL_IF_FAIL(msgid != NULL, NULL);

	for (cur = mimeinfo->children; cur != NULL; cur = cur->next) {
		attach_info = g_new0(AttachInfo, 1);
		if (attach_info == NULL) {
			return NULL;
		}
		attach_info->msgid = g_strdup(msgid);
		attach_info->attachmentname = g_strdup(cur->name);
		debug_print("msgid: %s \t attach_name : %s \n", msgid, attach_info->attachmentname);

		attach_list = g_slist_append(attach_list, attach_info);
		if (attach_list == NULL) {
			g_free(attach_info);
			return NULL;
		}
	}
	return attach_list;
}


/**
  This fucntion frees the call back function pointer
  @param none
  @return none
*/
void imap_cleanup(void)
{
	g_free(gl_imap4_callback_ptr);
	gl_imap4_callback_ptr = NULL;
}


/**
  This function checks for the size limit and downloads 
  whatever is within the limit
  @param prefs_account    pointer to the account info 
  @param session    pointer to the session info 
  @param header_file_name    file name to store header 
  @param body_file_name    file name to store the body of the message 
  @param mimeinfo    mimeinformation of the message 
  @param smime_check    indicates the message is smime 
  @return IMAP_SUCCESS or IMAP_FAIL 
*/
static gint
imap_msg_size_limit(PrefsAccount * prefs_account, Session * session,
		    gint uid_no, const gchar * header_file_name,
		    const gchar * body_file_name, MsgHeader * header,
		    const gchar * file_name, MimeInfo * mimeinfo, gboolean smime_check)
{
	gint size = 0;
	gint ret = IMAP_FAIL;

	size = imap_cmd_mail_size(session, uid_no);
	if (size != 0) {
		if (((float) size) / 1024 > (float) prefs_account->receive_settings.size_limit) {
			if (smime_check == TRUE) {
				return IMAP_FAIL;
			}
			UNSET_CB_FLAG;
			ret = download_imap_body(session, prefs_account,
						 header_file_name, body_file_name,
						 header, MSG_BODY_SEC, uid_no, file_name, mimeinfo);
			SET_CB_FLAG;
			return ret;
		}
	}
	ret = imap_cmd_fetch(IMAP_SESSION(session), uid_no, file_name);

	if (ret == IMAP_SUCCESS) {
		header->msgtag.fullydownloaded = TRUE;
	}
	return ret;
}

/**
  This function downloads body and header as separate files 
  @param prefs_account    pointer to the account info 
  @param session    pointer to the session info 
  @param uid_no    uid number of the message 
  @param header_file_name    header file name 
  @param body_file_name    file name of body 
  @param header    message header information 
  @param file_name    name of file to place received contents 
  @param mimeinfo    mimeinformation of the message 
  @return IMAP_SUCCESS or IMAP_FAIL 
*/
static gint
imap_get_body(PrefsAccount * prefs_account, Session * session,
	      gint uid_no, const gchar * header_file_name,
	      const gchar * body_file_name, MsgHeader * header,
	      const gchar * file_name, MimeInfo * mimeinfo)
{
	gint ret = IMAP_FAIL;
	gint ret_hd = IMAP_FAIL;
	MimeInfo *cur_mimeinfo = NULL;

	/* if file does not contain attachments,download the complete mail */
	if ((header->flags.perm_flags & MSG_ATTACH) == 0) {
		ret = imap_cmd_fetch(IMAP_SESSION(session), uid_no, file_name);
		return ret;
	}

	ret_hd = imap_cmd_fetch_attach(IMAP_SESSION(session), uid_no, header_file_name, HEAD);
	if (ret_hd != IMAP_SUCCESS) {
		return ret_hd;
	}
	for (cur_mimeinfo = mimeinfo; cur_mimeinfo != NULL; cur_mimeinfo = cur_mimeinfo->children) {

		if (cur_mimeinfo->content_type == NULL) {
			continue;
		}
		if ((g_ascii_strncasecmp(cur_mimeinfo->content_type,
					 MULTIPART_RELATED, strlen(MULTIPART_RELATED)) == 0)
		    ||
		    (g_ascii_strncasecmp
		     (cur_mimeinfo->content_type, MULTIPART_ALTERNATIVE,
		      strlen(MULTIPART_RELATED)) == 0)) {

			ret =
			    imap_get_multipart_body(IMAP_SESSION(session),
						    cur_mimeinfo, header->msgid, prefs_account);
			return ret;
		}
	}
	if (ret_hd == IMAP_SUCCESS) {
		ret =
		    imap_cmd_fetch_attach(IMAP_SESSION(session),
					  uid_no, body_file_name, MSG_BODY_SEC);

	}
	return ret;

}

/**
  This function sets message information on success 
  @param msg_info    pointer to the message information 
  @return IMAP_SUCCESS on success or IMAP error(IMAP_FAIL) 
*/
static gint imap_handle_success(MsgInfo * msg_info)
{
	gchar *folder_name = NULL;
	Folder *fold = NULL;

	if ((msg_info->header->flags.perm_flags & MSG_UNREAD) != 0) {
		if (folder_message_add_list(msg_info->header->msgid, UNREAD) != FOLDER_SUCCESS) {
			return IMAP_FAIL;
		}
	}

	/* if message is not already added into remote list add it now */
	if (folder_is_msg_remote(msg_info->header->msgid) == FALSE) {
		if (folder_add_list(msg_info->header->msgid, TRUE) != FOLDER_SUCCESS) {
			return IMAP_FAIL;
		}
	}
	folder_name = get_fold_name(msg_info->header->msgid);
	if (folder_name != NULL) {
		fold = folder_find_obj_by_name(folder_name);
		g_free(folder_name);
		if (fold == NULL) {
			return IMAP_FAIL;
		}
		msg_info->header->msgtag.foldertype = fold->type;
	}
	msg_info->header->msgtag.bodydownloaded = TRUE;

	/* if no attachments,then set as fullydownloaded */

	if ((msg_info->header->flags.perm_flags & MSG_ATTACH) == 0) {
		msg_info->header->msgtag.fullydownloaded = TRUE;
	}
	return IMAP_SUCCESS;
}

/**
  This function sets message information fields for all imap mails 
  @param prefs_account    pointer to the account information 
  @param msg_info    pointer to the message information 
  @return none 
*/
static void imap_set_msginfo_fields(PrefsAccount * prefs_account, MsgInfo * msg_info)
{
	msg_info->header->msgtag.protocol = PROTO_IMAP4;
	msg_info->header->msgtag.recvtype = prefs_account->receive_settings.recv_type;
	msg_info->header->msgtag.remote_copy = TRUE;
	folder_add_list(msg_info->header->msgid, TRUE);
}

/**
  This function sends the imap callback to indicate complete download
  @param none 
  @return none
*/
void imap_send_repopulate(void)
{
	IMAP_CB(IMAP_REPOP_ERROR);
}

/**
  This function sets the flag to indicate cancel operation
  @param none 
  @return none
*/
void imap_recv_cancel(void)
{
	sock_imap_cancel(TRUE, 1);	/*  for cancel support */
	gl_imap_recv_cancel = TRUE;
	gl_cancel_flag = TRUE;
}

/**
 This function fetches the inline messages parts appends to a file 
 along with mime information
 @param imap_session    pointer to the session information
 @param mimeinfo    pointer to the mime information
 @param fp    file pointer
 @param msgid    message id
 @param prefs_account    pointer to PrefsAccount
 @return success(IMAP_SUCCESS) or error code
*/
static gint
imap_get_inline_part(IMAPSession * imap_session,
		     MimeInfo * mimeinfo, FILE * fp,
		     const gchar * msgid, PrefsAccount * prefs_account)
{
	gchar *sub_file_name = NULL;
	gchar *tmp_file_name = NULL;
	gchar *filename = NULL;
	gint ok = IMAP_FAIL;

	if ((mimeinfo == NULL) || (fp == NULL) || (msgid == NULL)
	    || (prefs_account == NULL)) {
		return IMAP_FAIL;
	}
	if (mimeinfo->section_number == NULL) {
		return IMAP_FAIL;
	}
	if (strcmp(mimeinfo->section_number, HEADER_SECTION) != 0) {
		if (imap_write_mime_version(imap_session, fp, mimeinfo,
					    msgid, prefs_account) == IMAP_FAIL) {
			return IMAP_FAIL;
		}
		tmp_file_name = g_strdup_printf("%s%c%s%c%s",
						get_tmp_dir(), G_DIR_SEPARATOR,
						msgid, SECTION_SEPARATOR, mimeinfo->section_number);

		ok = imap_cmd_mime_header(imap_session, get_msgid_num(msgid),
					  tmp_file_name, mimeinfo->section_number);


		if (ok == IMAP_SUCCESS) {

			if (imap_file_append_attach_content(fp, tmp_file_name, FALSE)
			    == IMAP_FAIL) {
				g_free(tmp_file_name);
				return IMAP_FAIL;
			};
			g_free(tmp_file_name);

			/* get the body */
			ok = imap_get_part(imap_session, prefs_account, msgid, mimeinfo);

			if (ok == IMAP_FAIL) {
				return IMAP_FAIL;
			}

			GET_IMAP_MHTML_FILENAME(msgid, mimeinfo, sub_file_name);

			if (is_file_exist(sub_file_name) == FALSE) {
				g_free(sub_file_name);
				return IMAP_FAIL;
			}
			if (imap_file_append_attach_content(fp, sub_file_name, FALSE)
			    == IMAP_FAIL) {
				g_free(sub_file_name);
				return IMAP_FAIL;
			};
			g_free(sub_file_name);

		} else {
			g_free(tmp_file_name);
		}
	} else {

		imap_get_full_body(prefs_account, imap_session, msgid);

		filename = folder_get_message_file_path(msgid);

		if (imap_file_append_attach_content(fp, filename, TRUE) == IMAP_FAIL) {
			g_free(filename);
			unlink(filename);
			return IMAP_FAIL;
		};
		unlink(filename);
		g_free(filename);

	}
	return IMAP_SUCCESS;
}

/**
 This function reads a file contents and appends to another file
 @param fp    file pointer of the file to write 
 @param sub_file_name    name of file to read
 @param first_flag    indicates whether the body is the 
 first part of message 
 @return success(IMAP_SUCCESS) / error code
*/
static gint
imap_file_append_attach_content(FILE * fp, const gchar * sub_file_name, gboolean first_flag)
{
	FILE *sub_fp = NULL;
	guchar buf[BUFSIZ];
	size_t len = 0;

	memset(buf, '\0', BUFSIZ);

	if ((fp == NULL) || (sub_file_name == NULL)) {
		return IMAP_FAIL;
	}


	sub_fp = fopen(sub_file_name, "r+");
	if (sub_fp == NULL) {
		return IMAP_FAIL;
	}
	/* copy the remaining part of the file */
	while ((len = fread(buf, sizeof(gchar), sizeof(buf) - 1, sub_fp)) > 0) {
		fwrite(buf, strlen(buf), 1, fp);
		memset(buf, '\0', BUFSIZ);
	}
	fclose(sub_fp);
	return IMAP_SUCCESS;
}

/**
 This function reads mime version from the header file and writes to output
 @param imap_session    pointer to the imap session
 @param outfp    pointer to the output file
 @param mimeinfo    pointer to the mime information of file
 @param msgid    message id of the file 
 @param prefs_account    pointer to the account information 
 @return (IMAP_SUCCESS)success / error code
*/
static gint
imap_write_mime_version(IMAPSession * imap_session, FILE * outfp,
			MimeInfo * mimeinfo, const gchar * msgid, PrefsAccount * prefs_account)
{
	FILE *infp = NULL;
	guchar buf[BUFSIZ];
	gchar *infile_name = NULL;
	gint ok = IMAP_FAIL;

	memset(buf, '\0', BUFSIZ);

	infile_name = g_strdup_printf("%s%c%s%c%s%c%s",
				      get_mail_dir(), G_DIR_SEPARATOR,
				      INBOX_DIR, G_DIR_SEPARATOR,
				      msgid, SECTION_SEPARATOR, HEADER_SECTION);

	infp = fopen(infile_name, "r+");

	if (infp == NULL) {
		/* try to connect and get the header */
		ok = imap_cmd_mime_header(imap_session, get_msgid_num(msgid),
					  infile_name, mimeinfo->section_number);

		if (ok != IMAP_SUCCESS) {
                        
			g_free(infile_name);
			return IMAP_FAIL;
		}
	}
	procheader_copy_headers(infp, outfp);
	fclose(infp);
	g_free(infile_name);
	return IMAP_SUCCESS;

}


/**
 This function gets the multipart body of a message,
 which can be displayed in a browser
 @param imap_session    pointer to the session
 @param mimeinfo    pointer to the multipart,which has to be separated
 @param msgid   message id of the message
 @param account   account information pointer 
 @return IMAP_SUCCESS on success and IMAP_FAIL on failure 
*/
static gint
imap_get_multipart_body(IMAPSession * imap_session,
			MimeInfo * mimeinfo, const gchar * msgid, PrefsAccount * account)
{
	gchar *mhtml_file = NULL;
	FILE *outfp = NULL;
	gint ret_val = IMAP_FAIL;

	engine_errno = ENGINE_ERROR_DEFAULT;

	if (msgid == NULL) {
		engine_errno = ENGINE_ERROR_INVALID_PARAM;
		return IMAP_FAIL;
	}

	/* open the output file */
	mhtml_file = g_strdup_printf("%s%c%s%c%s%c%s",
				     get_mail_dir(), G_DIR_SEPARATOR,
				     INBOX_DIR, G_DIR_SEPARATOR,
				     msgid, SECTION_SEPARATOR, MHTML_EXTENSION);

	if (is_file_exist(mhtml_file) == TRUE) {
		g_free(mhtml_file);
		return IMAP_SUCCESS;
	}
	outfp = fopen(mhtml_file, "wb");

	if (outfp == NULL) {
		send_engine_error_to_ui(errno);
		imap_recv_cancel();
		g_free(mhtml_file);
		return IMAP_FAIL;
	}
	ret_val = imap_get_inline_part(imap_session, mimeinfo, outfp, msgid, account);

	/*error while writting to the .mime file, so unlink the file*/
	if(ret_val != IMAP_SUCCESS)
		unlink(mhtml_file);
		

	fclose(outfp);
	g_free(mhtml_file);
	return ret_val;
}

/**
 This function checks if the message has to be downloaded,
 for an already downloaded header and calls functions to download if needed. 
 @param msg_info    pointer to the multipart,which has to be separated
 @param session    pointer to the session
 @param prefs_account    pointer to the account information 
 @param mimeinfo    pointer to the mime information 
 @return (IMAP_SUCCESS)success / error code
*/
static gint
imap_download_msg(MsgInfo * msg_info, Session * session,
		  PrefsAccount * prefs_account, MimeInfo * mimeinfo)
{
	gint ret = IMAP_SUCCESS;
	gchar *file_name = NULL;
	gchar *body_file_name = NULL;
	gchar *header_file_name = NULL;

	engine_errno = ENGINE_ERROR_DEFAULT;

	file_name = folder_get_message_file_path(msg_info->header->msgid);
	body_file_name = imap_get_file_name(msg_info->header->msgid, MSG_BODY_SEC);
	header_file_name = imap_get_file_name(msg_info->header->msgid, HEAD);


	ret = imap_get_msg_for_hdr(msg_info, session, prefs_account,
				   header_file_name, body_file_name, file_name, mimeinfo);

	if (ret == IMAP_SUCCESS) {
		ret = imap_handle_success(msg_info);
		if (ret != IMAP_SUCCESS) {
			/* if error occurs,delete the downloaded file */
			if (is_file_entry_exist(body_file_name) == TRUE) {
				unlink(body_file_name);
			} else if (is_file_entry_exist(file_name) == TRUE) {
				unlink(file_name);
			}
			/* set the flags accordingly */
			msg_info->header->msgtag.fullydownloaded = FALSE;
			msg_info->header->msgtag.bodydownloaded = FALSE;
		}
	}
	g_free(file_name);
	g_free(body_file_name);
	g_free(header_file_name);
	return ret;

}

/**
 This function downloads body of a message or complete message,
 depending on size limit checks by calling relevent functions.
 @param msg_info    pointer to the multipart,which has to be separated
 @param session    pointer to the session
 @param prefs_account    pointer to the account information 
 @param header_file_name    name of file,to store header 
 @param body_file_name    name of file,to store body 
 @param file_name    name of file,to store entire message 
 @param mimeinfo    pointer to the mime information,read from file.
 This is allocated and freed by the calling function. 
 @return (IMAP_SUCCESS)success / error code
*/
static gint
imap_get_msg_for_hdr(MsgInfo * msg_info, Session * session,
		     PrefsAccount * prefs_account,
		     const gchar * header_file_name,
		     const gchar * body_file_name, const gchar * file_name, MimeInfo * mimeinfo)
{
	gint ret = IMAP_FAIL;
	gboolean size_limit_flag = FALSE;
	gint uid_no = 0;
	gint receive_type = 0;


	receive_type = prefs_account->receive_settings.recv_type;
	if ((msg_info->header->smime_type != SMIME_NONE)
	    && (receive_type == BODY))
		receive_type = BODY_ATTCH;

	uid_no = get_msgid_num(msg_info->header->msgid);

	if (prefs_account->receive_settings.enable_size_limit && CHECK_CB_FLAG) {

		size_limit_flag = TRUE;
	}

	if (receive_type == BODY_ATTCH) {

		if (size_limit_flag == TRUE) {
			ret =
			    imap_msg_size_limit(prefs_account, session,
						uid_no, header_file_name,
						body_file_name,
						msg_info->header, file_name, mimeinfo, FALSE);

		} else {
			ret = imap_cmd_fetch(IMAP_SESSION(session), uid_no, file_name);
			if (ret == IMAP_SUCCESS) {
				msg_info->header->msgtag.fullydownloaded = TRUE;
			}

		}

	} else if (prefs_account->receive_settings.recv_type == BODY) {

		if (size_limit_flag == TRUE) {

			UNSET_CB_FLAG;
			ret = download_imap_body(session, prefs_account,
						 header_file_name,
						 body_file_name,
						 msg_info->header,
						 MSG_BODY_SEC, uid_no, file_name, mimeinfo);
			SET_CB_FLAG;

		} else {
			ret = imap_get_body(prefs_account, session, uid_no,
					    header_file_name, body_file_name,
					    msg_info->header, file_name, mimeinfo);

		}
	}
	return ret;
}


/**
 This function checks if the message is downloaded or not 
 @param msgid    message id of the file.
 @return TRUE if message is already downloaded else FALSE 
*/
static gboolean imap_is_msg_exist(const gchar * msgid)
{
	gchar *file_name = NULL;
	gchar *header_file_name = NULL;
	gboolean ret = TRUE;

	file_name = folder_get_message_file_path(msgid);
	header_file_name = imap_get_file_name(msgid, HEAD);

	if ((is_file_entry_exist(header_file_name) == FALSE)
	    && (is_file_entry_exist(file_name) == FALSE)) {

		ret = FALSE;
	}
	g_free(file_name);
	g_free(header_file_name);
	return ret;
}

static gint
imap_get_part(IMAPSession * imap_session,
	      PrefsAccount * prefs_account, const gchar * msgid, MimeInfo * mimeinfo)
{

	gint ok = 0;
	gint body_size = 0;
	gchar *attach_file_name = NULL;
	gchar *tmp_file_path = NULL;

	if (prefs_account->receive_settings.enable_size_limit) {

		body_size = procmime_get_body_size(mimeinfo, (gchar *) mimeinfo->section_number);

		if (body_size == IMAP_FAIL) {
			return IMAP_FAIL;
		}
		if (((float) body_size) / 1024 > (float) prefs_account->receive_settings.size_limit) {
			IMAP_SET_ERROR_CB(IMAP_MSG_SIZE_EXCEED_ERROR);
			return IMAP_FAIL;
		}
	}

	tmp_file_path = folder_get_message_file_path(msgid);

	attach_file_name = g_strdup_printf("%s%c%s",
					   tmp_file_path, SECTION_SEPARATOR,
					   mimeinfo->section_number);
	g_free(tmp_file_path);

	ok = imap_cmd_fetch_attach(imap_session, get_msgid_num(msgid),
				   attach_file_name, (gchar *) mimeinfo->section_number);
	g_free(attach_file_name);

	return ok;

}

static gint
imap_get_full_body(PrefsAccount * prefs_account, IMAPSession * imap_session, const gchar * msgid)
{
	gint size = 0;
	gint uid_no = 0;
	gint ret = IMAP_FAIL;
	gchar *file_name = NULL;

	uid_no = get_msgid_num(msgid);

	if (prefs_account->receive_settings.enable_size_limit) {
		size = imap_cmd_mail_size(SESSION(imap_session), uid_no);
	}
	if (size != 0) {
		if (((float) size) / 1024 > (float) prefs_account->receive_settings.size_limit) {
			return ret;
		}
	}
	file_name = folder_get_message_file_path(msgid);
	ret = imap_cmd_fetch(imap_session, uid_no, file_name);

	g_free(file_name);
	return ret;
}

gint
imap_get_mhtml_body(MimeInfo * mimeinfo, FILE * fp,
		    const gchar * msgid, PrefsAccount * prefs_account)
{
	Session *session = NULL;
	gint exists = 0;
	gint unseen = 0;
	gint recent = 0;
	gint uid_validity = 0;
	gint ret_hd = 0;
	gint ret_val = IMAP_FAIL;
	gint ok = IMAP_FAIL;
	gint uid_no = 0;
	gchar *header_file_name = NULL;
	gchar *tmp_file_path = NULL;
	MsgFlags imap_flags = { 0, 0 };

	/* Creates a Session */
	session = imap_session_new(prefs_account, msgid);
	if (session == NULL) {
		engine_errno = ENGINE_ERROR_CONNECT_SERVER;
		return IMAP_FAIL;
	} else {

		IMAP_SET_MSG_STR_CB(IMAP_CONNECT_SERVER_SUCCESS,
				    prefs_account->user_settings.account_name, msgid);
	}

	/* selects the inbox in read only mode & fetches the inbox status. */

	if ((imap_cmd_do_uid_select(IMAP_SESSION(session), INBOX_DIR,
				    FALSE, &exists, &unseen, &recent,
				    &uid_validity, prefs_account, TRUE)) != IMAP_SUCCESS) {
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		return IMAP_FAIL;
	}
	uid_no = get_msgid_num(msgid);
	ok = imap_cmd_flags(IMAP_SESSION(session), uid_no, &imap_flags);

	if ((ok != IMAP_SUCCESS) || (imap_flags.perm_flags & MSG_DELETED)) {
		IMAP_SET_MSG_CB(IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER, msgid);
		imap_delete_header_file(msgid);
		imap_session_disconnect(session, FALSE);
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		return IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER;
	}
	tmp_file_path = folder_get_message_file_path(msgid);

	header_file_name = g_strdup_printf("%s%c%s", tmp_file_path, SECTION_SEPARATOR, HEAD);
	g_free(tmp_file_path);
	ret_hd = imap_cmd_fetch_attach(IMAP_SESSION(session), uid_no, header_file_name, HEAD);
	if (ret_hd == IMAP_SUCCESS) {

		ret_val = imap_get_inline_part(IMAP_SESSION(session),
					       mimeinfo, fp, msgid, prefs_account);
	}
	/* closes the session */
	IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
	imap_session_disconnect(session, FALSE);
	g_free(header_file_name);

	return ret_val;
}

/**
   This function returns the S/MIME type of the message
   It takes the body file from respective folder and 
   checks for S/MIME type.
   @param msg_info msginfo contaning the message ID
   @return smime type
  */
static gint imap_get_smime_type(MsgInfo * msg_info)
{
	gchar *body_file = NULL;
	gint smime_header = 0;
	gint smime_type = 0;

	body_file = g_strdup_printf("%s%c%s%c%s", get_mail_dir(),
				    G_DIR_SEPARATOR, INBOX_DIR, G_DIR_SEPARATOR,
				    msg_info->header->msgid);

	smime_header = msg_info->header->smime_type;

	smime_type = find_smime_type(body_file);

	g_free(body_file);

	if (smime_type == SMIME_NONE) {
		return smime_header;
	} else {
		return smime_type;
	}
}


GSList *imap_move_copy_msgs(const gchar * srcfolder,
			    const gchar * destfolder, PrefsAccount * prefs_account,
			    GSList * folder_msg_list,
			    gboolean move_flag, GSList * folder_append_list,
			    gint * imap_ret, gint * cur_ptr, gint total,
			    FolderProgressCb progress_cb,
			    GSList ** update_list_ptr, GSList ** folder_del_list_ptr)
{

	GSList *cur = NULL;
	gchar *msgid = NULL;
	gint ret_val = FOLDER_FAIL;
	gint tmpret_val = IMAP_SUCCESS;
	*imap_ret = IMAP_SUCCESS;
	gchar *filename = NULL;
	Session *session = NULL;
	gboolean local_copy = FALSE;
	gchar *dest_msgid = NULL;

	for (cur = folder_msg_list; cur != NULL; cur = cur->next) {

		msgid = cur->data;

		filename = folder_get_message_file_path(msgid);

		(*cur_ptr)++;

		tmpret_val = IMAP_SUCCESS;
		/* download the remote mails */

		local_copy = FALSE;

		if (is_file_entry_exist(filename) == TRUE) {

			local_copy = TRUE;

		}
		if (((local_copy == FALSE) || (move_flag == TRUE)) &&
		    (folder_is_msg_remote(msgid) == TRUE)) {

			progress_cb(*cur_ptr, total);

			if (session == NULL) {
				if ((session = imap_get_op_session(prefs_account))
				    == NULL) {
					*imap_ret = IMAP_ERROR;
					return folder_append_list;

				}
			}
			/* download the remote mails */
			tmpret_val = imap_cmd_fetch_foldermsg(session, msgid,
							      prefs_account, move_flag,
							      srcfolder, filename,
							      local_copy, total);
		}

		g_free(filename);
		if (move_flag == TRUE) {
			ret_val = folder_move_msg(srcfolder, destfolder, msgid, &dest_msgid);

		} else {
			ret_val = folder_copy_msg(srcfolder, destfolder, msgid, &dest_msgid);
		}
		if (ret_val == FOLDER_SUCCESS) {
			if (move_flag == TRUE) {
				if ((strcmp(srcfolder, INBOX_DIR) == 0) &&
				    (folder_is_msg_remote(msgid) == TRUE)) {

					ret_val = imap_delete_moved_msg(msgid, session);
				}
				folder_delete_list_msg(cur->data, TRUE);

			}
			ret_val = folder_update_unread(msgid, dest_msgid, move_flag);

			folder_append_list =
			    folder_append_header_list(folder_append_list, msgid,
						      destfolder, prefs_account,
						      FALSE, update_list_ptr, dest_msgid);
		}
		imap_get_op_error(tmpret_val, ret_val, imap_ret,
				  update_list_ptr, folder_del_list_ptr, prefs_account, msgid);


		(void) g_main_context_iteration(NULL, FALSE);

		if (gl_imap_recv_cancel == TRUE) {

			gl_imap_recv_cancel = FALSE;
			break;
		}
	}
	if (session != NULL) {
		imap_cmd_close(IMAP_SESSION(session));
		IMAP_DISCONNECT(session, IMAP_SUCCESS);
		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
	}

	return folder_append_list;
}

static gint
imap_cmd_fetch_foldermsg(Session * session, const gchar * msgid,
			 PrefsAccount * prefs_account,
			 gboolean move_flag, const gchar * srcfolder,
			 const gchar * filename, gboolean local_copy, gint total)
{
	gint ok = IMAP_SUCCESS;
	gint msgnum = 0;
	gboolean read_flag = FALSE;
	GSList *part_msgid_list = NULL;
	MsgFlags imap_flags = { 0, 0 };

	msgnum = get_msgid_num(msgid);


	ok = imap_cmd_flags(IMAP_SESSION(session), msgnum, &imap_flags);

	if ((ok != IMAP_SUCCESS)
	    || (imap_flags.perm_flags & MSG_DELETED)) {

		engine_errno = ENGINE_ERROR_IMAP_MSG_DELETE;
		part_msgid_list = folder_get_msgid_all_files(srcfolder, msgid, NULL, TRUE);

		folder_delete_list_msg(msgid, TRUE);
		folder_message_del_list(msgid, UNREAD);
		folder_remove_msgs(srcfolder, part_msgid_list);
		slist_free_strings(part_msgid_list);
		g_slist_free(part_msgid_list);
		return IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER;
	}
	if (local_copy == FALSE) {

		part_msgid_list = folder_rem_parts(srcfolder, msgid, &read_flag);

		ok = imap_cmd_fetch(IMAP_SESSION(session), msgnum, filename);

		if ((ok == IMAP_SUCCESS) && (part_msgid_list != NULL)) {

			if (folder_remove_msgs(srcfolder, part_msgid_list) != FOLDER_SUCCESS) {
				ok = IMAP_ERROR;

			}
			slist_free_strings(part_msgid_list);
			g_slist_free(part_msgid_list);
		}
		if ((read_flag == FALSE) && (ok == IMAP_SUCCESS) &&
		    (imap_flags.perm_flags & MSG_UNREAD)) {

			if (folder_message_add_list(msgid, UNREAD) != FOLDER_SUCCESS) {
				ok = IMAP_ERROR;
			}
		}
	}
	return ok;
}

static gint imap_delete_moved_msg(const gchar * msgid, Session * session)
{
	gchar *uid_no = NULL;
	gint msgnum = 0;
	gint del_ok = IMAP_SUCCESS;

	msgnum = get_msgid_num(msgid);
	uid_no = g_strdup_printf("%d", msgnum);

	/* sets the message flag deleted */
	del_ok = imap_set_message_flags(IMAP_SESSION(session), uid_no, IMAP_FLAG_DELETED, TRUE);

	if (del_ok != IMAP_SUCCESS) {
		engine_errno = ENGINE_ERROR_IMAP_MSG_DELETE;
		return IMAP_ERROR;
	}
	return IMAP_SUCCESS;
}

static Session *imap_get_op_session(PrefsAccount * prefs_account)
{
	Session *session = NULL;
	gint exists = 0;
	gint unseen = 0;
	gint recent = 0;
	gint uid_validity = 0;

	/* Creates a Session */
	session = imap_session_new(prefs_account, NULL);
	if (session == NULL) {
		engine_errno = ENGINE_ERROR_CONNECT_SERVER;
		return NULL;
	}

	/* selects the inbox in read only mode & fetches the 
	 * inbox status. */

	if ((imap_cmd_do_uid_select(IMAP_SESSION(session),
				    INBOX_DIR,
				    FALSE, &exists, &unseen, &recent,
				    &uid_validity, prefs_account, TRUE)) != IMAP_SUCCESS) {

		IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
		imap_session_disconnect(session, FALSE);
		return NULL;
	}
	return session;
}

static gboolean progress_func(gpointer data)
{
	ProgData *data_ptr = NULL;
	gint status = 0;

	data_ptr = (ProgData *) data;
	if ((gl_stop_timer == TRUE) || (gl_imap_recv_cancel == TRUE)) {
		return FALSE;
	}
	if (osso_email_gconf_get_int(EMAIL_GCONF_UI_CLOSED, &status) == FALSE) {
		return FALSE;
	}
	if (status == 1) {
		return FALSE;

	}

	IMAP_PROGRESS_CB(data_ptr->cur, data_ptr->total);
	return TRUE;
}

static gint imap_cmd_flags(IMAPSession * session, gint uid, MsgFlags * imap_flag_ptr)
{
	gchar *buf = NULL;
	MsgFlags imap_flags = { 0, 0 };
	gchar *cur_pos = NULL;
	gchar *ptr = NULL;
	gint ok = IMAP_SUCCESS;
	gchar tmp_buf[IMAPBUFSIZE];

	memset(tmp_buf, '\0', sizeof(tmp_buf));

	imap_cmd_gen_send(session, "UID FETCH %d (FLAGS)", uid);

	while ((ok = imap_cmd_gen_recv(session, &buf)) == IMAP_SUCCESS) {
		if (buf[0] != ASTERISK || buf[1] != SPACE_CHAR) {
			g_free(buf);
			return IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER;
		}
		if ((cur_pos = strstr(buf, IMAP_FETCH_CMD)) != NULL) {
			break;
		}
		ok = IMAP_ERROR;
		g_free(buf);
	}
	if (ok != IMAP_SUCCESS) {
		return ok;
	}
	if ((ok = imap_cmd_gen_recv(session, &ptr)) == IMAP_SUCCESS) {
		g_free(ptr);
	}
	if (ok != IMAP_SUCCESS) {
		g_free(buf);
		return ok;
	}

	if ((ptr = strstr(cur_pos, "FLAGS ")) != NULL) {
		cur_pos = ptr;
		cur_pos += 6;
		if (*cur_pos != '(') {
			log_message("*cur_pos != '('\n");
			g_free(buf);
			return IMAP_ERROR;
		}
		cur_pos++;
		cur_pos = strchr_cpy(cur_pos, ')', tmp_buf, sizeof(tmp_buf));

		if (tmp_buf == NULL) {
			g_warning("invalid FETCH response: %s\n", cur_pos);
			ok = IMAP_ERROR;

		} else {
			imap_flags = imap_parse_flags(tmp_buf);
			*imap_flag_ptr = imap_flags;
		}
	} else {
		ok = IMAP_ERROR;
	}

	g_free(buf);


	return ok;
}
static void
imap_get_op_error(gint tmpret_val, gint ret_val, gint * imap_ret,
		  GSList ** update_list_ptr, GSList ** folder_delete_list,
		  PrefsAccount * prefs_account, const gchar * msgid)
{

	if ((tmpret_val != IMAP_SUCCESS) && (*imap_ret == IMAP_SUCCESS)) {
		*imap_ret = tmpret_val;

	} else if ((ret_val != FOLDER_SUCCESS)
		   && (*imap_ret == IMAP_SUCCESS)) {
		*imap_ret = IMAP_ERROR;

	}
	if (*imap_ret == IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER) {

		/* check for always delete list ptr first */
		if (folder_delete_list != NULL) {
			*folder_delete_list =
			    folder_append_header_list(*folder_delete_list, msgid,
						      NULL, prefs_account, TRUE, NULL, NULL);
		} else if (update_list_ptr != NULL) {
			*update_list_ptr =
			    folder_append_header_list(*update_list_ptr, msgid,
						      NULL, prefs_account, TRUE, NULL, NULL);

		}
	}
}
PrefsAccount *imap_get_cur_account(void)
{
	return gl_current_account;
}

static gint
imap_cmd_search(IMAPSession * session, gint * first_uid_ptr,
		gint * last_uid_ptr,
		PrefsAccount * prefs_account,
		gboolean * update_first_ptr, gint msg_last_uid, gboolean new_msg_exist)
{
	gchar *cur_pos = NULL;
	gchar *ptr = NULL;
	gchar tmp[IMAP_CMD_SIZE];
	gint ok = IMAP_FAIL;
	gchar seq_set[IMAP_SEQ_SIZE];
	gint count = 0;
	gint msgnum = 0;
	GArray *uid_array = NULL;

	if (msg_last_uid != 1) {
		g_snprintf(seq_set, sizeof(seq_set), "%u:%u", 1, msg_last_uid);
	} else {
		g_snprintf(seq_set, sizeof(seq_set), "%u", msg_last_uid);
	}
	imap_cmd_gen_send(session, "UID SEARCH UNDELETED UID %s", seq_set);

	if ((ok = imap_cmd_gen_recv(session, &ptr)) == IMAP_SUCCESS) {
		if (ptr[0] != '*' || ptr[1] != ' ') {
			g_free(ptr);
			return ok;
		}
		if ((cur_pos = strstr(ptr, "SEARCH")) != NULL) {

			uid_array = g_array_new(FALSE, FALSE, sizeof(gint));

			cur_pos += 7;

			while (*cur_pos != '\0') {
				memset(tmp, sizeof(tmp), '\0');
				while (*cur_pos == SPACE_CHAR) {
					cur_pos++;
					if (*cur_pos == '\0') {
						break;
					}
				};

				sscanf(cur_pos, "%d", &msgnum);
				sscanf(cur_pos, "%s", tmp);

				cur_pos = cur_pos + strlen(tmp) + 1;
				g_array_append_val(uid_array, msgnum);

			}
			g_free(ptr);

			while ((ok = imap_cmd_gen_recv(session, &ptr)) == IMAP_SUCCESS) {
				if ((g_strrstr(ptr, "OK") != NULL)
				    || (g_strrstr(ptr, "ok") != NULL)) {
					break;
				}
			}
			g_free(ptr);

		} else {
			ok = IMAP_FAIL;
		}
	}

	if (ok != IMAP_SUCCESS) {
		IMAP_SET_ERROR_CB(IMAP_ERROR);
		g_array_free(uid_array, TRUE);
		return IMAP_FAIL;
	}
	if (new_msg_exist == TRUE) {
		count = imap_get_msg_count(uid_array, first_uid_ptr, last_uid_ptr, prefs_account);
	}
	/* if no messages were deleted,do not check */
	if ((prefs_account->receive_settings.num_of_msgs == uid_array->len) &&
	    (new_msg_exist == FALSE)) {

		g_array_free(uid_array, TRUE);
		return IMAP_SUCCESS;
	}
	/* Synchronise with the messages on server */
	imap_check_deleted_msgs(uid_array, prefs_account->user_settings.account_name);

	if ((prefs_account->receive_settings.last_msgid < *last_uid_ptr) &&
	    (prefs_account->receive_settings.last_msgid + 1 == *first_uid_ptr)
	    && (prefs_account->receive_settings.first_msgid != 0)) {
		*update_first_ptr = FALSE;
	}

	g_array_free(uid_array, TRUE);

	return count;
}

static gint
imap_download_body_check(Session * session,
			 MimeInfo * mimeinfo, const gchar * msgid,
			 PrefsAccount * prefs_account, gint uid_no,
			 const gchar * section_number, const gchar * body_filename)
{
	MimeInfo *cur_mimeinfo = NULL;
	gint ret = IMAP_FAIL;
	gint body_size = 0;

	if (prefs_account->receive_settings.enable_size_limit) {

		body_size = imap_cmd_get_body_size(session,
						   (gchar *) msgid, (gchar *) section_number);
		if (body_size == IMAP_FAIL) {
			IMAP_SET_MSG_CB(IMAP_COMPLETE, prefs_account->user_settings.account_name);
			return IMAP_FAIL;
		}
		if (((float) body_size) / 1024 > (float) prefs_account->receive_settings.size_limit) {
			IMAP_SET_ERROR_CB(IMAP_MSG_SIZE_EXCEED_ERROR);
			return IMAP_FAIL;
		}
	}

	for (cur_mimeinfo = mimeinfo; cur_mimeinfo != NULL; cur_mimeinfo = cur_mimeinfo->children) {

		if (cur_mimeinfo->content_type == NULL) {
			continue;
		}
		if ((g_ascii_strncasecmp(cur_mimeinfo->content_type,
					 MULTIPART_RELATED, strlen(MULTIPART_RELATED)) == 0)
		    ||
		    (g_ascii_strncasecmp
		     (cur_mimeinfo->content_type, MULTIPART_ALTERNATIVE,
		      strlen(MULTIPART_RELATED)) == 0)) {

			ret =
			    imap_get_multipart_body(IMAP_SESSION(session),
						    cur_mimeinfo, msgid, prefs_account);
			if (ret == IMAP_FAIL){
			       	if( !imap_is_session_cancelled()) 
					IMAP_SET_MSG_CB(IMAP_MSG_FETCH_ERROR, msgid);
				return ret;
			}
		}
	}
	ret = imap_cmd_fetch_attach(IMAP_SESSION(session), uid_no, body_filename, section_number);
	if (ret == IMAP_ERROR && !imap_is_session_cancelled()) {
		IMAP_SET_MSG_CB(IMAP_MSG_FETCH_ERROR, msgid);
		return IMAP_FAIL;
	}
	return IMAP_SUCCESS;
}
static MimeInfo *imap_scan_mimeinfo(const gchar * msgid, const gchar * filepath, Session * session)
{
	FILE *mime_fp;
	guchar buf[BUFSIZ];
	gchar *body_str = NULL;
	gchar *tmp_str = NULL;
	gchar *mime_file_name = NULL;
	size_t len;
	MimeInfo *mimeinfo = NULL;

	mime_file_name = g_strdup_printf("%s%c%s", filepath, SECTION_SEPARATOR, MIME_SUFFIX);
	if (is_file_entry_exist(mime_file_name) == TRUE) {
		memset(buf, '\0', BUFSIZ);
		mime_fp = fopen(mime_file_name, "r");
		if (mime_fp == NULL) {
			g_free(mime_file_name);
			return NULL;
		}
		while ((len = fread(buf, sizeof(gchar), sizeof(buf) - 1, mime_fp)) > 0) {
			if (tmp_str != NULL) {
				body_str = g_strconcat(tmp_str, buf, NULL);
				g_free(tmp_str);
			} else {
				body_str = g_strconcat(buf, NULL);
			}
			if (body_str == NULL) {
				fclose(mime_fp);
				g_free(mime_file_name);
				return NULL;
			}
			tmp_str = body_str;
		}
		fclose(mime_fp);
	} else {
		body_str = imap_cmd_body_structure(IMAP_SESSION(session), msgid);

		if (body_str == NULL) {
			IMAP_SET_MSG_CB(IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER, msgid);
			imap_delete_header_file(msgid);
		}
	}
	mimeinfo = procmime_scan_body(body_str, strlen(body_str));
	g_free(mime_file_name);
	g_free(body_str);
	return mimeinfo;
}

gboolean imap_delete_header_file(const gchar * msgid)
{
	gchar *path = NULL;
	GSList *message_list = NULL;
	gint count = 0;
	gint length = 0;
	gchar *name = NULL;

	if (msgid == NULL) {
		return FALSE;
	}

	message_list = folder_get_msgid_files(INBOX_DIR, msgid);

	if (message_list == NULL) {
		return FALSE;
	}

	length = g_slist_length(message_list);
	for (count = 0; count < length; count++) {
		name = g_slist_nth_data(message_list, count);
		path = folder_get_message_file_path(name);
		unlink(name);
		g_free(path);
	}

	slist_free_strings(message_list);
	g_slist_free(message_list);
	folder_delete_list_msg(msgid, TRUE);
	folder_delete_list_msg(msgid, FALSE);
	folder_message_del_list(msgid, UNREAD);
	return TRUE;
}

static gint
imap_get_msg_count(GArray * uid_array, gint * first_uid_ptr,
		   gint * last_uid_ptr, PrefsAccount * prefs_account)
{
	gint i = 0;
	gboolean start_count = FALSE;
	gint msgnum = 0;
	gint count = 0;
	gint start_index = -1;
	gint end_index = -1;

	for (i = uid_array->len - 1; i >= 0; i--) {

		msgnum = g_array_index(uid_array, gint, i);

		if ((*first_uid_ptr >= msgnum) && (start_index == -1)) {
			start_index = i;
		}
		if ((*last_uid_ptr >= msgnum) && (end_index == -1)) {
			end_index = i;
		}
		if ((start_count == FALSE) && (end_index != -1)) {

			start_count = TRUE;
			*last_uid_ptr = msgnum;

		}
		if ((prefs_account->receive_settings.last_msgid != msgnum) &&
		    (prefs_account->receive_settings.first_msgid != msgnum) &&
		    (start_count == TRUE)) {

			count++;
		}

		if ((prefs_account->receive_settings.recent_mails != 0) &&
		    (prefs_account->receive_settings.recent_mails == count)) {
			break;
		}
		if (msgnum <= *first_uid_ptr) {
			break;
		}
		if (start_index != -1) {
			break;
		}
	}

	*first_uid_ptr = msgnum;
	return count;
}

static gint
imap_get_range(PrefsAccount * prefs_account,
	       gint * tmp_last_ptr, gint * tmp_first_ptr,
	       gboolean * update_last_msgid, Session * session, gint * msg_last_uid)
{
	gint tmp_last_uid = 0;
	gint tmp_first_uid = 0;
	gint latest_last_uid = 0;
	gint first_uid = 0;

	*update_last_msgid = TRUE;

	if (gl_msgs_exists_server == 0) {
		return IMAP_FAIL;
	}
	latest_last_uid = imap_cmd_get_uid(IMAP_SESSION(session), gl_msgs_exists_server);

	if (latest_last_uid < 1) {
		return IMAP_FAIL;
	}
	*msg_last_uid = latest_last_uid;

	first_uid = imap_cmd_get_uid(IMAP_SESSION(session), 1);
	if (first_uid < 1) {
		return IMAP_FAIL;
	}

	if (prefs_account->receive_settings.last_msgid >= latest_last_uid) {

		if (prefs_account->receive_settings.first_msgid <= first_uid) {
			/* nothing to fetch */
			*tmp_first_ptr = first_uid;
			*tmp_last_ptr = latest_last_uid;
			return IMAP_FAIL;
		} else if (prefs_account->receive_settings.first_msgid != 0) {

			tmp_last_uid = prefs_account->receive_settings.first_msgid - 1;
			tmp_first_uid = first_uid;
			*update_last_msgid = FALSE;
		}

	} else {

		tmp_last_uid = latest_last_uid;
		tmp_first_uid = prefs_account->receive_settings.last_msgid + 1;
	}
	*tmp_last_ptr = tmp_last_uid;
	if (tmp_first_uid == 0) {
		tmp_first_uid = first_uid;
	}
	*tmp_first_ptr = tmp_first_uid;
	return IMAP_SUCCESS;
}
static void imap_check_deleted_msgs(GArray * uid_array, const gchar * account_name)
{

	gchar *file_account = NULL;
	DIR *dp = NULL;
	struct dirent *d = NULL;
	gchar *inbox_folder = NULL;
	gchar *msg_id = NULL;
	GSList *remove_msg_list = NULL;
	gint last_srch_num = 0;
	GSList *file_list = NULL;
	GSList *msgid_list = NULL;
	GSList *tmp = NULL;
	gchar *f_path = NULL;

	inbox_folder = g_strdup_printf("%s%c%s", get_mail_dir(), G_DIR_SEPARATOR, INBOX_DIR);


	if (change_dir(inbox_folder) < 0) {
		g_free(inbox_folder);
		return;
	}
	if ((dp = opendir(".")) == NULL) {
		g_free(inbox_folder);
		return;
	}
	d = readdir(dp);

	while (d != NULL) {
		if (check_msgid(d->d_name) == FALSE) {
			d = readdir(dp);
			continue;
		}
		if (dirent_is_regular_file(d)) {
			f_path = folder_get_message_file_path(d->d_name);
			if (folder_check_partial(f_path) == FALSE) {
				free(f_path);
				d = readdir(dp);
				continue;
			}
			free(f_path);

			file_account = get_account_name(d->d_name);
			msg_id = folder_get_msgid(d->d_name);

			if (strcmp(file_account, account_name) == 0) {
				file_list = g_slist_append(file_list, msg_id);
			}
			g_free(file_account);
		}
		d = readdir(dp);
	}
	g_free(inbox_folder);
	closedir(dp);

	msgid_list = g_slist_sort(file_list, folder_msgnum_sort_comparefunc);

	for (tmp = msgid_list; tmp != NULL; tmp = tmp->next) {

		msg_id = tmp->data;

		if (imap_is_header_deleted(uid_array, msg_id,
					   &remove_msg_list, &last_srch_num) == TRUE) {

			folder_delete_list_msg(msg_id, TRUE);
			folder_delete_list_msg(msg_id, FALSE);
			folder_message_del_list(msg_id, UNREAD);
		}
	}

	slist_free_strings(file_list);
	g_slist_free(file_list);

	if (remove_msg_list != NULL) {
		folder_remove_msgs(INBOX_DIR, remove_msg_list);
	}
	slist_free_strings(remove_msg_list);
	g_slist_free(remove_msg_list);

}

static gboolean
imap_is_header_deleted(GArray * uid_array,
		       const gchar * msgid, GSList ** rem_msg_list, gint * last_srch_num)
{
	gboolean body_download_flag = FALSE;
	gint count = 0;
	gint msgnum = 0;
	gint i = 0;
	gint serv_msgnum = 0;
	GSList *file_list = NULL;
	GSList *last_list = NULL;
	gchar *file_path = NULL;

	file_path = folder_get_message_file_path(msgid);

	if (is_file_entry_exist(file_path) == TRUE) {
		g_free(file_path);
		return FALSE;
	}
	file_list = folder_get_msgid_all_files(INBOX_DIR,
					       (gchar *) msgid, &body_download_flag, FALSE);
	if (file_list != NULL) {
		count = g_slist_length(file_list);
		if (count > 1) {
			/* body is downloaded */
			slist_free_strings(file_list);
			g_slist_free(file_list);
			return FALSE;
		}
	}
	msgnum = get_msgid_num(msgid);
	if (uid_array != NULL) {
		for (i = *last_srch_num; (i <= uid_array->len - 1) && (uid_array->len > 0); i++) {

			serv_msgnum = g_array_index(uid_array, gint, i);
			if (serv_msgnum == msgnum) {
				*last_srch_num = i + 1;
				return FALSE;
			}
			/* the next uid is greater so stop searching */
			if (serv_msgnum > msgnum) {
				*last_srch_num = i;
				break;
			}
		}
	}
	if (*rem_msg_list != NULL) {
		last_list = g_slist_last(*rem_msg_list);
		last_list->next = file_list;
	} else {
		*rem_msg_list = file_list;
	}
	gl_imap_delete_list = g_slist_append(gl_imap_delete_list, g_strdup(msgid));

	return TRUE;
}

gint
imap_retr_imap_msg(const gchar * msgid, PrefsAccount * account,
		   gboolean complete_download, Session ** session_ptr, gint * cur, gint total)
{

	GSList *msgid_list = NULL;
	gchar *srcfolder = INBOX_DIR;
	gchar *file_name = NULL;
	gchar *mime_file_name = NULL;
	gchar *enc_file_name = NULL;
	gchar *mhtml_file = NULL;
	MimeInfo *mimeinfo = NULL;
	MimeInfo *partinfo = NULL;
	gboolean smime_check = FALSE;
	gint exists = 0;
	gint unseen = 0;
	gint recent = 0;
	gint uid_validity = 0;

	file_name = folder_get_message_file_path(msgid);

	if (is_file_entry_exist(file_name) == TRUE) {
		return IMAP_SUCCESS;
	}
	mime_file_name = g_strdup_printf("%s%c%s", file_name, SECTION_SEPARATOR, MIME_SUFFIX);

	if (*session_ptr == NULL) {

		*session_ptr = imap_session_new(account, NULL);
		if (*session_ptr == NULL) {
			engine_errno = ENGINE_ERROR_CONNECT_SERVER;
			return IMAP_FAIL;
		}
		if ((imap_cmd_do_uid_select(IMAP_SESSION(*session_ptr),
					    INBOX_DIR,
					    FALSE, &exists, &unseen, &recent,
					    &uid_validity, account, FALSE)) != IMAP_SUCCESS) {
			IMAP_SET_MSG_CB(IMAP_COMPLETE, account->user_settings.account_name);

			imap_session_disconnect(*session_ptr, FALSE);
			*session_ptr = NULL;
			return IMAP_FAIL;
		}
	}
	if (is_file_entry_exist(mime_file_name) == FALSE) {
		mimeinfo = imap_fetch_msg_body_structure(account, (gchar *) msgid, session_ptr);
	} else {
		mimeinfo = procmime_read_mimeinfo(mime_file_name);
	}
	g_free(mime_file_name);

	if (mimeinfo == NULL) {
		g_free(file_name);
		return IMAP_FAIL;
	}

	partinfo = procmime_get_text_ptr(mimeinfo, FALSE);

	if (partinfo == NULL) {
		procmime_mimeinfo_free_all(mimeinfo);
		g_free(file_name);
		return IMAP_SUCCESS;
	}
	enc_file_name = g_strdup_printf("%s%c%s",
					file_name, SECTION_SEPARATOR, partinfo->section_number);

	if (is_file_entry_exist(enc_file_name) == TRUE && !complete_download) {
		procmime_mimeinfo_free_all(mimeinfo);
		g_free(file_name);
		g_free(enc_file_name);
		return IMAP_SUCCESS;

	}
	mhtml_file = g_strdup_printf("%s%c%s%c%s%c%s",
				     get_mail_dir(), G_DIR_SEPARATOR,
				     INBOX_DIR, G_DIR_SEPARATOR,
				     msgid, SECTION_SEPARATOR, MHTML_EXTENSION);

	if (is_file_entry_exist(mhtml_file) == TRUE && !complete_download) {
		procmime_mimeinfo_free_all(mimeinfo);
		g_free(file_name);
		g_free(mhtml_file);
		return IMAP_SUCCESS;
	}
	smime_check = procmime_is_smime(mimeinfo);

	if ((smime_check == TRUE)
	    || (procmime_mime_attachment(mimeinfo) == FALSE)) {
		complete_download = TRUE;

	}
	IMAP_PROGRESS_CB(*cur, total);

	if ((complete_download == TRUE)
	    || (account->receive_settings.recv_type == BODY_ATTCH)) {

		msgid_list = folder_get_msgid_all_files(srcfolder, msgid, NULL, FALSE);

		if(is_message_unread(msgid)) {
		imap_cmd_fetch_msg_body(msgid, account, FALSE,
					TRUE, smime_check, *session_ptr, FALSE);
		} else {
		imap_cmd_fetch_msg_body(msgid, account, TRUE,
					TRUE, smime_check, *session_ptr, FALSE);
		}

		imap_send_repopulate();

		/* if file is downloaded successfully,delete parts of the mail */
		if ((is_file_entry_exist(file_name) == TRUE)
		    && (msgid_list != NULL)) {
			folder_remove_msgs(srcfolder, msgid_list);
		}
		slist_free_strings(msgid_list);
		g_slist_free(msgid_list);

	} else {

		imap_fetch_msg_attachment(account, (gchar *) msgid,
					  partinfo->section_number,
					  TRUE, *session_ptr, mimeinfo, FALSE);
	}
	g_free(file_name);

	return IMAP_SUCCESS;

}

void imap_session_complete(Session * session)
{
	IMAP_SET_MSG_CB(IMAP_COMPLETE,
			(IMAP_SESSION(session))->ac_prefs->user_settings.account_name);
	imap_session_disconnect(session, FALSE);
}

gboolean imap_is_session_cancelled(void)
{
	return gl_imap_recv_cancel;
}

static gint imap_remove_list_msgs(GSList * c_list)
{
	MsgInfo *msg_info = NULL;
	gchar *header_file_name = NULL;
	gchar *mime_file_name = NULL;

	if (c_list == NULL) {
		return IMAP_FAIL;
	}
	while (c_list) {

		msg_info = (MsgInfo *) c_list->data;

		if (msg_info != NULL) {
			if (msg_info->header->msgid == NULL) {
				return IMAP_FAIL;
			}
			folder_delete_list_msg(msg_info->header->msgid, TRUE);
			folder_delete_list_msg(msg_info->header->msgid, FALSE);
			folder_message_del_list(msg_info->header->msgid, UNREAD);

			header_file_name = imap_get_file_name(msg_info->header->msgid, HEAD);

			if (is_file_entry_exist(header_file_name)
			    == TRUE) {
				unlink(header_file_name);
			}
			g_free(header_file_name);

			mime_file_name = g_strdup_printf("%s%c%s%c%s%c%s",
							 get_mail_dir(),
							 G_DIR_SEPARATOR, INBOX_DIR,
							 G_DIR_SEPARATOR,
							 msg_info->header->msgid,
							 SECTION_SEPARATOR, MIME_SUFFIX);

			if (is_file_entry_exist(mime_file_name)
			    == TRUE) {
				unlink(mime_file_name);
			}
			g_free(mime_file_name);
		}
		procmsg_msginfo_free((MsgInfo *) c_list->data);

		c_list = g_slist_remove(c_list, c_list->data);

	}
	return IMAP_SUCCESS;
}
static MimeInfo *imap_get_msg_mimeinfo(MsgInfo * msg_info,
				       Session * session, PrefsAccount * prefs_account)
{
	FILE *mime_fp = NULL;
	MimeInfo *mime_info = NULL;
	gchar *tmp = NULL;
	gchar *mime_file_name = NULL;
	gint tmp_len = 0;
	GSList *attach_list = NULL;

	tmp = imap_cmd_body_structure(IMAP_SESSION(session), msg_info->header->msgid);

	if (tmp == NULL) {
		return NULL;
	}
	if (prefs_account->receive_settings.recv_type != HEADERS) {

		mime_file_name = g_strdup_printf("%s%c%s%c%s%c%s",
						 get_mail_dir(),
						 G_DIR_SEPARATOR, INBOX_DIR,
						 G_DIR_SEPARATOR,
						 msg_info->header->msgid,
						 SECTION_SEPARATOR, MIME_SUFFIX);
		mime_fp = fopen(mime_file_name, "w");

		if (mime_fp == NULL) {
			send_engine_error_to_ui(errno);
			imap_recv_cancel();
			g_free(tmp);
			g_free(mime_file_name);
			return NULL;
		}

		if (fwrite(tmp, strlen(tmp), 1, mime_fp) < 0) {

			fclose(mime_fp);
			unlink(mime_file_name);
			g_free(mime_file_name);
			g_free(tmp);
			return NULL;
		}
		fclose(mime_fp);
		tmp_len = strlen(tmp);

		mime_info = (MimeInfo *) procmime_scan_body(tmp, strlen(tmp));
		g_free(tmp);

		attach_list = mimeinfo_to_attachlist(mime_info, msg_info->header->msgid);

		msg_info->attach_list = attach_list;

		if ((procmime_is_smime(mime_info) == TRUE) &&
		    (msg_info->header->smime_type == SMIME_NONE)) {
			msg_info->header->smime_type = SMIME_INTERNAL;
		}
		g_free(mime_file_name);
	}
	return mime_info;
}
static gboolean
check_if_all_attachments_downloaded(const gchar *msgid)
{
  gchar *mime_file_name = NULL, *file_path = NULL, *enc_filename = NULL;
  MimeInfo  *partinfo = NULL, *mimeinfo = NULL;

  file_path = folder_get_message_file_path(msgid);
  mime_file_name = g_strdup_printf("%s%c%s%c%s%c%s",
                         get_mail_dir(), G_DIR_SEPARATOR, INBOX_DIR, G_DIR_SEPARATOR,
                         msgid, SECTION_SEPARATOR, MIME_SUFFIX);

  mimeinfo = procmime_read_mimeinfo(mime_file_name);
  g_free(mime_file_name);
  if (mimeinfo == NULL) {
          g_free(file_path);
          return FALSE;
  }
   partinfo = mimeinfo;
   while (partinfo != NULL) {
         if (partinfo->section_number != NULL) {
                        enc_filename = g_strdup_printf("%s%c%s",
                                                       file_path, SECTION_SEPARATOR,
                                                       partinfo->section_number);
                        if (partinfo->content_disposition != NULL) {
                                if (strncasecmp
                                    (partinfo->content_disposition, ATTACHMENT,
                                     strlen(ATTACHMENT)) == 0) {
					if (is_file_entry_exist(enc_filename) == FALSE){
                                                g_free(file_path);
                                                g_free(enc_filename);
                                                procmime_mimeinfo_free_all(mimeinfo);
                                                return FALSE;
                                        }
                                }

                        }
                        g_free(enc_filename);
        }
        if(partinfo->sub)
                partinfo =  partinfo->next;
        else
                partinfo = procmime_mimeinfo_next(partinfo);
   }
   g_free(file_path);
   procmime_mimeinfo_free_all(mimeinfo);
   return TRUE;
}


gint imap_fetch_all_attachments(const gchar * msgid, PrefsAccount * account, GSList * mimeinfo_list)
{
	Session *session = NULL;
	GSList *tmp = NULL;
	MimeInfo *mimeinfo = NULL;
	gint exists = 0, unseen = 0, recent = 0, uid_validity = 0;

	session = imap_session_new(account, msgid);

	if (session == NULL) {
		return IMAP_FAIL;
	} else {
		IMAP_SET_MSG_STR_CB(IMAP_CONNECT_SERVER_SUCCESS,
				    account->user_settings.account_name, msgid);
	}
	/* selects the inbox in read only mode & fetches the 
	 * inbox status. */

	if ((imap_cmd_do_uid_select(IMAP_SESSION(session), INBOX_DIR,
				    FALSE, &exists, &unseen, &recent,
				    &uid_validity, account, TRUE)) != IMAP_SUCCESS) {
		imap_session_complete(session);
	}
	for (tmp = mimeinfo_list; tmp != NULL; tmp = tmp->next) {

		mimeinfo = tmp->data;

		if (mimeinfo == NULL) {
			imap_session_complete(session);
			return IMAP_FAIL;
		}

		if (imap_fetch_msg_attachment(account, msgid,
					      mimeinfo->section_number,
					      FALSE, session, mimeinfo, FALSE) == IMAP_FAIL) {
                      if(!imap_is_session_cancelled())
			IMAP_SET_MSG_CB(IMAP_EMAIL_ATTACH_NOT_AVAILABLE_ERROR, msgid);
			imap_session_complete(session);
			return IMAP_FAIL;

		}
	}
	if(check_if_all_attachments_downloaded(msgid)){
                GSList *msgid_list = NULL;
                gchar *srcfolder = INBOX_DIR, *file_name = NULL;

                msgid_list = folder_get_msgid_all_files(srcfolder, msgid, NULL, FALSE);
                imap_cmd_fetch_msg_body(msgid, account, TRUE, FALSE, FALSE, session, FALSE);
                imap_send_repopulate();

                file_name = folder_get_message_file_path(msgid);
                /* if file is downloaded successfully,delete parts of the mail */
                if ((is_file_entry_exist(file_name) == TRUE)
                                    && (msgid_list != NULL)) {
                       folder_remove_msgs(srcfolder, msgid_list);
                }
                g_free(file_name);
                slist_free_strings(msgid_list);
                g_slist_free(msgid_list);
        }

	imap_session_complete(session);

	return IMAP_SUCCESS;
}
