
/**
 * This file is part of osso-email-engine
 *
 * Copyright (c) 2006 Nokia Corporation.
 * 
 * Contact: Dirk-Jan Binnema <dirk-jan.binnema@nokia.com>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef __AL_H__
#define __AL_H__

#include "callbacks.h"
#include "email_internal_interfaces.h"

#if CL_USE
typedef enum {
        FOLDER_OP = -1,
        VIEW_OP,
        MOVE_OP,
        REMOVE_OP
} op;
#endif



gboolean al_register_to_engine(SmtpCallback * smtp_callback,
			       Pop3Callback * pop3_callback,
			       Imap4Callback * imap4_callback,
			       AcctMgmtCallback * acct_callback,
			       MmcHlrCallback * mmc_hlr_callback, AlCallback * al_callback);


void al_send_outbox_mail(const gchar * account, const gchar * msgid, gboolean keepconnalive);
void al_send_mail(MsgInfo * msginfo, const gchar * message);
gint al_send_selected_mails(gint foldertype, GSList * msgid_list);
void al_send_draft_mails(const GSList * msgid_list);
void al_send_save_draft_mail(MsgInfo * msginfo,
			     const gchar * message, gboolean flag, gchar * old_id);
void al_resend_mails(const GSList * msgid_list);
void al_send_cancel_mails(const GSList * msgid_list);

gint al_get_account_list(GSList ** account_list);

gboolean al_get_account_settings(const gchar * account_name,
				 SettingType type, PrefsAccount ** prefs_account);

gboolean al_set_account_settings(SettingType type, PrefsAccount * prefs_account);

gboolean al_delete_account(const gchar * account_name);

gboolean al_rename_account(const gchar * old_name, const gchar * new_name);

void al_recv_mail(const gchar * account_name, gboolean scheduled_op);

MimeInfo *al_recv_msg_body(const gchar * msgid,
			   gboolean complete_download,
			   gchar * signer_email_id, gchar * recip_email_id,
			   SMimeType smime_type, gboolean read_flag,
			   gboolean edit_flag, gboolean get_html_flag,
			   guint cert_id, gchar * password);

MsgInfo *al_recv_msg_attachment(const gchar * msgid, MimeInfo * mimeinfo);


void al_cancel_recv_mail(const gchar * account_name);

gint al_folder_create(const gchar * name, const gchar * path, FolderType type);

gint al_folder_delete(const gchar * name);

gint al_folder_rename(const gchar * oldfolder, const gchar * newfolder);

gint al_folder_move_msg(const gchar * srcfolder, const gchar * destfolder, const gchar * msgid);

gint al_folder_move_msgs(const gchar * srcfolder, const gchar * destfolder, GSList * msglist);

gint al_folder_copy_msg(const gchar * srcfolder, const gchar * destfolder, const gchar * msgid);

gint al_folder_copy_msgs(const gchar * srcfolder, const gchar * destfolder, GSList * msglist);

gint al_folder_remove_msg(const gchar * folder, const gchar * msgid, gboolean flag);

gint al_folder_remove_msgs(const gchar * folder, GSList * msglist,
			   gboolean flag, gboolean required);

gint al_folder_remove_all_msg(const gchar * folder, gboolean flag);

gint al_folder_get_list(GSList ** folderlist);

void al_engine_cleanup(void);
void al_engine_cleanup_memory(void);
gint al_get_engine_errno(void);

GSList *al_populate_folder(const gchar * name, gint index);
GSList *al_folder_remove_attachment(GSList * folder_attach_info_list);
gboolean al_html_to_text_converter(const gchar * filename,
				   const gchar * charset, gchar ** convertedtext);

gchar *al_recv_multipart_body(MimeInfo * mimeinfo, const gchar * msgid, const gchar * default_font);
gint al_get_outbox_mail(void);
void al_update_mail_list(void);
GSList *al_get_msg_header(GSList * msgid_list);
gint al_autosave_mail(const gchar * msgid, gint operation);
gboolean al_editor_open_mail(const gchar * msgid, const gint option);
gint al_folder_delete_msgs(const gchar * foldername, const gchar * account_name, gboolean required);
gint al_get_folder_size(const gchar * folder_name);
AccntStatusFlags al_check_accnt_status(const gchar * account_name);
/**
  This function returns the list of prefs account 
  @param prefs accountlist    OUT parameter, list of prefs account 
  are returned as out parameter. 
  This list has to be freed by the calling function.
  @return AL_SUP_FAIL on failure,AL_SUP_SUCCESS on success
 */

gint al_prefs_account_get_list(GSList ** prefs_account_list);

/**
  This function returns the MsgInfo in call back
  @param msgid msgid of the mail
  @param mimeinfo mime information
  @param attach_tag attachment identification tag.
  @return none
 */
/*MsgInfo * */
void
al_recv_multipart_attachment(const gchar * msgid, MimeInfo * mimeinfo,
			     gint attach_tag, const gchar * default_font);

/**
  This function removes all the attachments of thge messages in the list 
  @param msgid_list    list of messages 
  @return    list of messages,for which deletion was not successful 
  This list has to be freed by the calling function.
 */
GSList *al_folder_remove_all_attachment(GSList * msgid_list);

void al_folder_cancel_mail(void);

gint al_set_temp_passwd(const gchar * account_name, const gchar * passwd, gboolean cancel);

void al_cleanup_sessions(void);
#if CL_USE
gboolean al_is_connection_required(const gchar * msgid, op op_flag);
gboolean al_is_attachment_downloaded(const gchar * msgid, const gchar * sec_num);
/**
  This function handles iap connection cancel 
  @returns none 
*/
void al_iap_cancel(void);
#endif				/*endif */

/**
  This function removes the attachments of a non IMAP message 
   or IMAP messages in folders other than Inbox
  @param msgid msgid of the message   
  @param rem_fpos_list  list of attachments of message,need to be removed.
  Attachments are identifed by file pointer of each attachment.List has to 
  be constructed using GINT_TO_POINTER  
  @return    list of messages,for which deletion was not successful 
  This list has to be freed by the calling function.
 */
gint al_remove_msg_attachments(const gchar * msgid, GSList * rem_fpos_list);

/**
  This function gets the mounted path of archival folder

  @returns returns path of archival folder
*/
gchar *al_folder_get_archive_path(void);

/**
  This function performs send and receive both on the 
 account 

  @param   account_name  name of the account
  @return none
*/
void al_send_recv_mail(const gchar * account_name, gboolean sched_op);

/**
  This function gets the message id required to compose a draft mail

  @param - Msginfo - Message information structure
  @param - send_flag is the flag representing saving/sending of mail.
*/
void al_send_get_msg_id(MsgInfo * msginfo, gboolean send_flag);

/**
  This function retreives the contents of messages for the 
  list of messages 

  @param msg_list   list of messages,for which the contents have 
                    to be retreived 
  @return    AL_SUP_SUCCESS on successfully retreiving contents 
  for all messages,AL_SUP_FAIL otherwise.
*/
gint al_retr_msgs(GSList * msg_list);

GSList *al_recv_multiple_attachments(const gchar * msgid, GSList * mimeinfo_list,
				     GSList * attach_tag_list);
void al_imap_set_seen(const gchar *msgid);
#endif
