
/**
 * This file is part of osso-email-engine
 *
 * Copyright (c) 2006 Nokia Corporation.
 * 
 * Contact: Dirk-Jan Binnema <dirk-jan.binnema@nokia.com>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef __AL_SUPPORT_H__
#define __AL_SUPPORT_H__

#define AL_SUP_SUCCESS 0
#define AL_SUP_FAIL -1
#define AL_SUP_NOT_AVAILABLE -2
#define CONTENTID "Content-ID:"
#define CONTENTIDLEN 11


#define GET_DECODE_FILENAME( msgid, mimeinfo,dec_filename)          \
{                                                                   \
     dec_filename = g_strdup_printf( "%s%c%s%c%s%c%s",      \
                         get_mime_tmp_dir(),             \
                         G_DIR_SEPARATOR, msgid,                    \
                         SECTION_SEPARATOR,                         \
                         mimeinfo->section_number,                  \
                         SECTION_SEPARATOR, DEC_SUFFIX );           \
}

#define GET_SUB_DECODE_FILENAME( msgid,sub_file_name)                     \
{                                                                         \
    sub_filename = g_strdup_printf ( "%s%c%s%c%s",                        \
                           get_mime_tmp_dir(),                            \
			   G_DIR_SEPARATOR, msgid,                        \
                           SECTION_SEPARATOR, SUB_SUFFIX );               \
}

#define IS_MHTML_MESSAGE(mimeinfo)                                           \
                                                                             \
( (g_ascii_strcasecmp ( mimeinfo->content_type,                              \
		   MULTIPART_RELATED)  == 0)                                 \
             || (g_ascii_strcasecmp ( mimeinfo->content_type,                \
                 MULTIPART_ALTERNATIVE ) == 0 ))                             \

#define IS_ATTACH_MESSAGE(mimeinfo)                                         \
                                                                            \
 (g_ascii_strncasecmp ( mimeinfo->content_type,                             \
		   MULTIPART_MIXED,MIN(strlen(MULTIPART_MIXED),             \
			  strlen(mimeinfo->content_type)))  == 0)           \

#define FOLDER_SET_MSG_CB(msg)                                            \
if(gl_al_cb_ptr != NULL){                                                 \
    if(gl_al_cb_ptr->al_folder_msg_cb != NULL)                            \
        gl_al_cb_ptr->al_folder_msg_cb(msg,NULL);                         \
}

#define FOLDER_SET_ERROR_CB(msg,param)                                    \
if(gl_al_cb_ptr != NULL){                                                 \
    if(gl_al_cb_ptr->al_folder_msg_cb != NULL)                            \
        gl_al_cb_ptr->al_folder_msg_cb(msg,"%d",param);                   \
}

#define FOLDER_PROGRESS_CB(cur,total)                                         \
if(gl_al_cb_ptr != NULL){                                                     \
    if(gl_al_cb_ptr->al_folder_msg_cb != NULL)                                \
        gl_al_cb_ptr->al_folder_msg_cb(FOLDER_PROGRESS,"%d %d",cur,total);    \
}

#define FOLDER_DEL_PROGRESS_CB(cur,total)                                       \
if(gl_al_cb_ptr != NULL){                                                       \
    if(gl_al_cb_ptr->al_folder_msg_cb != NULL)                                  \
        gl_al_cb_ptr->al_folder_msg_cb(FOLDER_DEL_PROGRESS,"%d %d",cur,total);  \
}

#define FOLDER_MSG_LIST_CB(type,list)                                      \
if((gl_al_cb_ptr != NULL) && (list != NULL)){                              \
    if(gl_al_cb_ptr->al_folder_list_cb != NULL)                            \
        gl_al_cb_ptr->al_folder_list_cb(type,list);                         \
}
#define NO_PASSWD ""

MsgInfo *al_sup_get_imap_selective_attachment(MimeInfo * mimeinfo,
					      const gchar * msgid, PrefsAccount * account);

//AttachInfo *al_sup_get_attachment_info ( MimeInfo * mimeinfo,
//                                         const gchar * msgid );

MsgInfo *al_sup_get_attachment_info(MimeInfo * mimeinfo, const gchar * msgid);

MimeInfo *al_sup_get_imap_body(PrefsAccount * account,
			       const gchar * msgid, gboolean edit_flag, gboolean get_html_flag);

MsgInfo *al_sup_rfc822_attach(MimeInfo * mimeinfo, const gchar * msgid);

gint al_sup_get_mhtml_file(MimeInfo * mimeinfo, FILE * infp,
			   FILE * outfp, gboolean root_flag, const gchar * default_font);

gint al_sup_get_mhtml_contents(MimeInfo * mimeinfo, FILE * fp, FILE * outfp,
			       gboolean root_flag, gboolean copy_hdrs, const gchar * default_font);

MimeInfo *al_sup_recv_body_imap(PrefsAccount * account,
				const gchar * msgid,
				gboolean complete_download,
				SMimeType smime_type, gchar * signer_email_id,
				gchar * recip_email_id, gboolean edit_flag, gboolean get_html_flag);

gchar *al_sup_extract_mhtml(const gchar * msgid, MimeInfo * mimeinfo,
			    const gchar * filename, const gchar * default_font);

gint al_sup_folder_recv_msg(PrefsAccount * account, const gchar * msgid,
			    const gchar * src_folder,
			    gboolean move_msg_flag, const gchar * file_path);

gint
al_sup_folder_rem_msg(PrefsAccount * prefs_account, GSList * cur_list,
		      GSList ** folder_succ_list, gint total);

gint al_sup_sync_imap(PrefsAccount * account, gint first, gint last);


/**
  This reads the email type of attachment and returns RFc822 headers. 
  This function downloads the email mhtml attachment for imap if required.
  @param mimeinfo    the mimeinformation of the part or attachment            
  @param msgid    message id of the message
  @param account    the account information pointer in the global account list,
  for which the attachment has to be downloaded.Global account list created and 
  deleted by account management functions. 
  @return Message information pointer,this has to be freed by the caller
 */
MsgInfo *al_sup_get_imap_mhtml_attachment(MimeInfo * mimeinfo,
					  const gchar * msgid,
					  PrefsAccount * account, const gchar * default_font);
/**
  This function will be called by AL layer to extract the mhtml 
  attachment.
  @param mimeinfo    the mimeinformation of the body 
  @param msgid    message id of the message
  @return pointer to the message information of the attachment,
  which has to be freed by caller 
 */
MsgInfo *al_sup_get_mhtml_attach(MimeInfo * mimeinfo, const gchar * msgid,
				 const gchar * default_font);

/**
 *    This fucntion will be called by AL to populate inline image information 
 *    in mimeinfo 
 *    @param mimeinfo mimeinformation of mutlipart body
 *    @param msgid message id
 *    @return list of inline image information(to be freed by calling function);
 */
GSList *al_support_get_inline_imageinfo_list(MimeInfo * mimeinfo, const gchar * msgid);

/**
 This function will be called to split the fully downloaded 
 message in to parts,to delete the attachments 
 @param msgid    message id
 @param filepath    path to the file to split 
 @param return AL_SUP_SUCCESS on SUCCESS or AL_SUP_FAIL 
*/
gint al_sup_split_to_parts(const gchar * filepath, const gchar * msgid);


//MimeInfo *al_sup_read_mimeinfo ( const gchar * mime_file_name );


/**
  This function performs folder copy/move operation on accountwise list 
  @param account_list    list of messages,sorted according to account name 
  @param account    account information of the messages  
  @param srcfolder    source folder for the operation  
  @param destfolder    destination folder for the operation  
  @param move_flag    indicates whether the operation is move 
  @param folder_append_list_ptr    OUT parameter,pointer to list of messages, 
                                on which operation is successfully performed 
  @param total    total message number 
  @param update_list_ptr    OUT parameter,pointer to list of source 
                                   messages, on which operation is successfull 
  @return AL_SUP_SUCCESS or AL_SUP_FAIL 
*/
gint al_sup_folder_list_op(GSList * account_list, const gchar * account,
			   const gchar * srcfolder, const gchar * destfolder,
			   gboolean move_flag, GSList ** append_list_ptr,
			   gint * cur_ptr, gint total,
			   GSList ** update_list_ptr, GSList ** del_list_ptr);

/**
  This function returns Mimeinfo of an S/MIME mail. It takes an
  S/MIME file as input, verifies/decrypts the message, forms mimeinfo 
  and returns the MimeInfo

  @param filename - Input filename containing S/MIME contents
  @param signer_email_id - Sender's email ID
  @param recip_email_id - Recipient's email ID
  @param msgid - Unique ID of the message
  @return MimeInfo - Structure containing mime information
*/
MimeInfo *al_sup_get_smime_message_info(const gchar * filename,
					const gchar * signer_email_id, const gchar * recip_email_id,
					const gchar * msgid);

/**
  This function prepares a list of messages accountwise, and performs 
  folder copy/move operation on it 
  @param sorted_list    list of messages,sorted according to account name 
  @param folder_add_list_ptr    OUT parameter,pointer to list of messages, 
                                on which operation is successfully performed 
  @param folder_update_list_ptr    OUT parameter,pointer to list of source 
                                   messages, on which operation is successfull 
  @param move_flag    indicates whether the operation is move 
  @param total    total message number 
  @param cur_ptr OUT parameter,pointer to the current message number  
  @param srcfolder    source folder for the operation  
  @param destfolder    destination folder for the operation  
  @return AL_SUP_SUCCESS or AL_SUP_FAIL 
*/
gint
al_sup_accountwise_op(GSList * sorted_list, GSList ** folder_add_list_ptr,
		      GSList ** folder_update_list_ptr, gboolean move_flag,
		      gint total, const gchar * srcfolder,
		      const gchar * destfolder, GSList ** folder_del_list_ptr);


/**
  This function deletes the specific message from the local directory  
  @param account    account name of the message 
  @param folder_delete_list    OUT parameter,pointer to list of messages, 
                               on which operation is successfully performed 
  @param folder    source folder for the operation  
  @param curdata    message id for the opration  
  @param ret_val    OUT parameter,pointer to the return value  
  @return TRUE if the message has only local copy 
*/
gboolean al_sup_folder_rem_local(const gchar * account, GSList ** folder_delete_list,
				 const gchar * folder, const gchar * curdata, gint * ret_val,
				 gboolean * pop_flag);

/**
  This function prepares a list of messages accountwise, and performs 
  folder delete messages operation on it 
  @param sorted_list    list of messages,sorted according to account name 
  @param folder_delete_list_ptr    OUT parameter,pointer to list of messages, 
                                on which operation is successfully performed 
  @param folder_update_list_ptr    OUT parameter,pointer to list of only local 
                                   messages, for which operation is successful  
  @param total    total message number 
  @param folder    source folder for the operation  
  @return FOLDER_SUCCESS or FOLDER_FAIL 
*/
gint
al_sup_rem_accountwise(GSList * sorted_list,
		       GSList ** folder_delete_list_ptr,
		       GSList ** folder_update_list_ptr,
		       gint total, const gchar * folder, gboolean flag);

/**
  This function prepares a list of messages attachments 
  @param mimeinfo    mime information of the message 
  @param first_partflag    indicates whether the part is first part,
  which is not supposed to be considered as attachment 
  @param attach_count    total number of attachments 
  @param msgid    msgid of the mesasge  
  @param section_flag    indicates the message is the imap message for which 
  the section number of the attachment has to be added to list  
  @return    list of attachments.List is single linked list of 
  FolderAttachInfo if the section_flag is TRUE.Otherwise it is list of 
  filepositions of the attachment,added using GINT_TO_POINTER 
*/
GSList *al_sup_get_all_attach_list(MimeInfo * mimeinfo,
				   gboolean first_partflag,
				   gint * attach_count, const gchar * msgid, gboolean section_flag);

/**
  This function removes attachment from a fullydownloaded message 
  @param infp    pointer to the message file information 
  @param outfp    pointer to the tmp file,in which contents after 
  removing are stored
  @param fpos    file position for the part to remove 
  @param mimeinfo    mimeinformation of the message  
  @param msgid    msgid of the mesasge  
  @return    list of attachments.List is single linked list of 
  FolderAttachInfo if the section_flag is TRUE.Otherwise it is list of 
  filepositions of the attachment,added using GINT_TO_POINTER 
*/
gint al_sup_rem_part(FILE * infp, FILE * outfp,
		     glong fpos, GSList * rem_fpos_list, MimeInfo * mimeinfo, const gchar * msgid);

/**
  This function retreives the contents of messages for the 
  list of messages.this list has messages of particular account only. 

  @param account_list   list of messages,for which the contents have 
                    to be retreived 
  @param account   account information of account,to which the 
                   messages belong 
  @return    AL_SUP_SUCCESS on successfully retreiving contents 
  for all messages,AL_SUP_FAIL otherwise.
*/
gint al_sup_retr_list(GSList * account_list, const gchar * account);

/**
  This function sets the seen flag of the message on server 

  @param msgid   message id of the message 
  @param prefs_account   account information 
  @return    none 
*/
void al_sup_imap_set_seen(const gchar * msgid, PrefsAccount * account);

/**
  Function to convert msgid to a RetrData structure
  
  @param - List containing message ID
  @return List containing RetrData structure
*/
GSList *al_sup_retr_msglist(GSList * list);
MimeInfo *al_sup_get_sec_mimeinfo(MimeInfo *mimeinfo, const gchar * sec_num);

#endif				// __AL_SUPPORT_H__
