
/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2006 Hiroyuki Yamamoto
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __COMPOSE_H__
#define __COMPOSE_H__

#include <glib.h>
#include "common.h"
#include "prefs_account.h"
#include "procmsg.h"
#include "procmime.h"

typedef struct _ComposeInfo ComposeInfo;

#define QUOTE_IF_REQUIRED(out, str)			\
{							\
   if (*str != '"' && strpbrk(str, ",.[]<>")) {         \
       gchar *__tmp;				        \
       gint len;				        \
  						        \
       len = strlen(str) + 3;			        \
       Xalloca(__tmp, len, return -1);                  \
       g_snprintf(__tmp, len, "\"%s\"", str);           \
       out = __tmp;				        \
   } else {					        \
       Xstrdup_a(out, str, return -1); 	                \
  	}					        \
}

#define PUT_RECIPIENT_HEADER(header, str)                          \
{			                                           \
   if (*str != '\0') {		                                   \
       Xstrdup_a(str, str, return -1); 			           \
       g_strstrip(str);					           \
       if (*str != '\0') {					   \
           composeinfo->to_list = address_list_append              \
                                      (composeinfo->to_list, str); \
           compose_convert_header(buf, sizeof(buf),                \
                                  str, strlen(header) + 2, TRUE);  \
           fprintf(fp, "%s: %s\n", header, buf);		   \
       }			                                   \
   }								   \
}

struct _ComposeInfo {
	gchar *boundary;
	GSList *to_list;
	gboolean type;		/* Draft = 1  */
};

gint compose_check_for_valid_recipient(const MsgHeader * msgheader);
gint compose_check_for_valid_account(const PrefsAccount * account);
gint compose_check_entries(const MsgInfo * msginfo, const PrefsAccount * prefs_account);
gint compose_write_to_file(const gchar * message,
			   GSList * attach_list,
			   const gchar * file,
			   const PrefsAccount * prefs_account,
			   MsgInfo * msginfo, ComposeInfo * composeinfo);
gint compose_write_inline_images(FILE * fp, MsgInfo * msginfo, ComposeInfo * composeinfo);
gint
compose_write_attach(FILE * fp, MsgInfo * msginfo, GSList * attach_list, ComposeInfo * composeinfo);

EncodingType compose_get_attach_info(const gchar * filepath,
				     gchar ** attachmentname, gchar ** content_type);
gint compose_write_headers(FILE * fp,
			   const gchar * charset,
			   EncodingType encoding,
			   const PrefsAccount * account,
			   MsgInfo * msginfo, ComposeInfo * composeinfo);
void compose_convert_header(gchar * dest, gint len,
			    gchar * src, gint header_len, gboolean addr_field);
gint compose_generate_msgid(MsgInfo * msginfo,
			    const PrefsAccount * account,
			    gchar * buf, const gint len, gboolean draft);
void compose_put_date(FILE * fp, MsgInfo * msginfo);
gint compose_move_msg(const gchar * src, const gchar * dest);
gint compose_check_for_valid_attachment(GSList * attach_list);
void compose_free_ComposeInfoStruct(ComposeInfo * composeinfo);
gint compose_resend_mail_to_file(const gchar * message,
				 GSList * attach_list,
				 const gchar * file,
				 const PrefsAccount * prefs_account,
				 MsgInfo * msginfo, ComposeInfo * composeinfo);

gint compose_resend_mail(const gchar * file,
			 const gchar * sent_filename,
			 const PrefsAccount * account, MsgInfo * msginfo);

void compose_write_multipart_header(FILE * fp,
				    const MsgInfo * msginfo,
				    GSList * attach_list,
				    ComposeInfo * composeinfo,
				    const gchar * out_codeset, EncodingType encoding);

gint compose_write_headers_receipant_list(FILE * fp,
					  const PrefsAccount * account,
					  MsgInfo * msginfo,
					  ComposeInfo * composeinfo, gchar ** aliasname);

gint compose_write_headers_subject(FILE * fp, MsgInfo * msginfo, gchar * buf);

void compose_write_headers_msgsettings(FILE * fp,
				       MsgInfo * msginfo,
				       const PrefsAccount * account, const gchar * aliasname);

/**
   this write mail contents in the file for html mails
   @param composeinfo compose information
   @param out_codeset out code set
   @param encoding encoding type
   @param buf mail body contents
   @param pointer to the file where contents to be written
   @param file name of the file to which contents to be written
   @param retval status
   @param outlen buffer len
   @param plain_text plain text buffer 
   @return TRUE/FALSE success/failure of writing contents to the file
 */

gboolean compose_write_html_mail(const gchar * out_codeset,
				 EncodingType encoding, const gchar * buf, FILE * fp,
				 const gchar * file, gint retval, gint outlen, gchar * plain_text);

/**
  this function encodes the string in to the base64 format
  @param buf string to be converted
  @return base64 encoded buffer 
*/
gchar *base64_encode_buf(const gchar * buf);


#endif				/* __COMPOSE_H__ */
