
/**
 * This file is part of osso-email-engine
 *
 * Copyright (c) 2006 Nokia Corporation.
 * 
 * Contact: Dirk-Jan Binnema <dirk-jan.binnema@nokia.com>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef ENGINE_DBUS_H_
#define ENGINE_DBUS_H_

#include "callbacks.h"
#include "procmime.h"
#include "dbus_utils_xml.h"
#define BACKUP_START 2
#define BACK_FINISH  0
void engine_process_init(void);
void engine_dbus_init(void);
gboolean engine_dbus_ui_is_available(void);
gint engine_dbus_al_register_engine(void);
void engine_dbus_al_unregister_engine(void);
gint engine_dbus_smtp_param_value_missing(const ParamMissing param_number);
gint engine_dbus_smtp_set_msg(const gint msg_number, const gchar * ipaddr, const gchar * acc_name);
gint engine_dbus_smtp_set_percentage(const gint percent);
gint engine_dbus_smtp_update_outbox_mails_sent(const gint totalMails, const gint sentMails);
gint engine_dbus_smtp_update_folder_view(MsgHeader * msgheader, gint operation, gboolean status);
gint engine_dbus_pop3_param_value_missing(ParamMissing param_number);
gint engine_dbus_pop3_set_msg(gint msg_number, const gchar * format, ...);
gint engine_dbus_pop3_set_percentage(gint percent);
gint engine_dbus_pop3_recv_single(MsgHeader * msgheader);
gint engine_dbus_pop3_recv_complete(void);
gint engine_dbus_pop3_recv_body_complete(MimeInfo * mimeinfo,
					 const gchar * msgid, gboolean remote_copy);
gint engine_dbus_imap4_param_value_missing(ParamMissing param_number);
gint engine_dbus_imap4_set_msg(gint msg_number, const gchar * format, ...);
gint engine_dbus_imap4_msg_headers(GSList * msginfo_list);
gint engine_dbus_engine_error_code(gint error_code);
gint engine_dbus_send_mail(const gchar * recv_string, gchar ** reply);
gint engine_dbus_send_outbox_mail(const gchar * recv_string, gchar ** reply);
gint engine_dbus_cancel_send_mail(const gchar * recv_string, gchar ** reply);
gint engine_dbus_send_save_draft_mail(const gchar * recv_string, gchar ** reply);
gint engine_dbus_recv_mail(const gchar * recv_string, gchar ** reply);
gint engine_dbus_recv_msg_body(const gchar * recv_string, gchar ** reply);
gint engine_dbus_recv_msg_attachment(const gchar * recv_string, gchar ** reply);
gint engine_dbus_recv_multipart_body(const gchar * recv_string, gchar ** reply);
gint engine_dbus_populate_folder(const gchar * recv_string, gchar ** reply);
gint engine_dbus_cancel_recv_mail(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_create(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_delete(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_rename(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_move_msg(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_move_msgs(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_remove_msg(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_remove_msgs(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_remove_all_msg(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_get_list(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_copy_msg(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_copy_msgs(const gchar * recv_string, gchar ** reply);
gint engine_dbus_folder_remove_attachment(const gchar * recv_string, gchar ** reply);
gint engine_dbus_get_account_list(const gchar * recv_string, gchar ** reply);
gint engine_dbus_get_account_settings(const gchar * recv_string, gchar ** reply);
gint engine_dbus_set_account_settings(const gchar * recv_string, gchar ** reply);
gint engine_dbus_delete_account(const gchar * recv_string, gchar ** reply);
gint engine_dbus_rename_account(const gchar * recv_string, gchar ** reply);
osso_pack_return_t engine_dbus_account_check_status(const gchar * recv_str, gchar ** reply);

gint engine_dbus_server_thread(void);
gint engine_dbus_connect_to_remote(gchar * msg_to_send);
gint engine_dbus_connect_to_remote_for_recv_body(const gchar * msg_to_send);
gint engine_dbus_get_engine_errno(const gchar * recv_string, gchar ** reply);
gint engine_dbus_get_engine_init_status(const gchar * recv_string, gchar ** reply);
gint engine_dbus_html_to_text_converter(const gchar * recv_string, gchar ** reply);
gint engine_dbus_ui_tasknav_window_operation(const gchar * recv_string, gchar ** reply);
gint engine_dbus_tasknav_window_id(const gchar * windowid);

osso_pack_return_t engine_dbus_tasknav_msg_operation(const gchar * msgid, gint option);
gint engine_dbus_open_contacts(gint type, const gchar * contacts);
gint engine_dbus_open_inbox(gint type);
gint engine_dbus_editor_create(MsgInfo * msginfo, gchar * message, gchar ** ret_str);
gint engine_dbus_autosave_mail(const gchar * recv_string, gchar ** reply);
gint engine_dbus_get_msg_header_list(const gchar * recv_string, gchar ** reply);
gint engine_dbus_statesave_signal(void);
osso_pack_return_t engine_dbus_send_app_killer_signal(const gint exit_signal);
gboolean send_recv_dbus_handler_busy(void);

/**
  This method is used to handle the backup start 
  signal,by cancelling current operations.  
  @param  none 
  @return on success returns OSSO_PACK_OK,else OSSO_PACK_ERROR  
*/
void engine_dbus_backup_handle(void);

/**
  This method is used to send the signal to UI when backup start 
  signal is received. 
  @param  none 
  @return on success returns OSSO_PACK_OK,else OSSO_PACK_ERROR  
*/
osso_pack_return_t engine_dbus_backup_event(void);
/**
 This function is used update the seen flag on serever for imag unread mail after its read*/
osso_pack_return_t engine_dbus_update_seen_flag(const gchar * recv_str, gchar ** reply);

#endif				/* ENGINE_DBUS_H */
