
/*
 * LibSylph -- E-Mail client library
 * Copyright (C) 1999-2006 Hiroyuki Yamamoto
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <glib.h>
#include <stdio.h>
#include <string.h>
#ifndef __IMAP_H__
#define __IMAP_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib.h>
#include <time.h>


#include "common.h"
#include "prefs_account.h"
#include "session.h"
#include "procmsg.h"
#include "callbacks.h"
#include "folder.h"

#define IMAP_SESSION(obj)	((IMAPSession *)obj)

typedef struct _IMAPSession IMAPSession;
struct _IMAPSession {
	Session session;

	gboolean authenticated;

	gchar **capability;
	gboolean uidplus;

	gchar *mbox;
	guint cmd_count;
	PrefsAccount *ac_prefs;
	gboolean state;
};

typedef struct _ProgData ProgData;
struct _ProgData {
	gint cur;
	gint total;
};

#define IMAP_MSG_UID_LEN 15
#define IMAP_SUCCESS	0
#define IMAP_FAIL -1
#define IMAP_ERROR_BASE 3000
#define IMAP_PARAMETER_MISSING (IMAP_ERROR_BASE +1)
#define IMAP_CONNECT_SERVER_ERROR (IMAP_ERROR_BASE +2)
#define IMAP_UID_FETCH_ERROR (IMAP_ERROR_BASE +3)
#define IMAP_INBOX_SELECT_ERROR (IMAP_ERROR_BASE +4)
#define IMAP_CLOSE_ERROR (IMAP_ERROR_BASE +5)
#define IMAP_FLAG_SET_ERROR (IMAP_ERROR_BASE +6)
#define IMAP_EXPUNGE_ERROR (IMAP_ERROR_BASE +7)
#define IMAP_MSG_FETCH_ERROR (IMAP_ERROR_BASE +8)
#define IMAP_SOCKET (IMAP_ERROR_BASE +9)
#define IMAP_ERROR (IMAP_ERROR_BASE +10)
#define IMAP_AUTHFAIL	(IMAP_ERROR_BASE +11)
#define IMAP_PROTOCOL	(IMAP_ERROR_BASE +12)
#define IMAP_SYNTAX	(IMAP_ERROR_BASE +13)
#define IMAP_IOERR	(IMAP_ERROR_BASE +14)
#define IMAP_INVALID_UID	(IMAP_ERROR_BASE +15)
#define IMAP_MSG_SIZE_EXCEED_ERROR	(IMAP_ERROR_BASE +16)
#define IMAP_INVALID_MSGID_ERROR	(IMAP_ERROR_BASE +17)
#define IMAP_MAIL_ALREADY_DOWNLOADED	(IMAP_ERROR_BASE +18)
#define IMAP_MESSAGE_NOT_AVAILABLE_IN_SERVER	(IMAP_ERROR_BASE +19)
#define IMAP_NO_HEADERS_DOWNLOADED	(IMAP_ERROR_BASE +20)
#define IMAP_SERVER_NOT_SUPPORT_ERROR (IMAP_ERROR_BASE +21)
#define IMAP_INVALID_RECENT_MAIL_SETTING (IMAP_ERROR_BASE +22)
#define IMAP_HEADERS_DOWNLOAD_ERROR (IMAP_ERROR_BASE +23)
#define IMAP_HEADER_FETCH_ERROR (IMAP_ERROR_BASE +24)
#define IMAP_NOT_ABLE_TO_ALLOCATE_MEMORY_ERROR (IMAP_ERROR_BASE +25)
#define IMAP_FOLDER_NOT_FOUND_ERROR (IMAP_ERROR_BASE +25)
#define IMAP_CONNECT_SERVER_SUCCESS (IMAP_ERROR_BASE +26)
#define IMAP_STARTTLS_ERROR (IMAP_ERROR_BASE +27)
#define IMAP_REPOP_ERROR (IMAP_ERROR_BASE +28)
#define IMAP_PROGRESS (IMAP_ERROR_BASE +29)
#define IMAP_COMPLETE (IMAP_ERROR_BASE +30)
#define IMAP_MSG_DOWNLOAD (IMAP_ERROR_BASE +31)
#define IMAP_SERVER_SYNC_ERROR (IMAP_ERROR_BASE +32)
#define IMAP_EMAIL_ATTACH_NOT_AVAILABLE_ERROR (IMAP_ERROR_BASE +33)
#define IMAP_START (IMAP_ERROR_BASE +34)
#define IMAP_MAIL_BOX_LOCK_LOST (IMAP_ERROR_BASE +35)
#define IMAP_PASSWD_FAIL (IMAP_ERROR_BASE +36)
#define IMAP_CONNECTION_FLIGHT_MODE (IMAP_ERROR_BASE +37)
#define IMAP_PASSWD_INCORRECT (IMAP_ERROR_BASE +38)




#define IMAPBUFSIZE	8192
#define IMAP_HDRCMD_SIZE 32
#define HEXDIGEST_SIZE   33
#define IMAP_BODY_PEEKMIME_CMD "UID FETCH %d BODY.PEEK[%s.MIME]"
#define ASTERISK '*'
#define IMAP_FETCH_CMD "FETCH"
#define IMAP_CURLY_START '{'
#define IMAP_CURLY_END '}'
#define IMAP_END_BR ')'
#define IMAP_BODY_PEEK_CMD "UID FETCH %d BODY.PEEK[]"
#define ONE_KB 1024
#define IMAP_SEQ_SIZE 22
#define IMAP_PEEK_HEADER_CMD "UID FETCH %d BODY.PEEK[header]"
#define IMAPS_PORT	993
#define IMAP4_PORT	143
#define IMAP_CMD_SIZE    32

#define HEADER_NUM 10

#define IMAP_CMD_LIMIT	1000


#define RETURN_ERROR_IF_FAIL(cond)	\
   if (!(cond)) {			\
	g_free(buf);		        \
	return IMAP_ERROR;	        \
   }

typedef enum {
	IMAP_FLAG_SEEN = 1 << 0,
	IMAP_FLAG_ANSWERED = 1 << 1,
	IMAP_FLAG_FLAGGED = 1 << 2,
	IMAP_FLAG_DELETED = 1 << 3,
	IMAP_FLAG_DRAFT = 1 << 4
} IMAPFlags;

#define IMAP_IS_SEEN(flags)	((flags & IMAP_FLAG_SEEN) != 0)
#define IMAP_IS_ANSWERED(flags)	((flags & IMAP_FLAG_ANSWERED) != 0)
#define IMAP_IS_FLAGGED(flags)	((flags & IMAP_FLAG_FLAGGED) != 0)
#define IMAP_IS_DELETED(flags)	((flags & IMAP_FLAG_DELETED) != 0)
#define IMAP_IS_DRAFT(flags)	((flags & IMAP_FLAG_DRAFT) != 0)
#define IMAP_FIRST_UID 1
#define IMAP_ALL_UID   0
#define IMAP_NO_UID   -1
#define GET_IMAP_MHTML_FILENAME( msgid,mimeinfo,sub_file_name)            \
{                                                                         \
    sub_file_name = g_strdup_printf ( "%s%c%s%c%s%c%s",                   \
                        get_mail_dir(),                                   \
			G_DIR_SEPARATOR,                                  \
                        INBOX_DIR, G_DIR_SEPARATOR,                       \
                        msgid, SECTION_SEPARATOR,                         \
                        mimeinfo->section_number );                       \
}
#define IMAP_BODYSTR_CMD "uid FETCH %d bodystructure"

gboolean imap4_init(Imap4Callback * cb_ptr);

void imap_scan_inbox(PrefsAccount * prefs_account, InboxStatus * inbox_status);

void
imap_expunge_msg(PrefsAccount * prefs_account, GSList * msglist,
		 GSList ** folder_succ_list, ListType type,
		 gint * cur_ptr, gint total, FolderProgressCb progress_cb);

void imap_get_headers(PrefsAccount * prefs_account,
		      const gchar * folder, gint first_msgid, gint last_msgid);

void imap_update_msg_status(PrefsAccount * prefs_account, GSList * msg_status_list);

void imap_cmd_fetch_msg_body(const gchar * msgid, PrefsAccount * prefs_account,
			     gboolean read_flag, gboolean size_check,
			     gboolean smime_check, Session * in_session, gboolean close_session);

gint imap_fetch_msg_attachment(PrefsAccount * prefs_account, const gchar * msgid,
			       const gchar * section_number, gboolean size_limit_check,
			       Session * session, MimeInfo * mimeinfo, gboolean close_session);

MimeInfo *imap_fetch_msg_body_structure(PrefsAccount * prefs_account,
					const gchar * msgid, Session ** session);

gint imap_get_message_status(PrefsAccount * prefs_account, MsgStatus ** status);

gint imap_fetch_mime_header(PrefsAccount * prefs_account,
			    const gchar * msgid, gchar * section_number, gchar * file_name);

void imap_cleanup(void);

void imap_send_repopulate(void);

void imap_recv_cancel(void);

/**
  This function fetches the mhtml file contents and writes to the file,
  pointed by the file pointer  
  @param mimeinfo    pointer to the mime information
  @param fp    file pointer of the out file
  @param msgid    message id
  @param prefs_account    pointer to PrefsAccount
  @return success(IMAP_SUCCESS) or error code
*/
gint imap_get_mhtml_body(MimeInfo * mimeinfo, FILE * fp,
			 const gchar * msgid, PrefsAccount * prefs_account);

/**
  This function performs folder copy/move operation on imap account 
  @param srcfolder    source folder for the operation  
  @param destfolder    destination folder for the operation  
  @param prefs_account    account information of the messages  
  @param folder_msg_list    list of messages,sorted according to account name 
  @param move_flag    indicates whether the operation is move 
  @param folder_append_list    list of messages, on which operation is 
                               successfully performed.This is used for 
			       appending successful operations. 
  @param cur_ptr OUT parameter,pointer to the current message number  
  @param total    total message number 
  @param progress_cb    pointer to the function,which will be 
  called to send progress. 
  @param update_list_ptr    OUT parameter,pointer to list of source 
                            messages, on which operation is successfull 
  @param folder_del_list_ptr    OUT parameter,pointer to list of 
                            messages,to be deleted on move operation 
  @return list of messages on which operation is successful 
*/
GSList *imap_move_copy_msgs(const gchar * srcfolder,
			    const gchar * destfolder, PrefsAccount * prefs_account,
			    GSList * folder_msg_list,
			    gboolean move_flag, GSList * folder_append_list,
			    gint * imap_ret, gint * cur_ptr, gint total,
			    FolderProgressCb progress_cb,
			    GSList ** update_list_ptr, GSList ** folder_del_list_ptr);

void imap_session_disconnect(Session * session, gboolean flag);

/**
  This function checks if there is any account in use and returns it 

  @param none 
  @return pointer to account information,NULL if there is no account in use 
*/
PrefsAccount *imap_get_cur_account(void);

/**
  This function checks if only header file is present it deletes it
  for IMAP header file

  @param msgid - Message ID
  @return TRUE if file is deleted else FALSE
*/
gboolean imap_delete_header_file(const gchar * msgid);

/**
  This function retreives contents of the message,
  based upon retrieval type. 

  @param msgid    Message ID of message,whose contents 
                  have to be retreived.
  @param account   account information of the account 
  @param complete_download   flag indicates whether the 
  message has to be completely downloaded irrespective 
  of the retreival type. 
  @param session_ptr   pointer to the session information  
  @param cur   pointer to the number of message  
  @param total   total number of messages  
  @return  IMAP_SUCCESS on successful download,else IMAP_FAIL 
*/
gint imap_retr_imap_msg(const gchar * msgid, PrefsAccount * account,
			gboolean complete_download, Session ** session_ptr, gint * cur, gint total);

/**
  This function sends logout,complete callback to UI and then 
  destroys the session 

  @param session    pointer to the session information,
                   which has to be destroyed 
  @return none 
*/
void imap_session_complete(Session * session);

/**
  This function checks if the IMAP session was cancelled 

  @param    none 
  @return TRUE if the session was cancelled,else FALSE 
*/
gboolean imap_is_session_cancelled(void);

gint imap_fetch_all_attachments(const gchar * msgid, PrefsAccount * account,
				GSList * mimeinfo_list);
#endif				/* __IMAP_H__ */
