
/*
 * LibSylph -- E-Mail client library
 * Copyright (C) 1999-2006 Hiroyuki Yamamoto
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <glib.h>
#include <stdio.h>
#include <string.h>
#ifndef __POP_H__
#define __POP_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib.h>
#include <time.h>

#include "session.h"
#include "prefs_account.h"

typedef struct _Pop3MsgInfo Pop3MsgInfo;
typedef struct _Pop3Session Pop3Session;

#define POP3_SESSION(obj)	((Pop3Session *)obj)

typedef enum {
	POP3_READY,
	POP3_GREETING,
	POP3_GETAUTH_USER,
	POP3_GETAUTH_PASS,
	POP3_GETAUTH_APOP,
	POP3_GETRANGE_STAT,
	POP3_GETSIZE_LIST,
	POP3_GETSIZE_LIST_RECV,
	POP3_RETR,
	POP3_RETR_RECV,
	POP3_DELETE,
	POP3_LOGOUT,
	POP3_ERROR,
	POP3_TOP,
	POP3_NOP,
	POP3_TOP_RECV,
	POP3_GETAUTH_CRAM_MD5,
	POP3_GETAUTH_CRAM_MD5_RECV,
	POP3_GETRANGE_UIDL,
	POP3_GETRANGE_UIDL_RECV,
	POP3_STLS
} Pop3State;

typedef enum {
	PS_SUCCESS = 0,		/* command successful */
	PS_SOCKET = 2,		/* socket I/O woes */
	PS_AUTHFAIL = 3,	/* user authorization failed */
	PS_PROTOCOL = 4,	/* protocol violation */
	PS_IOERR = 6,		/* file I/O error */
	PS_ERROR = 7,		/* protocol error */
	PS_LOCKBUSY = 8,	/* server responded lock busy */
	PS_NOTSUPPORTED = 9,	/* command not supported */
	PS_DELERR = 10,		/* message is already deleted from server */
	PS_PASSWDFAIL = 11,	/* passwd is invalid */
	PS_PASSWD_CANCEL = 12,	/* passwd is invalid & cancelled */
	PS_SIZELMTERR
} Pop3ErrorValue;

typedef enum {
	RECV_TIME_KEEP = 2
} RecvTime;


struct _Pop3MsgInfo {
	guint size;
	time_t recv_time;
	guint received:1;
	guint deleted:1;
	gchar *rx_msgid;
	gchar *uidl;
	gint recv_num;
	gboolean skip_download;
};

struct _Pop3Session {
	Session session;
	Pop3State state;
	PrefsAccount *ac_prefs;
	gchar *greeting;
	gint count;
	gint prog_count;
	gint total_bytes;
	gint cur_msg;
	gint prog_cur_msg;
	gint cur_total_num;
	gint cur_total_bytes;
	gint cur_total_recv_bytes;
	Pop3MsgInfo *msg;
	gboolean new_msg_exist;
	time_t current_time;
	Pop3ErrorValue error_val;
	gchar *error_msg;
	gpointer data;
	gchar *msgid;
	gint get_msg_num;
	gint delete_msg_num;
	FolderOpernType operation;
	gchar *passwd;
	gint size_error;
	gchar *uidl;
	gboolean uidl_is_valid;
	GHashTable *uidl_table;
	GSList *retr_list;
	gint retr_count;
	gboolean delete_all;
	gboolean queue_flag;
};

#define POPBUFSIZE 512
#define POP3_PORT	110
#define POP3_FAIL -1
#define POP3_SUCCESS 0
#define POP3_TOP_CMD_FORMAT "TOP %d 0"
#define POP3_CRAM_MD5_AUTH "CRAM-MD5"
#define POP3_AUTH_CMD_FORMAT "AUTH %s"
#define RECV_MSG_NONE 0
#define IDLEN 128

extern gboolean gl_callback_enable_flag;	//defined in utils 


#define POP3_HAS_CALLBACK \
 ( (gl_pop3_callback_ptr != NULL ) && (CHECK_CB_FLAG ==TRUE) )

#define POP3_SET_MSG_CB(error_code)                                     \
{                                                                       \
if ( POP3_HAS_CALLBACK ) {                                              \
   if ( gl_pop3_callback_ptr->pop3_set_msg_cb != NULL ) {               \
           gl_pop3_callback_ptr->pop3_set_msg_cb ( error_code,NULL );   \
   }                                                                    \
}                                                                       \
}
#define POP3_CB_PARAM_MISSING(error_code)                                   \
{                                                                           \
if ( POP3_HAS_CALLBACK ) {                                                  \
   if ( gl_pop3_callback_ptr->pop3_param_value_missing_cb != NULL ) {       \
       gl_pop3_callback_ptr->pop3_param_value_missing_cb ( error_code);     \
   }                                                                        \
}                                                                           \
}
#define POP3_CB_COMPLETE()                                                \
{                                                                         \
if ( POP3_HAS_CALLBACK ) {                                                \
    if ( gl_pop3_callback_ptr->pop3_recv_complete_cb != NULL ) {          \
        gl_pop3_callback_ptr->pop3_recv_complete_cb ();                   \
    }                                                                     \
}                                                                         \
}

#define POP3_SET_MSG_SRV_CB(error_code,account)                        \
{                                                                       \
if ( POP3_HAS_CALLBACK ) {                                              \
   if ( gl_pop3_callback_ptr->pop3_set_msg_cb != NULL ) {               \
           gl_pop3_callback_ptr->pop3_set_msg_cb (                      \
           error_code,"%s",account->user_settings.account_name );     \
   }                                                                    \
}                                                                       \
}

#define POP3_SET_MSG_PROG_CB(error_code,account)                        \
{                                                                       \
   if ( gl_pop3_callback_ptr->pop3_set_msg_cb != NULL ) {               \
           gl_pop3_callback_ptr->pop3_set_msg_cb (                      \
           error_code,"%s",account->user_settings.account_name );     \
   }                                                                    \
}

#define POP3_SET_MSGBODY_SRV_CB(error_code,account,msgid)               \
{                                                                       \
if ( POP3_HAS_CALLBACK ) {                                              \
   if ( gl_pop3_callback_ptr->pop3_set_msg_cb != NULL ) {               \
           gl_pop3_callback_ptr->pop3_set_msg_cb (                      \
           error_code,"%s %s",account->user_settings.account_name,msgid ); \
   }                                                                    \
}                                                                       \
}

#define POP3_DEL_LIST_CB                                                      \
if ( POP3_HAS_CALLBACK ) {                                                    \
        if ( gl_pop3_callback_ptr->pop3_delete_list_cb != NULL ) {            \
            gl_pop3_callback_ptr->pop3_delete_list_cb (                       \
			    LIST_FOLDER_DEL,gl_pop3_delete_list );            \
        }                                                                     \
}
#define POP3_MSG_DOWNLOAD_CB(msgid,remote_copy)                             \
if ( POP3_HAS_CALLBACK ) {                                                  \
    if ( gl_pop3_callback_ptr->pop3_set_msg_cb != NULL ) {                  \
        gl_pop3_callback_ptr->                                              \
            pop3_set_msg_cb ( POP3_MSG_DOWNLOAD, "%s %d",                   \
                                        msgid,remote_copy );                \
    }                                                                       \
}
#define POP3_MSG_DELETE_ERROR_CB(msgid)                                     \
if ( POP3_HAS_CALLBACK ) {                                                  \
    if ( gl_pop3_callback_ptr->pop3_set_msg_cb != NULL ) {                  \
        gl_pop3_callback_ptr->                                              \
            pop3_set_msg_cb ( POP3_MSG_DELETE_ERROR, "%s",                  \
                                        msgid );                            \
    }                                                                       \
}

Session *pop3_session_new(PrefsAccount * account, const gchar * msgid);
gint pop3_write_msg_to_file(FILE * fp, const gchar * data, guint len);


/**
 This function stores the uidl table to the file specific to 
 the account   
 @param session pointer to the pop3 session 
 @return failure or success 
*/
gint pop3_write_uidl_list(Pop3Session * session);

/**
 This function checks whether the the specific entry is the hash 
 table,is removed in the server by checking against server list.   
 @param key    key to uidl table,the uid string 
 @param value    the value of the uidl table,the message number 
 @param user_data    the pop3 session pointer. 
 @return TRUE if the entry is to be removed. 
*/
gboolean pop3_hash_entry_removed(gpointer key, gpointer value, gpointer user_data);

/**
 This function synchronises the local headers to headers on server. 
 @param session    pointer to the pop3 session 
 @return none 
*/
void pop3_update_headers(Pop3Session * pop3_session);

gint pop3_logout_send(Pop3Session * session);

#if USE_SSL

#define POP3_SSL_PORT 995
extern gint ssl_error;

#endif
#endif				/* __POP_H__ */
