
/**
 * This file is part of osso-email-engine
 *
 * Copyright (c) 2006 Nokia Corporation.
 * 
 * Contact: Dirk-Jan Binnema <dirk-jan.binnema@nokia.com>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef __POP_RECV_H__
#define __POP_RECV_H__

#include <glib.h>
#include <time.h>

#include "common.h"
#include "prefs_account.h"
#include "procmsg.h"
#include "session.h"
#include "callbacks.h"
#include "pop.h"


typedef struct _Pop3RecvSession Pop3RecvSession;
typedef struct _Pop3MsgData Pop3MsgData;

typedef enum {
	POP3_RECV_SUCCESS,
	POP3_RECV_CONNECT_ERROR,
	POP3_RECV_AUTH_FAILED,
	POP3_RECV_LOCKED,
	POP3_RECV_ERROR,
	POP3_RECV_IO_ERROR,
	POP3_RECV_SOCKET_ERROR,
	POP3_RECV_EOF,
	POP3_RECV_CANCEL,
	POP3_RECV_NOSUPPORT,
	POP3_RECV_SSL_ERROR,
	POP3_RECV_DEL,
	POP3_RECV_PASSWD_CANCEL
} Pop3RecvState;


struct _Pop3RecvSession {
	Session *session;
	Pop3RecvState pop3_recv_state;
};

struct _Pop3MsgData {
	gint msg_num;
	gchar *srvr_msgid;
};

gint pop3_get_msg_header(Pop3Session * session, const gchar * file, gint msg_num, gchar * msgid);

void pop3_recv_cancel(void);

void pop3_recv_mail(PrefsAccount * account, const gchar * msgid,
		    FolderOpernType operation, gboolean queue_flag, GSList * retr_list);

gboolean pop3_init(Pop3Callback * cb_ptr);

gboolean pop3_init(Pop3Callback * cb_ptr);

gchar *pop3_get_new_msg_file(Pop3Session * pop3_session, const gchar * account_name,
			     const gchar * destfolder, gint * msgnum_ptr);

gboolean pop3_recv_check_state(void);

gint pop3_get_header_info(Pop3Session * session, const gchar * data, guint len, gint * msgnum_ptr);

void pop3_delete_msgnum(const gchar * msgid);

void pop3_cleanup(void);

gchar *return_file_if_smime(const gchar * msgid);

/**
  This function deletes the .hdr file once body is downloaded

  @param file - Path of body
  @return - TRUE on successful removal and FALSE if removal fails
*/
gboolean pop3_remove_header_file(const gchar * file);

void pop3_free_old_list(void);

/**
  Sends complete callback to UI even when callbacks are blocked

  @account - Account information
  @return void
*/
void pop3_send_complete_cb(PrefsAccount * account);

/** Functions iterates through list and adds msgids into
    delete list for queued deletion
    
    @param delete_list - List of message to be added for queued deletion
    @check_remotelist in case of TRUE flag message ids added to deleted list 
    @return - None
*/
void pop3_write_retr_del_list(GSList * delete_list, gboolean check_remotelist);

#endif				/* __POP_RECV_H__ */
