
/*
 * LibSylph -- E-Mail client library
 * Copyright (C) 1999-2006 Hiroyuki Yamamoto
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <glib.h>
#include <stdio.h>
#include <string.h>
#ifndef __SMTP_H__
#define __SMTP_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib.h>
#include "session.h"
#include "prefs_account.h"
#include "callbacks.h"

typedef struct _SMTPSession SMTPSession;

#define SMTP_PORT       25
#define SSMTP_PORT      465

#define SMTP_SESSION(obj)	((SMTPSession *)obj)

#define MSGBUFSIZE	8192
#define SMTP_PORT	25

#define CHECK_ENTRIES \
{ \
    if ( ( retval = \
           compose_check_entries ( msginfo, prefs_account ) ) != SM_OK ) { \
        debug_print ( " Errorcode(%d) : Invalid inputs to send mail \n", \
                      retval );                                          \
        SMTP_PARAM_VALUE_MISSING_CB ( retval );                          \
        return retval;                                                   \
    }                                                                    \
}

typedef enum {
	SM_OK = 0,
	SM_ERROR = 128,
	SM_UNRECOVERABLE = 129,
	SM_AUTHFAIL = 130,
	SM_CANCEL = 131
} SMTPErrorValue;

typedef enum {
	SMTP_READY,
	SMTP_CONNECTED,
	SMTP_HELO,
	SMTP_EHLO,
	SMTP_STARTTLS,
	SMTP_FROM,
	SMTP_AUTH,
	SMTP_AUTH_LOGIN_USER,
	SMTP_AUTH_LOGIN_PASS,
	SMTP_AUTH_CRAM_MD5,
	SMTP_RCPT,
	SMTP_DATA,
	SMTP_SEND_DATA,
	SMTP_EOM,
	SMTP_RSET,
	SMTP_QUIT,
	SMTP_ERROR,
	SMTP_CANCEL,
	SMTP_DISCONNECTED,
	SMTP_KEEPCONNALIVE,
	N_SMTP_PHASE
} SMTPState;

typedef enum {
	OUTBOX_WAITING = 1,
	OUTBOX_SENDING,
	OUTBOX_FAILED,
	OUTBOX_ON_REQUEST,
	OUTBOX_SUSPENDED
} SMTPOutboxStatus;

struct _SMTPSession {
	Session session;

	SMTPState state;
#if USE_SSL
	gboolean tls_init_done;
#endif
	gchar *hostname;

	gchar *user;
	gchar *pass;
	gushort port;
	gboolean  use_ssl;

	gchar *from;
	GSList *to_list;
	GSList *cur_to;

	guchar *send_data;
	FILE *send_data_fp;
	guint send_data_len;

	SMTPAuthType avail_auth_type;
	SMTPAuthType forced_auth_type;
	SMTPAuthType auth_type;

	SMTPErrorValue error_val;
	gchar *error_msg;
	gchar *account;
	gchar *iap_name;
};
gboolean smtp_init(SmtpCallback * cb);
void smtp_cleanup(void);
Session *smtp_session_new(void);
gint smtp_quit(SMTPSession * session);
gint smtp_use_connectionalive(Session * session);

#endif				/* __SMTP_H__ */
